//-----------------------------------------------------------------------------
// <copyright file="IfxMetricsInternal.h" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//
// The IfxMetrics internal header that shares information between the library
// and the unittests.
//-----------------------------------------------------------------------------

#pragma once

// The value of individual enum should always be smaller than 256 so that it could fit as the event type in ETW event
// header.
// Currently there are 2 types of MDM events:
// - Increment count metric
// - Set measure metric
typedef enum _MDM_EVENT_TYPE
{
    CountMetricIncrement = 10,
    MeasureMetricSet = 11
} MDM_EVENT_TYPE;

// The implicit layout of the MDM event payload.
// The format of the MDM event data blob includes the fixed-size part and variable-size part (the string table).
//
// The layout of the fixed-size part should only be extended in later version in order to ensure that the consumer
// could always process the MDM event blobs generated by different MDM provider versions.
//
//   Int16: number of dimensions.
//   <padding for 8-byte align> 
//
//   Int64: timestamp - this is the timestamp passed explicitly through MDM APIs or 0.
//   Int64: metric raw data 
//
// Start the variable-length part 
//
//    String: metric MonitorAccount name (in Azure, this could be the tenant name). 
//    String: metric namespace 
//    String: metric name 
//
//    String: dimension name 1 
//    String: dimension name 2 
//    ...
//
//    String: dimension value 1 
//    String: dimension value 2 
//    ...
//
// The blob format of the string is Int16 for the length (after 8-byte aligned) of the string followed by the
// null-terminated UTF8 string. The total size of the String (the length plus the string buffer) should always
// be 8-byte aligned.
//
// The maximum size of the MDM event payload is the sum of the following, which is 4560 bytes.
//     sizeof(MDM_EVENT_PAYLOAD), currently this is 24 bytes.
//     64 + 8          - (MAX_NAME_SIZE for monitoringAccount string + the 8-byte-aligned(sizeof(USHORT))
//     1024 + 8        - (MAX_NAMESPACE_SIZE for the metricNamespace string + the 8-byte-aligned(sizeof(USHORT))
//     64 + 8          - (MAX_NAME_SIZE for the metricName string + the 8-byte-aligned(sizeof(USHORT))
//     10 * (64 + 8)   - 10 dimension name strings (MAX_DIMENSION_NAME_SIZE for dimension name strings)
//     10 * (256 + 8)  - 10 dimension value strings (MAX_DIMENSION_VALUE_SIZE for dimension value strings)

// The fixed-size part of the MDM event payload
typedef struct _MDM_EVENT_PAYLOAD
{
    USHORT countDimension;
    WORD   reservedWord;   // for 8-byte aligned
    DWORD  reservedDword;  // for 8-byte aligned

    ULONG64 timestampUtc;
    LONG64  rawData;
} MDM_EVENT_PAYLOAD, *PMDM_EVENT_PAYLOAD;
