/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    VideoPacket.h

Abstract:
    This class defines a c-compatible container for video frames.  
    Using this container, frames are allocated such that unnecessary calls 
    to alloc/free are avoided, by keeping frames cached in a "free" list for
    future re-use

Author:

Revision History:

--*/

#pragma once


typedef struct RtcPalVideoPacketPool     * HVIDEOPACKETPOOL;
typedef struct RtcPalVideoEncodedFrameEx * HVIDEOPACKET;

#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------
// Functions below are generic, work for both raw
// and encoded frames

RTC_CLASS_LINKAGE
HVIDEOPACKET
RtcPalAllocateVideoPacketAndData(
    HVIDEOPACKETPOOL hPacketPool,
    uint32_t DataSize
    );

RTC_CLASS_LINKAGE
VOID
RtcPalFreeVideoPacketPool(
    HVIDEOPACKETPOOL hPacketPool
    );


RTC_CLASS_LINKAGE
HVIDEOPACKET
RtcPalGetVideoPacketFromSlistEntry(
    PSLIST_ENTRY pSListEntry
    );

RTC_CLASS_LINKAGE
VOID
RtcPalFreeVideoPacket(
    HVIDEOPACKET pFrameEx
    );


RTC_CLASS_LINKAGE
VOID
RtcPalDetachVideoPacket(
    HVIDEOPACKET hPacket
    );

RTC_CLASS_LINKAGE
VOID
RtcPalPrependVideoPacketToSlist(
    PSLIST_ENTRY pListHead,
    HVIDEOPACKET hPacket
    );
RTC_CLASS_LINKAGE
VOID
RtcPalAppendVideoPacketToSlist(
    PSLIST_ENTRY pListHead,
    HVIDEOPACKET hPacket
    );




// ----------------------------------------------------------
// ---- Functions below are for handling encoded packets----
RTC_CLASS_LINKAGE
HVIDEOPACKETPOOL
RtcPalAllocateVideoPacketPool(
    void
    );


RTC_CLASS_LINKAGE
HVIDEOPACKET
RtcPalAllocateVideoPacketFromBorrowedUnit(
    HVIDEOPACKETPOOL hPacketPool,
    RtcPalVideoEncodedUnit_t *pBorrowedUnit
    );


RTC_CLASS_LINKAGE
RtcPalVideoEncodedUnit_t *
RtcPalGetVideoPacketUnit(
    HVIDEOPACKET
    );



// ----------------------------------------------------------
// ---- Functions below are for handling raw video frames----
RTC_CLASS_LINKAGE
HVIDEOPACKETPOOL
RtcPalAllocateRawFramePool(
    void
    );

RTC_CLASS_LINKAGE
HVIDEOPACKET
RtcPalCopyRawVideoFrame(
    HVIDEOPACKETPOOL hPacketPool,
    const RtcPalVideoRawFrame_t *pSourceFrame
    );


RTC_CLASS_LINKAGE
RtcPalVideoRawFrame_t *
RtcPalGetVideoPacketRawFrame(
    HVIDEOPACKET
    );




#ifdef __cplusplus
}
#endif

