#ifndef __RTCPAL_ARM_H__
#define __RTCPAL_ARM_H__

#ifdef _MSC_VER
// 4127 - conditional expression is constant
#pragma warning(disable: 4127)
#define UNALIGNED __unaligned
#else
#define UNALIGNED
#endif

#ifdef __cplusplus
#ifndef _MSC_VER
//
// on arm, if the size of the struct is equal or more than 32 bytes, LLVM/GCC will use ldm/stm 
// instruction to optimize the struct assignment. So:
//  
//      strcutObjA = strcutObjB
//
// require the address of both strcutObjA and strcutObjB are aligned. Otherwise, it will cause
// bus error. (bug#2973403).
//
// here, we overload the assignment operator and use memcpy instead.
//
extern "C" int __cdecl memcpy_s(void * dst, size_t sizeInBytes, const void * src, size_t count);

template<typename T>
struct _UnAlignStruct
{
    T& operator=(const _UnAlignStruct& other)
    {
        if ((_UnAlignStruct*)&other != this)
        {
            if ((unsigned int)this & 0x3 || (unsigned int)&other & 0x3)
            {
                // TODO: here, we can't use memcpy, compiler will optimize memcpy, 
                // if the size parameter is 64, 32 etc, it will be optimized with 
                // stm/ldm also, so we use memcpy_s. But later, we should consider fix
                // memcpy issue, because no one can remember use memcpy_s not memcpy
                // everywhere.
                memcpy_s(this, sizeof(T), &other, sizeof(T));
            }
            else
            {
                memcpy(this, &other, sizeof(T));
            }
        }
        return *(T*)this;
    }
};

#define unalign_struct(T)   struct T : public _UnAlignStruct<T>

#else
#define unalign_struct(T)   struct T 
#endif

extern "C" {
#endif

// -----------------------------------------------------------------------------
// CPU Detect functions and stubs
// -----------------------------------------------------------------------------
// General functions
unsigned int RTCAPI RtcPalGetNumberofLogicalProcessorsPerPackage();
unsigned int RTCAPI RtcPalGetNumberofPackages();
unsigned int RTCAPI RtcPalGetNumberofProcessors();
unsigned int RTCAPI RtcPalGetNumberOfCores();
unsigned int RTCAPI RtcPalGetCPUClockMHz();

// X86 functions
#define RtcPalCheckCMov() (0)
#define RtcPalCheckMMX() (0)
#define RtcPalCheckSSE() (0)
#define RtcPalCheckSSE2() (0)
#define RtcPalCheckSSE3() (0)
#define RtcPalCheckSSSE3() (0)
#define RtcPalCheckSSE4_1() (0)
#define RtcPalCheckSSE4_2() (0)
#define RtcPalCheckSSE4a() (0)
#define RtcPalCheckMSSE() (0)
#define RtcPalCheckMOVBE() (0)

// ARM functions
// -----------------------------------------------------------------------------
#if RTCPAL_CPUFEATURE_NEON_ALWAYSAVAILABLE
#define RtcPalCheckNEON() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckNEON();
#endif

#ifndef _mm_prefetch
#define _mm_prefetch(whatever1,whatever2)
#endif

// __cpuid is very x86 centric.
// TODO: Remove __cpuid function completely and have aeccore #ifdef on RTCPAL_ARCH_X86 macro
static
void
RTCPAL_FORCEINLINE
__cpuid(int32_t CPUInfo[4], uint32_t InfoType)
{
    UNREFERENCED_PARAMETER(InfoType);
    CPUInfo[0] = 0;
    CPUInfo[1] = 0;
    CPUInfo[2] = 0;
    CPUInfo[3] = 0;
}

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_ARM_H__

