#ifndef __RTCPAL_X86_H__
#define __RTCPAL_X86_H__

#ifdef _MSC_VER
// 4127 - conditional expression is constant
#pragma warning(disable: 4127)
#endif

#if defined(_MSC_VER) && (defined(_M_IA64) || defined(_M_AMD64))
#define UNALIGNED __unaligned
#else
#define UNALIGNED
#endif

#ifdef __cplusplus

#define unalign_struct(T)   struct T

extern "C" {
#endif

// -----------------------------------------------------------------------------
// CPU Detect functions and stubs
// -----------------------------------------------------------------------------
// General functions
unsigned int RTCAPI RtcPalGetNumberofLogicalProcessorsPerPackage();
unsigned int RTCAPI RtcPalGetNumberofPackages();
unsigned int RTCAPI RtcPalGetNumberofProcessors();
unsigned int RTCAPI RtcPalGetNumberOfCores();
unsigned int RTCAPI RtcPalGetCPUClockMHz();
// X86 functions
#if RTCPAL_CPUFEATURE_CMOV_ALWAYSAVAILABLE
#define RtcPalCheckCMov() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckCMov();
#endif

#if RTCPAL_CPUFEATURE_MMX_ALWAYSAVAILABLE
#define RtcPalCheckMMX() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckMMX();
#endif

#if RTCPAL_CPUFEATURE_SSE_ALWAYSAVAILABLE
#define RtcPalCheckSSE() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE();
#endif

#if RTCPAL_CPUFEATURE_SSE2_ALWAYSAVAILABLE
#define RtcPalCheckSSE2() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE2();
#endif

#if RTCPAL_CPUFEATURE_SSE3_ALWAYSAVAILABLE
#define RtcPalCheckSSE3() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE3();
#endif

#if RTCPAL_CPUFEATURE_SSSE3_ALWAYSAVAILABLE
#define RtcPalCheckSSSE3() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSSE3();
#endif

#if RTCPAL_CPUFEATURE_SSE4_1_ALWAYSAVAILABLE
#define RtcPalCheckSSE4_1() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE4_1();
#endif

#if RTCPAL_CPUFEATURE_SSE4_2_ALWAYSAVAILABLE
#define RtcPalCheckSSE4_2() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE4_2();
#endif

#if RTCPAL_CPUFEATURE_SSE4a_ALWAYSAVAILABLE
#define RtcPalCheckSSE4a() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckSSE4a();
#endif

#if RTCPAL_CPUFEATURE_MSSE_ALWAYSAVAILABLE
#define RtcPalCheckMSSE() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckMSSE();
#endif

#if RTCPAL_CPUFEATURE_MOVBE_ALWAYSAVAILABLE
#define RtcPalCheckMOVBE() (1)
#else
RTCPAL_BOOL RTCAPI RtcPalCheckMOVBE();
#endif

// ARM functions
#define RtcPalCheckNEON() (0)

// -----------------------------------------------------------------------------

#if defined(__GNUC__)
static
void
RTCPAL_FORCEINLINE
__cpuid(int32_t CPUInfo[4], uint32_t InfoType)
{
#ifdef RTCPAL_ARCH_X64
    asm("cpuid"
        : "=a"(CPUInfo[0]),
          "=b"(CPUInfo[1]),
          "=c"(CPUInfo[2]),
          "=d"(CPUInfo[3])
        : "a"(InfoType));
#else
    // 32-bit code
    // register ebx can't be used in -fPIC (Position Independent Code).
    // Note: PIC is enabled across the board for all MACBU projects
    asm("pushl %%ebx\n"
        "cpuid\n"
        "movl %%ebx, %1\n"
        "popl %%ebx\n"
        : "=a"(CPUInfo[0]),
          "=r"(CPUInfo[1]),
          "=c"(CPUInfo[2]),
          "=d"(CPUInfo[3])
        : "a"(InfoType));
#endif
}

#ifndef __linux 

static
uint64_t
RTCPAL_FORCEINLINE
__rdtsc(void)
{
#ifdef RTCPAL_ARCH_X64
    // rdtsc is designed for IA32 and returns 64bit timestamp in EDX:EAX on 64-bit mode
    uint32_t lo32;
    uint32_t hi32;

    __asm__ __volatile__("rdtsc" : "=a" (lo32), "=d" (hi32));

    return (((uint64_t)hi32) << 32) | lo32;
#else
    uint64_t x;

    __asm__ __volatile__("rdtsc" : "=A" (x));

    return x;
#endif
}

#endif

#endif // __GNUC__

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_X86_H__

