#pragma once

#if !defined(_WIN32)

#ifndef __CONSUMER_H__
#define __CONSUMER_H__

#include "std.h"

#include "rtcpal.h"
#include "evntprov.h"
#include "evntrace.h"

#include "util.h"
#include "throwable.h"
#include "winport.h"
#include "evntcons.h"
#include "tracehs.h"

#define RTL_STATIC_LIST_HEAD(x) LIST_ENTRY x = {&x, &x}

#define INVALID_PROCESSTRACE_HANDLE ((TRACEHANDLE)INVALID_HANDLE_VALUE)

#define EVENT_TRACE_USE_RAWTIMESTAMP        0x00000002

#define EVENT_TRACE_USE_GLOBAL_SEQUENCE     0x00004000  // Use global sequence no.
#define EVENT_TRACE_USE_LOCAL_SEQUENCE      0x00008000  // Use local sequence no.

#define EVENT_TRACE_USE_SEQUENCE            0x0004  

#define EVENT_HEADER_FLAG_TRACE_MESSAGE     0x0008

#define EVENT_TRACE_BUFFER_INTERFACE_MODE   0x00040000

#define PROCESS_TRACE_MODE_EVENT_RECORD             0x10000000
#define PROCESS_TRACE_MODE_RAW_TIMESTAMP            0x00001000
#define PROCESS_TRACE_MODE_REAL_TIME                0x00000100


#define ETW_BUFFER_TYPE_RUNDOWN             1  
#define ETW_BUFFER_TYPE_CTX_SWAP            2  

#define ETW_BUFFER_TYPE_BATCHED             5  



#define TRACE_VERSION_MAJOR              1

#define EVENT_TRACE_GET_RAWEVENT         0x00000100

#define MAXIMUM_PROC_ALL_PLATFORMS      640
#define MAX_NUMBER_OF_PROCESSORS        (MAXIMUM_PROC_ALL_PLATFORMS)

#define EVENT_TRACE_USE_KBYTES_FOR_SIZE     0x00002000  // Use KBytes as file size unit

#define EVENT_TRACE_MODE_RESERVED           0x00100000  // Reserved bit, used to signal Heap/Critsec tracing
#define EVENT_TRACE_BUFFERING_MODE          0x00000400  // Buffering mode only

#define EVENT_TRACE_CLOCK_RAW          0x00000000
#define EVENT_TRACE_CLOCK_CPUCYCLE     0x00000003

#define EVENT_TRACE_GROUP_PROCESS              0x0300
#define EVENT_TRACE_GROUP_IMAGE                0x1400

#define EVENT_TRACE_TYPE_LOAD                  0x0A      // Load image
#define EVENT_TRACE_USE_PROCTIME               0x0001    // ProcessorTime field is valid
#define EVENT_TRACE_USE_NOCPUTIME              0x0002    // No Kernel/User/Processor Times

#define EVENT_TRACE_COMPRESSED_MODE         0x04000000
#define EVENT_TRACE_PRIVATE_LOGGER_MODE     0x00000800  // Process Private Logger



#define EVENT_HEADER_FLAG_EXTENDED_INFO         0x0001
#define EVENT_HEADER_FLAG_PRIVATE_SESSION       0x0002

#define EVENT_HEADER_FLAG_NO_CPUTIME            0x0010
#define EVENT_HEADER_FLAG_32_BIT_HEADER         0x0020
#define EVENT_HEADER_FLAG_64_BIT_HEADER         0x0040

#define EVENT_HEADER_FLAG_CLASSIC_HEADER        0x0100
#define EVENT_HEADER_FLAG_PROCESSOR_INDEX       0x0200

 
#define EVENT_HEADER_EXT_TYPE_PEBS_INDEX           0x0007
#define EVENT_HEADER_EXT_TYPE_PMC_COUNTERS         0x0008  

#define EVENT_HEADER_EXT_TYPE_EVENT_SCHEMA_TL      0x000B  
#define EVENT_HEADER_EXT_TYPE_PROV_TRAITS          0x000C 

#define ETW_MAX_EXTENDED_ITEM           (14)

#define ETW_BUFFER_FLAG_COMPRESSED     0x0040

#define KERNEL_LOGGER_NAME L"Kernel Logger"
#define MAX_KERNEL_TRACE_EVENTS         0x1F  

#define BYTES_PER_MB                    (1024 * BYTES_PER_KIB)

#define TRACE_MESSAGE                       0x10000000  


#define TRACE_HEADER_PEBS_INDEX_FLAG    0x00008000  
//#define TRACE_HEADER_SPARE_FLAG1        0x00004000  
//#define TRACE_HEADER_SPARE_FLAG2        0x00002000  
//#define TRACE_HEADER_SPARE_FLAG3        0x00001000  
//#define TRACE_HEADER_SPARE_FLAG4        0x00000800  
#define TRACE_HEADER_PMC_COUNTERS_MASK  0x00000700  
#define TRACE_HEADER_PMC_COUNTERS_SHIFT 8

#define TRACE_HEADER_EXT_ITEMS_MASK     (TRACE_HEADER_PEBS_INDEX_FLAG | TRACE_HEADER_PMC_COUNTERS_MASK)

#define EVENT_TRACE_READ_BEHIND          0x00000200

#define TIME_IN_RANGE(x, a, b) ((x >= a) && (x <= b))

#define MAX_TRACE_BUFFER_CACHE_SIZE     (29)

#define INVALID_STREAM_ID               (0xFFFFFFFF)


EXTERN_C const GUID GUID_NULL;

typedef struct _EVENT_EXTENDED_ITEM_PEBS_INDEX {
    ULONG64 PebsIndex;
} EVENT_EXTENDED_ITEM_PEBS_INDEX, *PEVENT_EXTENDED_ITEM_PEBS_INDEX;


//
// This is a compact version of EVENT_HEADER_EXTENDED_DATA_ITEM. This struct
// does not have DataPtr.
// We use this struct to log extended header item in a file.
//
typedef struct _ETWP_HEADER_EXTENDED_DATA_ITEM {

    USHORT      Size;                           // Total size of extended info
    USHORT      ExtType;                        // Extended info type
    struct {
        USHORT  Linkage             :  1;       // Indicates additional extended
                                                // data item
        USHORT  Reserved2           : 15;
    };
    USHORT      DataSize;                       // Size of extended info data

} ETWP_HEADER_EXTENDED_DATA_ITEM, *PETWP_HEADER_EXTENDED_DATA_ITEM;


/*
    On Windows:
    #define PETW_BUFFER_HEADER PWMI_BUFFER_HEADER
    and PWMI_BUFFER_HEADER is ETW_BUFFER_HEADER
*/
typedef ETW_BUFFER_HEADER* PETW_BUFFER_HEADER;

typedef SYSTEM_TRACE_HEADER* PSYSTEM_TRACE_HEADER;

#ifndef EVENT_HEADER_EXTENDED_DATA_ITEM_DEF
#define EVENT_HEADER_EXTENDED_DATA_ITEM_DEF
typedef struct _EVENT_HEADER_EXTENDED_DATA_ITEM
{
    USHORT Reserved1;
    USHORT ExtType;
    USHORT Linkage;
    USHORT DataSize;
    ULONGLONG DataPtr;
} EVENT_HEADER_EXTENDED_DATA_ITEM, *PEVENT_HEADER_EXTENDED_DATA_ITEM;

#endif // EVENT_HEADER_EXTENDED_DATA_ITEM_DEF

#ifndef EVENT_RECORD_DEF
#define EVENT_RECORD_DEF
typedef struct _EVENT_RECORD {

    EVENT_HEADER        EventHeader;            // Event header
    ETW_BUFFER_CONTEXT  BufferContext;          // Buffer context
    USHORT              ExtendedDataCount;      // Number of extended
                                                // data items
    USHORT              UserDataLength;         // User data length
    PEVENT_HEADER_EXTENDED_DATA_ITEM            // Pointer to an array of 
                        ExtendedData;           // extended data items                                               
    PVOID               UserData;               // Pointer to user data
    PVOID               UserContext;            // Context from OpenTrace
} EVENT_RECORD, *PEVENT_RECORD;

typedef const EVENT_RECORD *PCEVENT_RECORD;
#endif // EVENT_RECORD_DEF


typedef VOID (WINAPI *PEVENT_CALLBACK)( PEVENT_TRACE pEvent );
typedef VOID (WINAPI *PEVENT_RECORD_CALLBACK) (PEVENT_RECORD EventRecord);


typedef struct _EVENT_TRACE_LOGFILEW *PEVENT_TRACE_LOGFILEW;

typedef struct _EVENT_TRACE_LOGFILEA *PEVENT_TRACE_LOGFILEA;

#ifdef UNICODE

#define PEVENT_TRACE_LOGFILE PEVENT_TRACE_LOGFILEW

#else

#define PEVENT_TRACE_LOGFILE PEVENT_TRACE_LOGFILEA

#endif // UNICODE


typedef ULONG (WINAPI *PEVENT_TRACE_BUFFER_CALLBACKW) (PEVENT_TRACE_LOGFILEW Logfile);

typedef ULONG (WINAPI *PEVENT_TRACE_BUFFER_CALLBACKA) (PEVENT_TRACE_LOGFILEA Logfile);


struct _EVENT_TRACE_LOGFILEW {
    LPWSTR                  LogFileName;      // Logfile Name
    LPWSTR                  LoggerName;       // LoggerName
    LONGLONG                CurrentTime;      // timestamp of last event
    ULONG                   BuffersRead;      // buffers read to date
    union {
        // Mode of the logfile
        ULONG               ProcessTraceMode;
        ULONG               LogFileMode;
        
    };
    
    EVENT_TRACE             CurrentEvent;     // Current Event from this stream.
    TRACE_LOGFILE_HEADER    LogfileHeader;    // logfile header structure
    PEVENT_TRACE_BUFFER_CALLBACKW             // callback before each buffer
                            BufferCallback;   // is read
    //
    // following variables are filled for BufferCallback.
    //
    ULONG                   BufferSize;
    ULONG                   Filled;
    ULONG                   EventsLost;
    //
    // following needs to be propaged to each buffer
    //
    union {
        PEVENT_RECORD_CALLBACK  EventRecordCallback;
        // Callback with EVENT_TRACE
        PEVENT_CALLBACK         EventCallback;
        
    };

    ULONG                   IsKernelTrace;    // TRUE for kernel logfile

    PVOID                   Context;          // reserved for internal use
};

struct _EVENT_TRACE_LOGFILEA {
    LPSTR                   LogFileName;      // Logfile Name
    LPSTR                   LoggerName;       // LoggerName
    LONGLONG                CurrentTime;      // timestamp of last event
    ULONG                   BuffersRead;      // buffers read to date
    union {
        ULONG               LogFileMode;      // Mode of the logfile
        ULONG               ProcessTraceMode; // Processing flags
    };
    EVENT_TRACE             CurrentEvent;     // Current Event from this stream
    TRACE_LOGFILE_HEADER    LogfileHeader;    // logfile header structure
    PEVENT_TRACE_BUFFER_CALLBACKA             // callback before each buffer
                            BufferCallback;   // is read

    //
    // following variables are filled for BufferCallback.
    //
    ULONG                   BufferSize;
    ULONG                   Filled;
    ULONG                   EventsLost;
    //
    // following needs to be propaged to each buffer
    //
    union {
        PEVENT_CALLBACK         EventCallback;  // callback for every event
        PEVENT_RECORD_CALLBACK  EventRecordCallback;
    };


    ULONG                   IsKernelTrace;  // TRUE for kernel logfile

    PVOID                   Context;        // reserved for internal use
};


//
// EventRecord followd by ExtendedItems array
//

typedef struct _ETW_EVENT_INFO {
    union {
        struct {
            EVENT_RECORD EventRecord;
            struct {
                ULONG InstaceId;
                ULONG ParentInstanceId;
                GUID  ParentGuid;
            } InstanceInfo;
            EVENT_HEADER_EXTENDED_DATA_ITEM
                         ExtendedItems[ETW_MAX_EXTENDED_ITEM];
        };
        EVENT_TRACE EventTrace;
    };
} ETW_EVENT_INFO, *PETW_EVENT_INFO;



TRACEHANDLE
OpenTraceW(
    PEVENT_TRACE_LOGFILEW Logfile
);

TRACEHANDLE
OpenTraceA(
    PEVENT_TRACE_LOGFILEA Logfile
);

ULONG
ProcessTrace(
    PTRACEHANDLE HandleArray,
    ULONG HandleCount,
    LPFILETIME StartTime,
    LPFILETIME EndTime
);


ULONG
CloseTrace(
    TRACEHANDLE TraceHandle
);

typedef struct _EVENT_TRACE_LOGFILEW EVENT_TRACE_LOGFILEW;
typedef struct _EVENT_TRACE_LOGFILEA EVENT_TRACE_LOGFILEA;

#if defined(UNICODE)

#define EVENT_TRACE_LOGFILE EVENT_TRACE_LOGFILEW
#define PEVENT_TRACE_BUFFER_CALLBACK PEVENT_TRACE_BUFFER_CALLBACKW

#define OpenTrace OpenTraceW

#else

#define EVENT_TRACE_LOGFILE EVENT_TRACE_LOGFILEA
#define PEVENT_TRACE_BUFFER_CALLBACK PEVENT_TRACE_BUFFER_CALLBACKA
#define OpenTrace OpenTraceA

#endif // UNICODE

#define FlagOn(F,SF)    ((bool)(((F) & (SF)) != 0))
#define SetFlag(F,SF)       { (F) |= (SF); }

#define BooleanFlagOn(F,SF) ((bool)(((F) & (SF)) != 0))

// Number of maximum number of logger instances to support.
#define MAXLOGGERS 64

#define RtlCopyMemory memcpy

#define NT_ASSERT ASSERT

#define EtwpFree free

//
// Forward declaration for TRACELOG_CONTEXT
//
typedef struct _TRACELOG_CONTEXT *PTRACELOG_CONTEXT;

typedef enum _TRACELOG_HANDLE_STATE {
    HandleStateOpened,
    HandleStateProcessed,
    HandleStateClosed
} TRACELOG_HANDLE_STATE;


typedef struct _SINGLE_LIST_ENTRY {
    struct _SINGLE_LIST_ENTRY *Next;
} SINGLE_LIST_ENTRY, *PSINGLE_LIST_ENTRY;

typedef struct _TRACE_BUFFER_LIST_ENTRY {
    SINGLE_LIST_ENTRY Entry;
    LONG           FileIndex;      // Index in File of this Buffer
    ULONG          BufferOffset;   // Offset in Buffer for the current event
    ETW_EVENT_INFO Event;          // CurrentEvent for this buffer
} TRACE_BUFFER_LIST_ENTRY, *PTRACE_BUFFER_LIST_ENTRY;


typedef
ULONG
(*PETW_MAP_EVENT_FUNCTION)(
    PTRACELOG_CONTEXT TracelogContext,
    PVOID StrcEvent,
    PVOID EventRecord,
    ULONG TraceType,
    PETW_BUFFER_HEADER TraceBuffer
);

typedef struct _TRACE_BUFFER_CACHE_ENTRY {
    LONG Index;
    PETW_BUFFER_HEADER Buffer;
} TRACE_BUFFER_CACHE_ENTRY, *PTRACE_BUFFER_CACHE_ENTRY;

typedef struct _BUFFER_DESCRIPTOR {
    ULONGLONG FileOffset;
    ULONG     BufferSize;
} BUFFER_DESCRIPTOR, *PBUFFER_DESCRIPTOR;


typedef struct _TRACE_STREAM_ID {
    union {
        struct {
            USHORT LogId;
            union {
                USHORT BufferId;
                struct {
                    USHORT Type : 4;
                    USHORT ProcessorIndex : 12;
                };
            };
        };
        ULONG Id;
    };
} TRACE_STREAM_ID, *PTRACE_STREAM_ID;


typedef struct _TRACELOG_CONTEXT {
    LIST_ENTRY                Entry;          // Keeps track of storage allocations.

    //
    // Corresponding TraceHandle and it's state
    //
    TRACEHANDLE               TraceHandle;
    TRACELOG_HANDLE_STATE     HandleState;
    ULONG                     MaximumProcessors;

    ULONG                     ConversionFlags;    // Indicates event processing options

    ULONG                      ClockType;
    ULONG                      CpuSpeedInMHz;
    LARGE_INTEGER              PerfFreq;             // Frequency from the LogFile
    LARGE_INTEGER              StartTime;            // Start Wall clock time
    LARGE_INTEGER              StartPerfClock;       // Start PerfClock value
    DOUBLE                     TimeMultiplier;

    //
    // Copy of the Logfile datastructure passed by
    // OpenTrace
    //
    EVENT_TRACE_LOGFILEW       Logfile;

    //
    // Function pointer to appropriate
    // and MapEvent functions
    //
    PETW_MAP_EVENT_FUNCTION    MapEventFunction;

    //
    // Flag to tell if this feed is RT
    //
    UCHAR                     IsRealTime;
    USHORT                    CompressionFormat;

    HANDLE                    ConsumerHandle;

    //
    // This implements a caching scheme for Sequential files with repeated
    // call to ProcessTrace.
    //
    // Only valid for Sequential, used for
    // Read Behind.
    // Upon Exit From ProcessTrace, this
    // value can be cached to avoid rescanning.
    // Maximum Read Position for the file.
    //

    ULONGLONG                 OldMaxReadPosition;

    LONGLONG                  LastTimeStamp;

    LONG                      BufferBeingRead;
    OVERLAPPED                AsynchRead;
    HANDLE                    FileHandle;

#ifdef _REAL_TIME_DEBUG
    TRACEHANDLE               DebugTraceHandle;
#endif

    //
    // Raw data for the Logfile header event.
    //

    ULONG                       LogfileHeaderEventSize;
    PSYSTEM_TRACE_HEADER        LogfileHeaderEvent;

    //
    // Scratch space to hold compressed trace buffer.
    //

    // PETW_BUFFER_HEADER          ScratchBuffer;
    PVOID                       WorkSpace;

    //
    // For real-time mode, event and buffer lost count are stored here.
    //

    ULONG                       EventsLostCount;
    ULONG                       BuffersLostCount;

    //
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // !! Fields Below this will be reset upon ProcessTrace exit.!!
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    //

    ULONG                       LoggerId;            // Logger Id of this DataFeed.

    BOOLEAN                     CtxSwapStream;
    BOOLEAN                     PerFileStream;

    ULONG                       BufferSize;
    ULONG                       BufferCount;
    ULONG                       StartBuffer;         // Start of the Circular Buffers.
    ULONG                       FirstBuffer;         // Jump off point to start reading.
    ULONG                       LastBuffer;          // last of the buffers in the boundary.

    SINGLE_LIST_ENTRY           Root;
    PTRACE_BUFFER_LIST_ENTRY    BufferList;
    PVOID                       BufferCacheSpace;

    LIST_ENTRY                  StreamListHead;                    // Used to free the Stream datastructures
    PTRACE_STREAM_ID BufferStreamMap;  // Maps each buffer in the file to its stream.
    PBUFFER_DESCRIPTOR BufferMap;      // Trace buffer offsets/sizes for compressed files
    ULONGLONG                   MaxReadPosition;

    PTRACE_STREAM_HEAP          StreamHeap;

    union {
        PTRACE_STREAM_CONTEXT       BufferStream[1];
        TRACE_BUFFER_CACHE_ENTRY    BufferCache[MAX_TRACE_BUFFER_CACHE_SIZE];
    };

} TRACELOG_CONTEXT, *PTRACELOG_CONTEXT;




typedef
ULONG
(*PETW_NEXT_EVENT_FUNCTION)(
    PTRACE_STREAM_CONTEXT Stream
);

//
// All buffers from a real-time session or a log file are logically grouped into
// a per-processor "stream" which is described by the following structure.
//

typedef struct _TRACE_STREAM_CONTEXT {
    LIST_ENTRY         AllocEntry;       // Used to free storage
    PETW_NEXT_EVENT_FUNCTION NextEventFunction;
    PETW_EVENT_INFO    CurrentEvent;
    PTRACELOG_CONTEXT  TraceContext;     // back pointer to the LogFileContext
    PETW_BUFFER_HEADER StreamBuffer;     // CurrentBuffer for this stream
    USHORT             AllocEventSize;   // Allocation size iff CurrentEvent points to a heap allocation, otherwise 0
    ULONG              BufferCount;
    ULONG              CurrentOffset;    // Offset into the current buffer.
    ULONG              NextBuffer;       // Index of logfile buffer currently read.
    ULONG              NextBufferSize;   // Size of the next buffer in the stream.
    ULONGLONG          NextBufferOffset; // Index of logfile buffer currently read.
    TRACE_STREAM_ID    StreamId;
    BOOLEAN            IsActive;         // Is this Stream still active?
    BOOLEAN            ScanDone;         // For circular logfiles
    BOOLEAN            FileStream;       // Stream represents entire file

#if DBG

    ULONG              EventCount;       // Number of Events detected in current buffer
    ULONG              CbCount;

#endif

} TRACE_STREAM_CONTEXT, *PTRACE_STREAM_CONTEXT;


typedef struct _EVENT_TRACE_CALLBACK {
    LIST_ENTRY          Entry;
    GUID                Guid;
    PEVENT_CALLBACK     CallbackRoutine;
} EVENT_TRACE_CALLBACK, *PEVENT_TRACE_CALLBACK;

extern LIST_ENTRY EtwpEventCallbackList;



void
EtwpAcquireConsumerLock ();

void
EtwpReleaseConsumerLock ();

PTRACELOG_CONTEXT
EtwpLookupTracelogContextLocked (
    TRACEHANDLE TraceHandle
);

VOID
EtwpFreeTraceContextWithLock (
    PTRACELOG_CONTEXT TracelogContext
);

ULONG
EtwpMapEventToEventRecord (
    PTRACELOG_CONTEXT TracelogContext,
    PVOID SrcEvent,
    PVOID EventRecord,
    ULONG TraceType,
    PETW_BUFFER_HEADER TraceBuffer
);

ULONG
EtwpMapEventToEventTrace (
    PTRACELOG_CONTEXT TracelogContext,
    PVOID SrcEvent,
    PVOID Event,
    ULONG TraceType,
    PETW_BUFFER_HEADER TraceBuffer
);

BOOL
EtwpSynchReadFile (
    HANDLE LogFile,
    LPCWSTR LogFileName,
    LPVOID Buffer,
    DWORD NumberOfBytesToRead,
    LPDWORD NumberOfBytesRead,
    ULONGLONG ReadPosition,
    BOOLEAN IsRealTime
);

VOID
EtwpCalculateTimeMultipler (
    PTRACELOG_CONTEXT TracelogContext
);

BOOLEAN
EtwpIsPerFileStream (
    PTRACELOG_CONTEXT Context
);

ULONG
EtwpFreeTraceHandle (
    TRACEHANDLE TraceHandle
);

VOID
EtwpCopyLogHeader (
    PTRACE_LOGFILE_HEADER pLogFileHeader,
    PVOID MofData,
    ULONG MofLength,
    PWSTR *LoggerName
);

ULONG
EtwpSetupLogFileStreams (
    PTRACE_STREAM_HEAP StreamHeap,
    ULONG LogFileCount,
    PTRACELOG_CONTEXT *ContextArray,
    LONGLONG StartTime,
    LONGLONG EndTime
);

ULONG
EtwpCalculateMaxStreams (
    ULONG LogFileCount,
    PTRACELOG_CONTEXT *ContextArray
);


ULONG
EtwpProcessLogHeader (
    PTRACELOG_CONTEXT *Context,
    ULONG LogfileCount,
    BOOLEAN IsOpenTrace
);

ULONG
EtwpProcessTraceLog (
    PTRACELOG_CONTEXT *Logfiles,
    ULONG LogfileCount,
    LONGLONG StartTime,
    LONGLONG EndTime
);

VOID
EtwpGetEventTimeStamp (
    PETW_EVENT_INFO Event,
    PLARGE_INTEGER TimeStamp
);

ULONG
EtwpDoEventCallbacks (
    PTRACELOG_CONTEXT TracelogContext,
    PETW_EVENT_INFO Event
);

ULONG
EtwpDoBufferCallback (
    PTRACELOG_CONTEXT TracelogContext,
    PETW_BUFFER_HEADER Buffer
);

PEVENT_TRACE_CALLBACK
EtwpGetCallbackRoutine (
    LPGUID Guid
);

void
EtwpCleanupTraceLog (
    PTRACELOG_CONTEXT pEntry,
    BOOLEAN bSaveLastOffset
);

LONG
EtwpEventCompare (
    PETW_EVENT_INFO Event1,
    PTRACELOG_CONTEXT TracelogContext1,
    PETW_EVENT_INFO Event2,
    PTRACELOG_CONTEXT TracelogContext2
);

BOOLEAN
EtwpGetEventSequence (
    PTRACELOG_CONTEXT TracelogContext,
    PETW_EVENT_INFO Event,
    PULONG SequenceNumber
);

ULONG
EtwpAddFileStream (
    PTRACE_STREAM_HEAP StreamHeap,
    PTRACELOG_CONTEXT Context
);

// RTCPAL supports certain versions of ETL only.
BOOLEAN
IS_ETL_VER_SUPPORTED (
    PTRACE_LOGFILE_HEADER logFileHeader
);

ULONG
EtwpInitializeCompression (
    PTRACELOG_CONTEXT TracelogContext
);

ULONG
EtwpAddTraceStream (
    PTRACE_STREAM_HEAP StreamHeap,
    PTRACELOG_CONTEXT Context,
    USHORT StreamType,
    USHORT ProcessorIndex,
    LONGLONG StartTime,
    LONGLONG EndTime
);

ULONG
EtwpGetLastBufferWithMarker (
    PTRACELOG_CONTEXT pContext
);


BOOLEAN
EtwpValidateBuffer (
    PTRACELOG_CONTEXT TracelogContext,
    PETW_BUFFER_HEADER Buffer
);

BOOLEAN
EtwpGetNextEvent (
    PETW_BUFFER_HEADER Buffer,
    PTRACELOG_CONTEXT TracelogContext,
    PULONG OffsetPtr,
    PETW_EVENT_INFO Event
);

ULONG
EtwpGetNextBuffer (
    PTRACE_STREAM_CONTEXT Stream
);

ULONG
EtwpAdvanceToNewEventEx (
    PTRACE_STREAM_CONTEXT Stream
);

ULONG
EtwpParseTraceEvent (
    PTRACELOG_CONTEXT TracelogContext,
    PETW_BUFFER_HEADER TraceBuffer,
    ULONG Offset,
    ULONG TraceType,
    PETW_EVENT_INFO EventInfo
);

BOOLEAN
EtwpLookupNextBufferIndex (
    PTRACE_STREAM_CONTEXT Stream
);

void* EtwpAlloc(size_t Size);

#endif // __CONSUMER_H__

#endif // !_WIN32
