/** \file error.h

    This file defines the ErrorT object.
 */
#pragma once
#include "rtcpal.h"
#include <string>

namespace throwable
{
struct ErrorT
{
    /// Max callstack will be saved.
    static const int kMaxBackTrace = 10;
    
    /// Error code.
    int error;
    
    /// The line number where the error occured.
    int line;
    
    /// The file where the error occured.
    std::string file;
    
    /// The error message.
    std::string msg;
    
    /// The callstack where the error occured.
    void* callstack[kMaxBackTrace];
    
    ErrorT(int error, int line, const char* file, const char* msg)
        : error(error), line(line), file(file), msg(msg)
    {
        for (int i = 0; i < kMaxBackTrace; ++i)
        {
            callstack[i] = 0;
        }
    }
};

#define Throw(error)    \
    do {    \
        throwable::ErrorT e(error, __LINE__, __FILE__, ""); \
        (void)RtcPalBacktrace(e.callstack, throwable::ErrorT::kMaxBackTrace, 0);   \
        throw e;    \
    } while(false)
    
#define ThrowIf(T, error)   \
    do {    \
        if (T)  {   \
            Throw(error);   \
        }   \
    } while(false)
}
