#pragma once

#ifndef __ETWCOMMON_H__
#define __ETWCOMMON_H__

#include "consumer.h"

// Trace header for NTPERF events 
typedef struct _PERFINFO_TRACE_HEADER {  
    union {  
        ULONG       Marker;  
        struct {  
            USHORT  Version;  
            UCHAR   HeaderType;  
            UCHAR   Flags;  //WMI uses this flag to identify event types          
        };  
    };  
    union {  
        ULONG            Header;    // both sizes must be the same!          
        WMI_TRACE_PACKET Packet;  
    };  
    union {  
        //PERFINFO_TIMESTAMP TS;  
        LARGE_INTEGER      SystemTime;  
    };
    UCHAR Data[1];  
} PERFINFO_TRACE_HEADER, *PPERFINFO_TRACE_HEADER;  

 

typedef struct _WMI_TRACE_MESSAGE_PACKET {  // must be ULONG!!      
    USHORT  MessageNumber;                  // The message Number, index of messages by GUID
                                            // Or ComponentID      
    USHORT  OptionFlags ;                   // Flags associated with the message  
} WMI_TRACE_MESSAGE_PACKET, *PWMI_TRACE_MESSAGE_PACKET;  


typedef struct _MESSAGE_TRACE_HEADER {
    union {  
        ULONG       Marker;  
        struct {  
            USHORT  Size;                   // Total Size of the message including header              
            UCHAR   Reserved;               // Unused and reserved              
            UCHAR   Version;                // The message structure type (TRACE_MESSAGE_FLAG)          
        };  
    };  
    union {  
        ULONG            Header;            // both sizes must be the same!          
        WMI_TRACE_MESSAGE_PACKET Packet;  
    };  
} MESSAGE_TRACE_HEADER, *PMESSAGE_TRACE_HEADER;  



/*++

Routine Description:

    This routine is used in parsing to advance to the next event
    in trace buffer.

    N.B. BufferSize and Alignment should be set to correct value
         in the buffer header.

Arguments:

    BufferHeader - Supplies pointer to trace buffer containging ETW
        events.

    Offset - Supplies offset into the buffer at which we should start
        looking for the next event.

    EventSize - Receives size of the event rounded up if valid event
        was found, or zero.

Returned Value:

    TraceType of the event or zero if parsing failed.

--*/
ULONG
EtwpGetNextEventOffsetType (
    PETW_BUFFER_HEADER BufferHeader,
    ULONG Offset,
    PULONG EventSize
);


#endif // __ETWCOMMON_H__