#pragma once

#ifndef __ETWRTCOMMON_H__
#define __ETWRTCOMMON_H__

#include <string>

extern "C" {
#include <signal.h>
}
#include "rtcpal.h"

namespace etw {


/// Track provider process status and information.
/// A process pid is valid only when pid > 0 and startTicks == pid's startTicks in OS.
struct ProviderPSInfo{
    union {
        int32_t pid;        // provider pid
        int32_t isEnabled;  // 0: provider disabled; 1: provider enabled
    };
    union {
        int32_t reserved;   // reserved
        int32_t maxIndex;   // max slot index with a valid process.
    };
    int64_t startTicks;    // process start time in ticks

    ProviderPSInfo() : pid(-1), maxIndex(-1), startTicks(0) {}
};


class ETWRT {
public:
    /// signal that controller sends to provider to start/stop provider.
    static constexpr int REALTIME_SIGNAL = SIGUSR1;

    static std::string GetEtlFullPath(const std::string& guid);
    static std::string GetEtlFullPath(const GUID& guid);

    static bool IsPidValid(const ProviderPSInfo& ps);

    static int64_t GetPSStartTimeInTicks(int32_t pid);

    static ULONG CreateFIFO(const std::string& filepath);

    static bool SendSignal(pid_t pid);

private:
    /// Runtime configuration directory. If this path is not pre-created,
    /// provider initialization will create it as long as provider process
    /// has the permission to do so.
    static constexpr const char* CONFIG_PATH = "/var/etw";
};




} //namespace etw

#endif // __ETWRTCOMMON_H__
