/** \mainpage ETW for No-Windows Platform Manual
    
    \section intro_sec Introduction
    ETW is a high performance event framework used on Windows platform. Unfortunately,
    there is no similar framework on Non-Windows platform. So we implement a simple
    framework on Non-Windows platform which can generate the etw file with the same 
    format. You can use any etw tools which you are using on windows to process 
    these files.
    
    \section etw_intro_sec Glossary
    \li \b Provider
        Provider in ETW is the event producer. It will generate the events. If you
        want to use a provider, you need invoke EventRegister() function to register.
        Then you can invoke EventWrite() to write your event. At last, you need 
        invoke EventUnregister() to unregister the provider.
    \li \b Session
        Session in ETW is the event consumer. If you want to use a session, you 
        need invoke StartTrace() function to start a event session. Then you need
        invoke EnableSession() to notify the framework, which provider should be 
        enabled in this session. That means, although there may be a lot of providers,
        you can only get the events which are generated by the providers you 
        enabled via EnableSession(). If you don't want to get any event, you need
        invoke StopTrace() to stop the session and release the resources.
        
    \section impl_sec Implementation
    We implements the folling APIs. All of them have the same prototype with Windows
    API:
    \li EventRegister()
    \li EventUnregister()
    \li EventWrite()
    \li StartTrace()
    \li EnableTrace()
    \li StopTrace()
    All these function won't throw exception and will return win32 error code.
    
    \li provider.cpp implements the Provider object.
    \li session.cpp implements the Session object.
    
    When user invokes StartTrace(), a Session object will be created and a seperate
    thread will be launched. When EventWrite() function is invoked(), the corresponding
    provider will check which session enables it. Then, the provider will write
    the event data to the session's buffer. If the buffer is full, it will notify
    the session logger thread to flush its buffer to the file, wait and write the 
    event data again. You can check the code to get the detail.
    
    \section file_sec Structure of ETW File
    The whole etw file can be treated as an array of the ETW_BUFFER.
    \code
    struct ETW_FILE
    {
        ETW_BUFFER[]
    };
    \endcode
    All of the etw buffers have the same size. The structure of the ETW_BUFFER looks
    like this:
    \code
    struct ETW_BUFFER
    {
        ETW_BUFFER_HEADER;
        USER_DATA;
    }
    \endcode
    The first buffer (ETW_BUFFER[0]) is the ETW_LOGFILE_HEADER. It defines the buffer
    size and other datas. Other buffers are actual event data. You can check the 
    code to get the detail.
 */
