#pragma once

#include "session.h"
#include "provider.h"
#include <unistd.h>
#include <sys/ptrace.h>
#include <sys/times.h>

using namespace etw;

#include "util.h"

// Definitions to enable Porting the Windows version of TraceEvent
#define RtlCopyMemory memcpy
#define ProbeForRead(PVOID, SIZE_T, ULONG);
#define PROBE_ALIGNMENT
#define ProbeForReadSmallStructure(GuidPtr, SIZE_T, ULONG);
#define KernelMode   1

// Defining a new volatile keyword to keep the code base similar to windows
#define VOLATILE 

// Defining Max retries to acquire buffer 
#ifndef MaxRetry
#define MaxRetry     5
#endif

ULONG
RtcPalControlTraceW (
    _In_ TRACEHANDLE TraceHandle,
    _In_opt_ LPCWSTR InstanceName,
    _Inout_ PEVENT_TRACE_PROPERTIES Properties,
    _In_ ULONG ControlCode
    );

ULONG
RtcPalControlTraceA (
    _In_ TRACEHANDLE TraceHandle,
    _In_opt_ LPCSTR InstanceName,
    _Inout_ PEVENT_TRACE_PROPERTIES Properties,
    _In_ ULONG ControlCode
    );


TRACEHANDLE
RtcPalGetTraceLoggerHandle (
    _In_ PVOID Buffer
    );

UCHAR
RtcPalGetTraceEnableLevel (
    _In_ TRACEHANDLE TraceHandle
    );

ULONG
RtcPalGetTraceEnableFlags (
    _In_ TRACEHANDLE TraceHandle
    );

ULONG
RtcPalRegisterTraceGuidsW2 (
    _In_ WMIDPREQUEST RequestAddress,
    _In_opt_ PVOID RequestContext,
    _In_ LPCGUID ControlGuid,
    _In_ ULONG GuidCount,
    _In_reads_opt_(GuidCount) PTRACE_GUID_REGISTRATION TraceGuidReg,
    _In_opt_ LPCWSTR MofImagePath,
    _In_opt_ LPCWSTR MofResourceName,
    _Out_ PREGHANDLE RegistrationHandle
    );

ULONG
RtcPalUnregisterTraceGuids (
    _In_ REGHANDLE RegistrationHandle
    );

ULONG
RtcPalTraceMessageVa (
    _In_ TRACEHANDLE LoggerHandle,
    _In_ ULONG MessageFlags,
    _In_ LPCGUID MessageGuid,
    _In_ USHORT MessageNumber,
    _In_ va_list MessageArgList
);

ULONG
RtcPalTraceMessage (
    _In_ TRACEHANDLE LoggerHandle,
    _In_ ULONG MessageFlags,
    _In_ LPCGUID MessageGuid,
    _In_ USHORT MessageNumber,
    ...
);

ULONG RtcPalTraceEvent(
    _In_ TRACEHANDLE SessionHandle,
    _In_ PEVENT_TRACE_HEADER EventTrace
);


#define TraceMessage        RtcPalTraceMessage

#define ControlTraceW           RtcPalControlTraceW
#define ControlTraceA           RtcPalControlTraceA
#define GetTraceLoggerHandle    RtcPalGetTraceLoggerHandle
#define GetTraceEnableLevel     RtcPalGetTraceEnableLevel
#define GetTraceEnableFlags     RtcPalGetTraceEnableFlags
#define RegisterTraceGuids      RtcPalRegisterTraceGuidsW2
#define RegisterTraceGuidsW     RtcPalRegisterTraceGuidsW2
#define UnregisterTraceGuids    RtcPalUnregisterTraceGuids
#define TraceEvent		RtcPalTraceEvent


#if defined(UNICODE)
#define ControlTrace ControlTraceW
#else
#define ControlTrace ControlTraceA
#endif
