/** \file winport.h

    This file includes structures and constants which come from windows internal 
    code.
 */

#pragma once

#include "evntrace.h"

// define versions used in ETW header
// simulate XP
#define OS_MAJOR_VERSION 5
#define OS_MINOR_VERSION 1
#define ETL_SUB_VERSION 1
#define ETL_SUBMINOR_VERSION 5

#define DEFAULT_TRACE_ALIGNMENT             8
#define ALIGN_TO_POWER2( x, n ) (((ULONG)(x) + ((n)-1)) & ~((ULONG)(n)-1))

#define BYTES_PER_KIB                       1024

#define DEFAULT_BUFFER_SIZE                 (4*BYTES_PER_KIB)
#define MIN_ETW_BUFFER_SIZE                 1
#define MAX_ETW_BUFFER_SIZE                 (1 * BYTES_PER_KIB * BYTES_PER_KIB)
#define DEFAULT_LOG_BUFFER_SIZE             DEFAULT_BUFFER_SIZE

#define COMPACT_HEADER_SIZE (RTL_SIZEOF_THROUGH_FIELD(SYSTEM_TRACE_HEADER, SystemTime))

#define TRACE_HEADER_FLAG                   0x80000000
#define TRACE_HEADER_EVENT_TRACE            0x40000000

#define TRACE_HEADER_ENUM_MASK              0x00FF0000

#define TRACE_HEADER_TYPE_COMPACT32         3
#define TRACE_HEADER_TYPE_COMPACT64         4

#define TRACE_HEADER_TYPE_INSTANCE32        11
#define TRACE_HEADER_TYPE_ERROR             13

#define TRACE_HEADER_TYPE_MESSAGE           15
#define TRACE_HEADER_TYPE_PERFINFO32        16
#define TRACE_HEADER_TYPE_PERFINFO64        17

#define TRACE_HEADER_TYPE_EVENT_HEADER32    18
#define TRACE_HEADER_TYPE_EVENT_HEADER64    19
#define TRACE_HEADER_TYPE_FULL_HEADER32     10
#define TRACE_HEADER_TYPE_FULL_HEADER64     20

#define TRACE_HEADER_TYPE_INSTANCE64        21

#define EVENT_TRACE_GROUP_HEADER               0x0000

#define ETW_BUFFER_TYPE_GENERIC             0
#define ETW_BUFFER_TYPE_HEADER              4

#define TRACE_HEADER_TYPE_SYSTEM32          1
#define TRACE_HEADER_TYPE_SYSTEM64          2

#define ETW_BUFFER_FLAG_FLUSH_MARKER        0x0001
#define ETW_BUFFER_FLAG_PROC_INDEX          0x0020

#define EVENT_TRACE_CLOCK_PERFCOUNTER       1
#define EVENT_TRACE_CLOCK_SYSTEMTIME        2

#define EVENT_HEADER_EVENT32      ((USHORT)(((TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE) >> 16) | \
                                      TRACE_HEADER_TYPE_EVENT_HEADER32))

#define EVENT_HEADER_EVENT64      ((USHORT)(((TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE) >> 16) | \
                                      TRACE_HEADER_TYPE_EVENT_HEADER64))

#define SYSTEM_TRACE_MARKER32     (TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE \
                                    | (TRACE_HEADER_TYPE_SYSTEM32 << 16))

#define SYSTEM_TRACE_MARKER64     (TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE \
                                    | (TRACE_HEADER_TYPE_SYSTEM64 << 16))

#define TRACE_HEADER_FULL32       (TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE \
                                  | (TRACE_HEADER_TYPE_FULL_HEADER32 << 16))

#define TRACE_HEADER_FULL64       (TRACE_HEADER_FLAG | TRACE_HEADER_EVENT_TRACE \
                                  | (TRACE_HEADER_TYPE_FULL_HEADER64 << 16))

#ifdef __x86_64__
#define TRACE_HEADER_FULL       TRACE_HEADER_FULL64
#else
#define TRACE_HEADER_FULL       TRACE_HEADER_FULL32
#endif


//#ifndef ETW_BUFFER_CONTEXT_DEF
// #define ETW_BUFFER_CONTEXT _ETW_BUFFER_CONTEXT
typedef struct _ETW_BUFFER_CONTEXT
{
    union 
    {
        struct 
        {
            UCHAR ProcessorNumber;
            UCHAR Alignment;
        };
        USHORT ProcessorIndex;
    };
    USHORT LoggerId;
}ETW_BUFFER_CONTEXT;
//#endif

struct ETW_REF_CLOCK 
{
    LARGE_INTEGER StartTime;
    LARGE_INTEGER StartPerfClock;
};

struct ETW_BUFFER_HEADER
{
    ULONG                              BufferSize;         // BufferSize
    ULONG                              SavedOffset;        // Temp saved offset
    volatile ULONG                     CurrentOffset;      // Current offset
    volatile LONG                      ReferenceCount;     // Reference count
    LARGE_INTEGER                      TimeStamp;          // Flush time stamp
    LONGLONG                           SequenceNumber;     // Buffer sequence number

    union 
    {
        struct 
        {                                                  // DBG_INFO buffers send to debugger
            ULONGLONG                  ClockType : 3;
            ULONGLONG                  Frequency : 61;
        };
        void*                          SlistEntry;         // Local list when flushing
        ETW_BUFFER_HEADER*             NextBuffer;         // FlushList
    };

    _ETW_BUFFER_CONTEXT                 ClientContext;      // LoggerId/ProcessorIndex
    ULONG                              State;              // (Free/GeneralLogging/Flush)

    ULONG                              Offset;             // Offset when flushing (can overlap SavedOffset)
    USHORT                             BufferFlag;         // (flush marker, events lost)
    USHORT                             BufferType;         // (generic/rundown/cswitch/reftime)
    union 
    {
        ULONG                          Padding1[4];
        ETW_REF_CLOCK                  ReferenceTime;      // persistent real-time
        LIST_ENTRY                     GlobalEntry;        // Global list entry
        struct 
        {
            PVOID                      Pointer0;
            PVOID                      Pointer1;
        };
    };
};
 

struct WMI_TRACE_PACKET             // must be ULONG!!
{   
    USHORT  Size;
    union
    {
        USHORT  HookId;
        struct 
        {
            UCHAR   Type;
            UCHAR   Group;
        };
    };
};

typedef WMI_TRACE_PACKET* PWMI_TRACE_PACKET;

struct SYSTEM_TRACE_HEADER 
{
    union 
    {
        ULONG       Marker;
        struct 
        {
            USHORT  Version;
            UCHAR   HeaderType;
            UCHAR   Flags;
        };
    };
    union 
    {
        ULONG            Header;    // both sizes must be the same!
        WMI_TRACE_PACKET Packet;
    };
    ULONG           ThreadId;
    ULONG           ProcessId;
    LARGE_INTEGER   SystemTime;
    ULONG   KernelTime;
    ULONG   UserTime;
};

struct ETW_LOGFILE_HEADER 
{
    ETW_BUFFER_HEADER    BufferHeader;
    SYSTEM_TRACE_HEADER  SystemHeader;
    TRACE_LOGFILE_HEADER TraceLogfileHeader;
};

enum ETW_BUFFER_STATE
{
   EtwBufferStateFree,
   EtwBufferStateGeneralLogging,
   EtwBufferStateCSwitch,
   EtwBufferStateFlush,
   EtwBufferStateMaximum
};

#define EtwpGetLoggerContext(LoggerId) \
        (LoggerId < MAXLOGGERS) ? \
        EtwpLoggerContext[LoggerId] : INVALID_LOGGER_CONTEXT


typedef struct _ETW_LOGGER_HANDLE {
    BOOLEAN DereferenceAndLeave;
} ETW_LOGGER_HANDLE, *PETW_LOGGER_HANDLE;


typedef struct _ETW_BUFFER_HANDLE {
    ETW_BUFFER_HEADER* TraceBuffer;
    //PEX_FAST_REF BufferFastRef;
    ULONG EventOffset;
}ETW_BUFFER_HANDLE;

//Event Trace definition
typedef struct _EVENT_TRACE {
    EVENT_TRACE_HEADER      Header;             // Event trace header
    ULONG                   InstanceId;         // Instance Id of this event
    ULONG                   ParentInstanceId;   // Parent Instance Id.
    GUID                    ParentGuid;         // Parent Guid;
    PVOID                   MofData;            // Pointer to Variable Data
    ULONG                   MofLength;          // Variable Datablock Length
    union {
        ULONG               ClientContext;
        ETW_BUFFER_CONTEXT  BufferContext;
    };
} EVENT_TRACE, *PEVENT_TRACE;

