/*++

Abstract:
    Interface definition for the public interface of audio platform media extension.

--*/

#ifndef __rtcpal_apis_audio_extension__
#define __rtcpal_apis_audio_extension__

#include <Mfidl.h>
#include <string>

// only support C++ interfaces for media extension
#ifdef __cplusplus

const std::wstring c_ORTCAudioTestDevice_DefaultAudioDevice = L"ORTCAudioTestDevice_DefaultAudioDevice";
const std::wstring c_ORTCAudioTestDevice_DefaultCommunicationDevice = L"ORTCAudioTestDevice_DefaultCommunicationDevice";

typedef enum _RtcPalAudioMediaExtensionType_e
{
    RtcPalAudioMediaExtensionType_Start=0x10000, //start type for audio. Video type must be less than this value.
    RtcPalAudioMediaExtensionType_ORTCCustomDeviceManager,
    RtcPalAudioMediaExtensionType_ORTCTest,
    RtcPalAudioMediaExtensionType_End
} RtcPalAudioMediaExtensionType_e;

inline bool IsSupportedRtcPalAudioMediaExtensionType(int type)
{
    return ( (type > RtcPalAudioMediaExtensionType_Start) && (type < RtcPalAudioMediaExtensionType_End) );
}

/*
* Interface for device manager extension
*/
class IRtcPalAudioORTCCustomDeviceManagerExtension
{
public: 
    virtual ~IRtcPalAudioORTCCustomDeviceManagerExtension(){};

    virtual HRESULT AddDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId, 
        IMFMediaSource *pMFMediaSource) = 0; 

    virtual HRESULT AddDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId, 
        IMFAttributes* pRenderAttributes,
        IOrtcSourceStreamOnSampleCallback *pOnSampleCallback) = 0;
    
    virtual HRESULT RemoveDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId) = 0;
};

class IRtcPalAudioORTCTestExtension
{
public: 
    virtual ~IRtcPalAudioORTCTestExtension(){};

    virtual HRESULT AddTestDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId, 
        IMFMediaSource *pMFMediaSource) = 0; 

    virtual HRESULT AddTestDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId, 
        IMFAttributes* pRenderAttributes,
        IOrtcSourceStreamOnSampleCallback *pOnSampleCallback) = 0;
    
    virtual HRESULT RemoveTestDevice(
        RtcPalDeviceType_e eType, 
        const WCHAR* privateId) = 0;

    virtual HRESULT CreateTestMFMediaSource(
        IMFMediaSource** ppMFMediaSource)=0;

    virtual HRESULT CreateTestOnSampleCallback(
        IOrtcSourceStreamOnSampleCallback** ppOnSampleCallback)=0;

    virtual HRESULT CreateTestMFMediaSource(
        const WCHAR* privateId, //display name of the device as private Id
        IMFMediaSource** ppMFMediaSource)=0;

    virtual HRESULT CreateTestOnSampleCallback(
        const WCHAR* privateId, //display name of the device as private Id
        IOrtcSourceStreamOnSampleCallback** ppOnSampleCallback)=0;
    
};

#endif //__cplusplus

#endif  /*  __rtcpal_apis_audio_extension__*/






