#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_analyze.h

Abstract:

    PAL Video Analyzer interface

Specification:

    

--*/

#ifndef __RTCPAL_APIS_VIDEO_ANALYZE_H__
#define __RTCPAL_APIS_VIDEO_ANALYZE_H__


// =============================================================================
// Video Analyzer
// =============================================================================



// ----------------------------------------------------------------------------
// RtcPalVideoAnalyzerStart
// 
// Start the Analyzer with the given Config.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoAnalyzerStart(
    __in            HPALVIDEOANALYZER hVideoAnalyzer,
    __in            RtmVideoAnalyzerConfig_t* pConfig
);


// ----------------------------------------------------------------------------
// RtcPalVideoAnalyzerStop
// 
// Stop streaming data to the Analyzer. Analyzer may be destroyed without
// explicit call to RtcPalVideoAnalyzerStop.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoAnalyzerStop(
    __in            HPALVIDEOANALYZER hVideoAnalyzer
);


// -----------------------------------------------------------------------------
// RtcPalVideoAnalyzerGetOutput
// 
// Retrieve latest result from the Video Analyzer object.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoAnalyzerGetResult(
    __in            HPALVIDEOANALYZER hVideoAnalyzer,
    __out           RtmVideoAnalyzerResult_t* pResult
);


// -----------------------------------------------------------------------------
// RtcPalVideoAnalyzerGetStatistics
// 
// Query the Video Analyzer for real-time processing statistics.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoAnalyzerGetStatistics(
    __in            HPALVIDEOANALYZER hVideoAnalyzer,
    __out           RtcPalVideoProcessingStats_t* pStats
);


// -----------------------------------------------------------------------------
// RtcPalVideoAnalyzerResetStatistics
// 
// Reset the statistics of the Video Analyzer.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoAnalyzerResetStatistics(
    __in            HPALVIDEOANALYZER hVideoAnalyzer
);


// ----------------------------------------------------------------------------
// RtcPalVideoAnalyzerGetMetrics
// 
// Retrieve latest metrics from the Video Analyzer object.
// ----------------------------------------------------------------------------
HRESULT
RtcPalVideoAnalyzerGetMetrics(
    /* __in    */   HPALVIDEOANALYZER hVideoAnalyzer,
    /* __out   */   RtmVideoAnalyzerMetrics_t* pResult
);


#endif // __RTCPAL_APIS_VIDEO_ANALYZE_H__


