#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_encode.h

Abstract:

    PAL Video Encoder interface

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_ENCODE_H__
#define __RTCPAL_APIS_VIDEO_ENCODE_H__


// =============================================================================
// Video Encoder
// =============================================================================


// ----------------------------------------------------------------------------
// RtcPalVideoEncoderStart
// 
// Start the Encoder with the given MLE Config. This is the only way to 
// change streams, and apply changes to stream parameters that are not dynamic.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderStart(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __in            MLE_Config* pMLEConfig
);


// ----------------------------------------------------------------------------
// RtcPalVideoEncoderStop
// 
// Stop all the streams on the Encoder. Encoder may be destroyed without
// explicit call to RtcPalVideoEncoderStop.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderStop(
    __in            HPALVIDEOENCODER hVideoEncoder
);


// ----------------------------------------------------------------------------
// RtcPalVideoEncoderGetParameter
//
// Get an encoder parameter. Parameters are defined in 
// videodsp/codecs/inc/MLE_Types.h
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderGetParameter(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __in            uint32_t ulParamType, 
    __out           uint8_t* pbBlob,
    __inout         uint32_t* pulBlobSize
);


// ----------------------------------------------------------------------------
// RtcPalVideoEncoderSetParameter
//
// Set an encoder parameter. Parameters are defined in 
// videodsp/codecs/inc/MLE_Types.h
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderSetParameter(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __in            uint32_t ulParamType, 
    __in            uint8_t* pbBlob,
    __in            uint32_t ulBlobSize
);

// -----------------------------------------------------------------------------
// RtcPalVideoEncoderGetOutput
// 
// Retrieve encoded output from the Video Encoder object.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderGetOutput(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __out           MLE_EncodedUnit* pEncUnit
);


// -----------------------------------------------------------------------------
// RtcPalVideoEncoderGetStatistics
// 
// Query the Video Encoder for real-time processing statistics.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderGetStatistics(
    __in            HPALVIDEOENCODER hVideoEncoder,
    __out           RtcPalVideoEncoderProcessingStats_t* pStats
);


// -----------------------------------------------------------------------------
// RtcPalGetUiID
// 
// Get the codec uiID based on its type
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT  RtcPalGetUiID(  void * tEncodeType,UINT * uiID );

// -----------------------------------------------------------------------------
// RtcPalVideoEncoderResetStatistics
// 
// Reset the statistics of the Video Encoder.
// -----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoEncoderResetStatistics(
    __in            HPALVIDEOENCODER hVideoEncoder
);

#endif // __RTCPAL_APIS_VIDEO_ENCODE_H__


