#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_extension.h

Abstract:

    PAL Video to PAL Video Extension interface

    BUGBUG - RtcPalVideoExtension contains implementation specific details. This header files
    must be moved to PAL private folders once MLD work settles down ...

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_EXTENSION_H__
#define __RTCPAL_APIS_VIDEO_EXTENSION_H__

#include "rtcpal_apis_video_renderer.h"

// -----------------------------------------------------------------------------
// Video Extension class
//
// Base class used by the decoder and the video extension to facilitate 
// exchange of resources.
// -----------------------------------------------------------------------------
class RtcPalVideoExtension : public IRtcPalVideoRenderer
{
protected:

    // Constructor & destructor
    RtcPalVideoExtension(); 
    
    virtual ~RtcPalVideoExtension(); 
    virtual void CopyLastFrameInfo(const RtcPalVideoRawFrame_t* pFrame);
    virtual void InitializeLastFrameInfo();

    // common fields
    RTCPAL_SLIM_LOCK m_LastFrameLock;
    RtcPalVideoFrameResolution_t m_LastFrameInfo;
    
public:

   
    // class from context
    static HRESULT CreateInstance(
        const void *pvRenderContext,
        IRtcPalVideoRendererCallback * pCallback,
        RtcPalVideoExtension** ppVideoExtensionOut
    );
    
   
    // Calculate source and destination clipping rectangles for render operation
    static HRESULT ComputeClippingRectangles(
        const RECT rClient,
        uint16_t usFrameWidth,
        uint16_t usFrameHeight,
        uint16_t usOrgFrameWidth,
        uint16_t usOrgFrameHeight,
        const RtcPalVideoCropInfo_t * pCropInfo,
        const MMVR_ImageProcessingOption* pOptions,
        RECT* prSrc,
        RECT* prDst
    );
    
    // Calculate smart crop rect for a frame without valid smart crop info
    static HRESULT ComputeSmartCropRectFromCropType(
        uint16_t usOriginalWidth,
        uint16_t usOriginalHeight,
        RtcPalSmartCropType eSmartCropType,
        RtcPalVideoCropInfo_t * pResultCropInfo
    );

    // Enable/disable mirroring
    virtual HRESULT EnableMirroring(RTCPAL_BOOL) {return S_FALSE;}
    
    virtual void * GetLastFrameMetrics() { return NULL; }
    virtual HRESULT GetLastFrameInfo(__out RtcPalVideoFrameResolution_t * pLastFrameInfo);
    
    virtual HRESULT RenderBlankFrame() { return E_NOTIMPL; }
    virtual HRESULT GetPreferredSize(uint16_t* pusWidth, uint16_t* pusHeight)
    {
        UNREFERENCED_PARAMETER(pusWidth);
        UNREFERENCED_PARAMETER(pusHeight);
        return E_NOTIMPL;
    }

    virtual RTCPAL_BOOL CanSupportHardwareSample() const {
        return TRUE;
    }
};


#endif // __RTCPAL_APIS_VIDEO_EXTENSION_H__


