#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_apis_video_preview.h

Abstract:

    PAL Video Preview interface

Specification:

    http://livecommteam/sites/main/media/W15/Media%20Stack%20W15%20Plan%20Tracking/W15%20Video%20PAL%20Interface%20Design%20Doc.docx

--*/

#ifndef __RTCPAL_APIS_VIDEO_PREVIEW_H__
#define __RTCPAL_APIS_VIDEO_PREVIEW_H__


// =============================================================================
// Video Preview
// =============================================================================


// ----------------------------------------------------------------------------
// RtcPalVideoPreviewGetSupportedFormatList
// 
// Query a list of supported Video Formats for the Video Preview object. 
// If the array is of insufficient length, pulCount will return the number of 
// supported Video Formats but the array will not be populated.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPreviewGetSupportedFormatList(
    __in            HPALVIDEOPREVIEW hVideoPreview,
    __out           RtcPalVideoFormat_t* pVideoFormats,
    __inout         uint32_t* pulCount
);


// ----------------------------------------------------------------------------
// RtcPalVideoPreviewGetCurrentFormat
// 
// Query the current Video Format from the Video Preview object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPreviewGetCurrentFormat(
    __in            HPALVIDEOPREVIEW hVideoPreview,
    __out           RtcPalVideoFormat_t* pVideoFormat
);


// ----------------------------------------------------------------------------
// RtcPalVideoPreviewSetCurrentFormat
// 
// Set the current capture Video Format of the Video Preview object.
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPreviewSetCurrentFormat(
    __in            HPALVIDEOPREVIEW hVideoPreview,
    __in            RtcPalVideoFormat_t* pVideoFormat
);


// ----------------------------------------------------------------------------
// RtcPalVideoPreviewStart
// 
// Start Previewing
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPreviewStart(
    __in            HPALVIDEOPREVIEW hVideoPreview
);


// ----------------------------------------------------------------------------
// RtcPalVideoPreviewStop
// 
// Stop Previewing
// ----------------------------------------------------------------------------
RTC_CLASS_LINKAGE
HRESULT
RtcPalVideoPreviewStop(
    __in            HPALVIDEOPREVIEW hVideoPreview
);


#endif // __RTCPAL_APIS_VIDEO_PREVIEW_H__


