/*++

Abstract:
    Interface definitionas virutal devices.

--*/

#ifndef __rtcpal_apis_virutal_device
#define __rtcpal_apis_virutal_device

// at this layer, all interfaces are C-styled
#ifdef __cplusplus
extern "C"
{
#endif

typedef enum _RtcPalVirtualDeviceAcousticsProfile
{
    VirtualDeviceAcousticsProfile_NotSpecified	= 0,
    VirtualDeviceAcousticsProfile_Desktop 	= 1,
    VirtualDeviceAcousticsProfile_Slate 	= 2,
    VirtualDeviceAcousticsProfile_Laptop 	= 3
} RtcPalVirtualDeviceAcousticsProfile;

typedef enum _RtcPalVirutalDeviceOpenType
{
    VirutalDeviceNormalOpen,
    VirutalDeviceLoopBackOpen
} RtcPalVirutalDeviceOpenType;

typedef int32_t (*PRtcPalVadVolumeChangeCB)(void* pvContext, RtcPalDeviceVolumeInfo_t* pVolInfo);

typedef HRESULT (*PRtcPalVadActivateCB) (
    __in void * pDeviceContext , 
    __in void * pPalContext, 
    __in RtcPalEventHandle, 
    __out RVDAudioFormatDesc_t *, 
    __in RtcPalVirutalDeviceOpenType,
    __out void ** ppSessionContext);

typedef HRESULT (*PRtcPalVadPropertyChangeCB) (__in void *pPalContext, __in RVDDeviceInfo_t *pDeviceInfo);

// The following are per-session call-back functions.
// The first pointer is always the session object.
typedef HRESULT (*PRtcPalVadStartCB) (__in void * pSessionContext);
typedef HRESULT (*PRtcPalVadStopCB) (__in void *pSessionContext);
typedef HRESULT (*PRtcPalVadLockBufferCB) (__in void * pSessionContext, 
    RtcPalDeviceAudioBufferInfo_t *);

typedef HRESULT (*PRtcPalVadReleaseBufferCB) (__in void *pSessionContext, RtcPalDeviceAudioBufferInfo_t *);
typedef HRESULT (*PRtcPalVadRegisterCallBackCB)(__in void *pSessionContext, PRtcPalVadVolumeChangeCB);
typedef HRESULT (*PRtcPalVadUnregisterCallBackCB)(__in void *pSessionContext);

// End of Per-Session call-back.

// The following are per-device call-back functions.
// The first pointer is always the device object.
typedef HRESULT (*PRtcPalVadGetGlitchInfoCB) 
 ( __in void *pDeviceContext, uint32_t uiThreshold, uint64_t* pQPCPos, uint64_t* ui64Pos, int32_t* piPadding);
typedef HRESULT (*PRtcPalVadSetVolumeCB) (__in void *pDeviceContext, __inout RtcPalDeviceVolumeInfo_t* pvol);

typedef HRESULT (*PRtcPalVadGetVolumeCB) (__in void *pDeviceContext,  __inout RtcPalDeviceVolumeInfo_t* pvol, __out_opt uint64_t* pui64VolChangeTS);
typedef HRESULT (*PRtcPalVadGetQueuedBytesCB)(__in void *pDeviceContext, uint32_t* puiBytes);

typedef HRESULT (*PRtcPalVadGetCertifiedListDeviceIDCB)(__in void *pDeviceContext, __inout uint32_t* puVersion, __inout wchar_t* pwzUniqueDeviceId, __inout uint32_t* puSize);
typedef HRESULT (*PRtcPalVadGetAudioProcessingFeaturesCB) (__in void *pDeviceContext, __inout uint32_t* puFeatureFlags);
typedef HRESULT (*PRtcPalVadGetWritableProcessingFeaturesCB) (__in void *pDeviceContext, __inout uint32_t* puWritableFlags);

// The following are audio routing related call-back functions.
// They are optional.
typedef HRESULT (*PRtcPalVadSetActiveAudioEndpointCB)(__in void *pDeviceContext, __in RtcPalAudioEndpointType_e eAudioEndpoint);
typedef HRESULT (*PRtcPalVadRegisterPropertyChangeCB)(__in void *pDeviceContext, __in PRtcPalVadPropertyChangeCB pfnCallback, __in void *pPalContext);
typedef HRESULT (*PRtcPalVadUnregisterPropertyChangeCB)(__in void *pDeviceContext);
// End of Per-Device call-back.

typedef struct _RtcPalVadFuncTbl {
    PRtcPalVadActivateCB pfnActivateCB;
    PRtcPalVadStartCB pfnStartCB;
    PRtcPalVadStopCB pfnStopCB;
    PRtcPalVadLockBufferCB pfnLockBufferCB;
    PRtcPalVadReleaseBufferCB pfnReleaseBufferCB;
    PRtcPalVadGetGlitchInfoCB pfnGetGlitchInfoCB;
    PRtcPalVadSetVolumeCB pfnSetVolumeCB;
    PRtcPalVadGetVolumeCB pfnGetVolumeCB;
    PRtcPalVadGetQueuedBytesCB pfnGetQueuedBytesCB;
    PRtcPalVadRegisterCallBackCB pfnRegisterCallBackCB;
    PRtcPalVadUnregisterCallBackCB pfnUnregisterCallBackCB;
    PRtcPalVadSetActiveAudioEndpointCB pfnSetActiveAudioEndpointCB;
    PRtcPalVadRegisterPropertyChangeCB pfnRegisterPropertyChangeCB;
    PRtcPalVadUnregisterPropertyChangeCB pfnUnregisterPropertyChangeCB;
    PRtcPalVadGetCertifiedListDeviceIDCB pfnGetCertifiedListDeviceIDCB;
    PRtcPalVadGetAudioProcessingFeaturesCB pfnGetAudioProcessingFeaturesCB; 
    PRtcPalVadGetWritableProcessingFeaturesCB pfnGetWritableProcessingFeaturesCB; 
} RtcPalVadFuncPtrTbl;

extern
HRESULT
RtcPalCreateVirtualAudioDevice(
    __in void * pDeviceContext,
    __in RVDDeviceInfo_t * pDeviceInfo,
    __in RtcPalVadFuncPtrTbl * pFuncTbl,
    __out void ** ppPalContext,
    __in RtcPalVirtualDeviceAcousticsProfile eVirtualDeviceAcousticsProfile    
);

extern
HRESULT
RtcPalDeleteVirtualAudioDevice(
    __in void * pPalContext
);


#define IsRtcPalVirtualDevice(__pDeviceInfo) \
    ((__pDeviceInfo)->capabilities & RPDDEVICE_CAPABILITY_VIRTUALDEVICE) \


#define RtcPalSetVirtualDeviceFlag(__pDeviceInfo) \
    ((__pDeviceInfo)->capabilities |= RPDDEVICE_CAPABILITY_VIRTUALDEVICE) \



// at this layer, all interfaces are C-styled
#ifdef __cplusplus
}
#endif

#endif  /*  __rtcpal_apis_virtual_device __ */






