#pragma once

#include "rtcpal_private.h"

class RtcPalCriticalSection
{
public:
    RtcPalCriticalSection() { RtcPalInitializeCriticalSection(&m_section); }
    ~RtcPalCriticalSection() { RtcPalDeleteCriticalSection(&m_section); }
    
    // Manual locking/unlocking is not recommended: consider RtcPalAutoCriticalSection instead
    void Enter()
    {
        RtcPalEnterCriticalSection(&m_section);
    }
    void Leave()
    {
        RtcPalLeaveCriticalSection(&m_section);
    }

private:
    friend class RtcPalAutoCriticalSection;
    RTCPAL_CRITICAL_SECTION m_section;
};

class RtcPalAutoCriticalSection
{
    RTCPAL_CRITICAL_SECTION* m_pSection;
    
public:
    explicit RtcPalAutoCriticalSection(RtcPalCriticalSection& section) : m_pSection(&section.m_section)
    {
        RTCPAL_ASSERT(m_pSection != NULL);
        RtcPalEnterCriticalSection(m_pSection);
    }
    
    explicit RtcPalAutoCriticalSection(RTCPAL_CRITICAL_SECTION* pSection) : m_pSection(pSection)
    {
        RTCPAL_ASSERT(m_pSection != NULL);
        RtcPalEnterCriticalSection(m_pSection);
    }
    
    ~RtcPalAutoCriticalSection()
    {
        RtcPalLeaveCriticalSection(m_pSection);
    }
};
