/*++

Module Name:

    rtcpal_filesink.h

Abstract:

    Media Foundation's File Sink APIs.

Author:

Revision History:

--*/

#ifndef __RTCPAL_FILESINK_H__
#define __RTCPAL_FILESINK_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _RtcPalSourceReader RtcPalSourceReader;
typedef struct _RtcPalSinkWriter RtcPalSinkWriter;
typedef struct _RtcPalMediaType RtcPalMediaType;
typedef struct _RtcPalSample RtcPalSample;
typedef struct _RtcPalMediaBuffer RtcPalMediaBuffer;

typedef enum RTCPAL_SINKWRITER_CREATE_TYPE {
    RTCPAL_SINKWRITER_CREATE_TYPE_NO_STARTING,
    RTCPAL_SINKWRITER_CREATE_TYPE_STARTING
} RTCPAL_SINKWRITER_CREATE_TYPE;


typedef enum RTCPAL_SINKWRITER_CLOSE_TYPE {
    RTCPAL_SINKWRITER_CLOSE_TYPE_NO_FINALIZE,
    RTCPAL_SINKWRITER_CLOSE_TYPE_FINALIZE
} RTCPAL_SINKWRITER_CLOSE_TYPE;


//
// Must match MF_SOURCE_READER_FIRST_VIDEO_STREAM
//            MF_SOURCE_READER_FIRST_AUDIO_STREAM
//
typedef enum RTCPAL_SOURCE_READER_STREAM
{
    SOURCE_READER_FIRST_VIDEO_STREAM = 0xFFFFFFFC,
    SOURCE_READER_FIRST_AUDIO_STREAM = 0xFFFFFFFD    
} RTCPAL_SOURCE_READER_STREAM;


typedef enum RTCPAL_MEDIA_SUB_TYPE {
    RTCPAL_MEDIA_SUB_TYPE_WVC1 = 0,
    RTCPAL_MEDIA_SUB_TYPE_VC1S,
    RTCPAL_MEDIA_SUB_TYPE_H264,
    RTCPAL_MEDIA_SUB_TYPE_MAX, 
    RTCPAL_MEDIA_SUB_TYPE_UNDEFINED = RTCPAL_MEDIA_SUB_TYPE_MAX
} RTCPAL_MEDIA_SUB_TYPE;


typedef enum RTCPAL_INTERLACE_MODE {
    RTCPAL_INTERLACE_MODE_Unknown                       = 0,
    RTCPAL_INTERLACE_MODE_Progressive                   = 2,
    RTCPAL_INTERLACE_MODE_FieldInterleavedUpperFirst    = 3,
    RTCPAL_INTERLACE_MODE_FieldInterleavedLowerFirst    = 4,
    RTCPAL_INTERLACE_MODE_FieldSingleUpper              = 5,
    RTCPAL_INTERLACE_MODE_FieldSingleLower              = 6,
    RTCPAL_INTERLACE_MODE_MixedInterlaceOrProgressive   = 7        
}RTCPAL_INTERLACE_MODE;


typedef enum RTCPAL_SINKWRITER_MP4_OPTION
{
    RTCPAL_SINKWRITER_MP4_DEFAULT = 0,
    RTCPAL_SINKWRITER_MP4_FRAGMENTAL   
} RTCPAL_SINKWRITER_MP4_OPTION;

typedef struct _RtcPalSinkWriterFormat
{
    // Aspect Ratio
    uint32_t usNumerator;
    uint32_t usDenominator;
    RTCPAL_INTERLACE_MODE  interlaceMode;

    // blob
    uint8_t * pBlobBuf;
    uint32_t  usBlobBufSize;
    RTCPAL_SINKWRITER_MP4_OPTION mp4_option;
} RtcPalSinkWriterFormat;

extern
HRESULT 
RtcPalAVWriterStartup();

extern
void 
RtcPalAVWriterShutdown();



extern
HRESULT
RtcPalCheckFragmentalMp4Support(
    __out RTCPAL_BOOL * pfSupported
);



extern
HRESULT 
RtcPalCreateSinkWriterFromURL(
     __in const WCHAR* pszInputURL,
     __in RtcPalVideoFormat_t * pVideoFmt,
     __in RtcPalSinkWriterFormat * pSinkWriterFmt,
     __in RTCPAL_SINKWRITER_CREATE_TYPE type,     
     __out RtcPalSinkWriter ** ppSinkWriter);




extern
HRESULT
RtcPalSinkWriterClose(      
    __in  RtcPalSinkWriter * pSinkWriter,
    __in  RTCPAL_SINKWRITER_CLOSE_TYPE CloseType
);




extern 
UCHAR *
RtcPalGetLockedMediaBuffer(
    __in  DWORD size,
    __out RtcPalMediaBuffer ** ppLockedBufferHandle  
);


extern 
HRESULT
RtcPalFreeLockedMediaBuffer(
    __in RtcPalMediaBuffer * pLockedBufferHandle
);

extern
HRESULT
RtcPalWriteMediaBufferToSinkWriter(
    __in RtcPalSinkWriter * pWriter, 
    __in RtcPalMediaBuffer * pLockedBufferHdl,
    __in DWORD dBufferLen,
    __in uint64_t SampleDuration,
    __in uint64_t qwFileTimestamp, 
    __in RTCPAL_BOOL fKeyFrame, 
    __in RTCPAL_BOOL fDiscontinuity
);



#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_DATA_STRUCTURE_H__

