/*++

Abstract:

    This header file abstract the differences of compilers

    It defines the macros which work on most compilers. Then it includes
    compiler specific header files, e.g. rtcpal_msc.h for MSVC and rtcpal_gcc.h
    for GNUC. This gives compiler specific header file a chance for overriding
    the declarations.

Author:

Revision History:

--*/
#ifndef __RTCPAL_COMPILER_H__
#define __RTCPAL_COMPILER_H__

#define RTCPAL_CDECL
#define RTCPAL_STDCALL
#define RTCPAL_THISCALL
#define RTCPAL_FASTCALL

#define RTCPAL_NOTHROW
#define RTCPAL_NOTHROW_IMPL

#define RTCPAL_NOVTABLE

#define RTCPAL_SELECTANY
#define RTCPAL_DLL_EXPORT
#define RTCPAL_DLL_IMPORT
#define RTCPAL_DLL_LOCAL
#define RTCPAL_TRANSPARENT_UNION

#define RTCPAL_INLINE       inline
#define RTCPAL_FORCEINLINE  inline

// RTCPAL_LIKELY is a compiler hint which tells compiler that
// boolean expression (x) is likely true. This hint helps compiler
// produce better branch instruction
#define RTCPAL_LIKELY(x)    x

// RTCPAL_UNLIKELY is a compiler hint which tells compiler that
// boolean expression (x) is unlikely true
#define RTCPAL_UNLIKELY(x)  x

#define RTCPAL_FIELD_OFFSET(type, field)    ((LONG)(LONG_PTR)&(((type *)0)->field))

// C++ supports multiple typedefs for the same type. C doesn't support this but C supports
// extern. extern won't work if used in C++ class members. So only use extern for C.
#ifdef __cplusplus
#if __cplusplus >= 201103L
#define RTCPAL_C_ASSERT(e) static_assert(e, #e)
#else
#define RTCPAL_C_ASSERT(e)          typedef char __RTCPAL_C_ASSERT__[(e)?1:-1]
#endif // __cplusplus >= 201103L
#else
#define RTCPAL_C_ASSERT(e)          extern char __RTCPAL_C_ASSERT__[(e)?1:-1]
#endif

//#define RTCPAL_C_ASSERT(e)          typedef char __RTCPAL_C_ASSERT__[(e)?1:-1]
#define RTCPAL_C_ASSERT_POW2(x)     RTCPAL_C_ASSERT(((x) & (x - 1)) == 0)

#define RTCPAL_ROUND_UP_COUNT(Count,Pow2)   (((Count)+(Pow2)-1) & (~(((int32_t)(Pow2))-1)) )
#define RTCPAL_ROUND_UP_POINTER(Ptr,Pow2)   ((void *)((((uintptr_t)(Ptr))+(Pow2)-1) & (~(((int32_t)(Pow2))-1)) ) )

#ifdef __cplusplus
    template <class TYPE>
    class RtcPalAlignmentDetector {
    public:
        char x;
        TYPE test;
    };
    #define RTCPAL_TYPE_ALIGNMENT(t)  RTCPAL_FIELD_OFFSET(RtcPalAlignmentDetector<t>, test)
#else
    #define RTCPAL_TYPE_ALIGNMENT(t)  RTCPAL_FIELD_OFFSET(struct { char x; t test; }, test)
#endif

#define RTCPAL_ALIGNED(val)

#ifdef __cplusplus
    #define EXTERN_C    extern "C"
#else
    #define EXTERN_C    extern
#endif

#define RtcPalDebugBreak()	(void)raise(SIGINT)

#define RTCPAL_UNREFERENCED_PARAMETER(x)    {auto __temp = x;}

//
// Give compiler specific header file a chance for overriding the general definitions
//
#ifdef _MSC_VER
    #include <rtcpal_msc.h>
#elif defined(__GNUC__)
    #include <rtcpal_gcc.h>
#else
    #error "Unsupported compiler"
#endif

#endif // __RTCPAL_COMPILER_H__

