#ifndef __RTCPAL_CRYPT_H__
#define __RTCPAL_CRYPT_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef enum _RtcPalCryptoMode_e
{
    RtcPalCryptoMode_Start = 0,
    RtcPalCryptoMode_Default = RtcPalCryptoMode_Start,
    RtcPalCryptoMode_FIPS,
    RtcPalCryptoMode_HighPerf,      // Use high performance crypto module
    RtcPalCryptoMode_End
} RtcPalCryptoMode_e;

// RtcPalCryptoMode_Default
//      1) if FIPS policy is enabled or can't be queried from the OS.
//         Use FIPS certified module
//      2) if FIPS policy isn't enabled in the OS
//         PAL can choose crypto module
//
// RtcPalCryptoMode_FIPS
//      use FIPS certified crypto module
//
// RtcPalCryptoMode_HighPerf
//      use high performance crypto module
RTCPAL_ERROR
RTCAPI
RtcPalCryptStartup(
    RtcPalCryptoMode_e eMode
    );

void
RTCAPI
RtcPalCryptCleanup(
    void
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptGenRandom(
    __in uint32_t Len,
    __inout_bcount(Len) uint8_t * Buffer
    );

#define RTCPAL_SHA1_DIGEST_LEN      (20)
#define RTCPAL_SHA256_DIGEST_LEN    (32)
#define RTCPAL_SHA384_DIGEST_LEN    (48)
#define RTCPAL_SHA512_DIGEST_LEN    (64)

#include <pshpack1.h>
typedef struct RTCPAL_SHA1_DIGEST {
    uint8_t Digest[RTCPAL_SHA1_DIGEST_LEN];
} RTCPAL_SHA1_DIGEST, * PRTCPAL_SHA1_DIGEST;

typedef struct RTCPAL_SHA256_DIGEST {
    uint8_t Digest[RTCPAL_SHA256_DIGEST_LEN];
} RTCPAL_SHA256_DIGEST, * PRTCPAL_SHA256_DIGEST;

typedef struct RTCPAL_SHA384_DIGEST {
    uint8_t Digest[RTCPAL_SHA384_DIGEST_LEN];
} RTCPAL_SHA384_DIGEST, * PRTCPAL_SHA384_DIGEST;

typedef struct RTCPAL_SHA512_DIGEST {
    uint8_t Digest[RTCPAL_SHA512_DIGEST_LEN];
} RTCPAL_SHA512_DIGEST, * PRTCPAL_SHA512_DIGEST;
#include <poppack.h>

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA1CreateKey(
    __out_ecount(1) RTCPAL_HMACSHA1KEY * OpaqueKey,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Key
    );

void
RTCAPI
RtcPalCryptHMACSHA1DestroyKey(
    __in RTCPAL_HMACSHA1KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA1Create(
    __out_ecount(1) RTCPAL_HMACSHA1 * OpaqueHMAC,
    __in const RTCPAL_HMACSHA1KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA1Update(
    __in RTCPAL_HMACSHA1 * OpaqueHMAC,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA1Final(
    __in RTCPAL_HMACSHA1 * OpaqueHMAC,
    __out_ecount(1) PRTCPAL_SHA1_DIGEST Digest
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA1DupeContext(
    __in const RTCPAL_HMACSHA1 * OpaqueHMAC1,
    __inout RTCPAL_HMACSHA1 * OpaqueHMAC2
    );

void
RTCAPI
RtcPalCryptHMACSHA1Destroy(
    __in RTCPAL_HMACSHA1 * OpaqueHMAC
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA256CreateKey(
    __out_ecount(1) RTCPAL_HMACSHA256KEY * OpaqueKey,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Key
    );

void
RTCAPI
RtcPalCryptHMACSHA256DestroyKey(
    __in RTCPAL_HMACSHA256KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA256Create(
    __out_ecount(1) RTCPAL_HMACSHA256 * OpaqueHMAC,
    __in const RTCPAL_HMACSHA256KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA256Update(
    __in RTCPAL_HMACSHA256 * OpaqueHMAC,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA256Final(
    __in RTCPAL_HMACSHA256 * OpaqueHMAC,
    __out_ecount(1) PRTCPAL_SHA256_DIGEST Digest
    );


RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA256DupeContext(
    __in const RTCPAL_HMACSHA256 * OpaqueHMAC1,
    __inout RTCPAL_HMACSHA256 * OpaqueHMAC2
    );

void
RTCAPI
RtcPalCryptHMACSHA256Destroy(
    __in RTCPAL_HMACSHA256 * OpaqueHMAC
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA384CreateKey(
    __out_ecount(1) RTCPAL_HMACSHA384KEY * OpaqueKey,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Key
    );

void
RTCAPI
RtcPalCryptHMACSHA384DestroyKey(
    __in RTCPAL_HMACSHA384KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA384Create(
    __out_ecount(1) RTCPAL_HMACSHA384 * OpaqueHMAC,
    __in const RTCPAL_HMACSHA384KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA384Update(
    __in RTCPAL_HMACSHA384 * OpaqueHMAC,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA384Final(
    __in RTCPAL_HMACSHA384 * OpaqueHMAC,
    __out_ecount(1) PRTCPAL_SHA384_DIGEST Digest
    );


RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA384DupeContext(
    __in const RTCPAL_HMACSHA384 * OpaqueHMAC1,
    __inout RTCPAL_HMACSHA384 * OpaqueHMAC2
    );

void
RTCAPI
RtcPalCryptHMACSHA384Destroy(
    __in RTCPAL_HMACSHA384 * OpaqueHMAC
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA512CreateKey(
    __out_ecount(1) RTCPAL_HMACSHA512KEY * OpaqueKey,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Key
    );

void
RTCAPI
RtcPalCryptHMACSHA512DestroyKey(
    __in RTCPAL_HMACSHA512KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA512Create(
    __out_ecount(1) RTCPAL_HMACSHA512 * OpaqueHMAC,
    __in const RTCPAL_HMACSHA512KEY * OpaqueKey
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA512Update(
    __in RTCPAL_HMACSHA512 * OpaqueHMAC,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA512Final(
    __in RTCPAL_HMACSHA512 * OpaqueHMAC,
    __out_ecount(1) PRTCPAL_SHA512_DIGEST Digest
    );


RTCPAL_ERROR
RTCAPI
RtcPalCryptHMACSHA512DupeContext(
    __in const RTCPAL_HMACSHA512 * OpaqueHMAC1,
    __inout RTCPAL_HMACSHA512 * OpaqueHMAC2
    );

void
RTCAPI
RtcPalCryptHMACSHA512Destroy(
    __in RTCPAL_HMACSHA512 * OpaqueHMAC
    );

// Hash Algorithms

// SHA-256
RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA256Create(
    __out_ecount(1) RTCPAL_SHA256_CTX * Ctx
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA256Update(
    __in RTCPAL_SHA256_CTX * Ctx,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA256Final(
    __in RTCPAL_SHA256_CTX * Ctx,
    __out_ecount(1) PRTCPAL_SHA256_DIGEST Digest
    );

void
RTCAPI
RtcPalCryptSHA256Destroy(
    __in RTCPAL_SHA256_CTX * Ctx
    );

// SHA-384
RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA384Create(
    __out_ecount(1) RTCPAL_SHA384_CTX * Ctx
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA384Update(
    __in RTCPAL_SHA384_CTX * Ctx,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA384Final(
    __in RTCPAL_SHA384_CTX * Ctx,
    __out_ecount(1) PRTCPAL_SHA384_DIGEST Digest
    );

void
RTCAPI
RtcPalCryptSHA384Destroy(
    __in RTCPAL_SHA384_CTX * Ctx
    );

// SHA-512
RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA512Create(
    __out_ecount(1) RTCPAL_SHA512_CTX * Ctx
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA512Update(
    __in RTCPAL_SHA512_CTX * Ctx,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptSHA512Final(
    __in RTCPAL_SHA512_CTX * Ctx,
    __out_ecount(1) PRTCPAL_SHA512_DIGEST Digest
    );

void
RTCAPI
RtcPalCryptSHA512Destroy(
    __in RTCPAL_SHA512_CTX * Ctx
    );

// MD5 for backward compatibility
RTCPAL_ERROR
RTCAPI
RtcPalCryptMD5Create(
    __out_ecount(1) RTCPAL_MD5_CTX * OpaqueMD5
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptMD5Update(
    __in RTCPAL_MD5_CTX * OpaqueMD5,
    __in uint32_t Len,
    __in_bcount(Len) const uint8_t * Data
    );

#define RTCPAL_MD5_DIGEST_LEN   16

#include <pshpack1.h>
typedef struct RTCPAL_MD5_DIGEST {
    uint8_t Digest[RTCPAL_MD5_DIGEST_LEN];
} RTCPAL_MD5_DIGEST, * PRTCPAL_MD5_DIGEST;
#include <poppack.h>

RTCPAL_ERROR
RTCAPI
RtcPalCryptMD5Final(
    __in RTCPAL_MD5_CTX * OpaqueMD5,
    __out_ecount(1) PRTCPAL_MD5_DIGEST Digest
    );

void
RTCAPI
RtcPalCryptMD5Destroy(
    __in RTCPAL_MD5_CTX * OpaqueMD5
    );

struct RTCPAL_DATA_BLOB {
    uint32_t cbData;
    uint8_t * pbData;
};

#if RTCPAL_HAVE_MEMORY_ENCRYPTION
RTCPAL_ERROR
RTCAPI
RtcPalCryptProtectData(
    __in const struct RTCPAL_DATA_BLOB * Plain,
    __out struct RTCPAL_DATA_BLOB * Encrypted
    );

RTCPAL_ERROR
RTCAPI
RtcPalCryptUnprotectData(
    __in const struct RTCPAL_DATA_BLOB * Encrypted,
    __out struct RTCPAL_DATA_BLOB * Plain
    );
#endif // RTCPAL_HAVE_MEMORY_ENCRYPTION

// Marking this as required to check return code since we have places 
// in the media stack where we ignore the error code.
__checkReturn                           
RTCPAL_ERROR
RTCAPI
RtcPalSetAesKey(
    __in_bcount(cbKey) const uint8_t * pbKey,
    __in uint32_t cbKey,
    __inout RTCPAL_AESKEY* pAesKey
    );

#define RTCPAL_AES_BLOCK_SIZE   16

#include <pshpack1.h>
typedef struct RTCPAL_AES_BLOCK {
    uint8_t Block[RTCPAL_AES_BLOCK_SIZE];
} RTCPAL_AES_BLOCK, * PRTCPAL_AES_BLOCK;
#include <poppack.h>

// Make sure there is no padding in the struct. This is required
// for RtcPalAesEncrypt to take an array of struct.
RTCPAL_C_ASSERT(sizeof(RTCPAL_AES_BLOCK) == RTCPAL_AES_BLOCK_SIZE);

// limit the number of blocks that AES can run (to prevent integer overflow)
#define RTCPAL_AES_MAX_BLOCK_COUNT  65536

__checkReturn
RTCPAL_ERROR
RTCAPI
RtcPalAesEncrypt(
    __in RTCPAL_AESKEY* pAesKey,
    __in_range(1,RTCPAL_AES_MAX_BLOCK_COUNT) uint32_t BlkCnt,
    __in_ecount(BlkCnt) const RTCPAL_AES_BLOCK * pPlainText,
    __out_ecount(BlkCnt) RTCPAL_AES_BLOCK * pCipherText
    );

void
RTCAPI
RtcPalDestroyAesKey(
    __in RTCPAL_AESKEY* pAesKey
    );

void
RTCAPI
RtcPalSecureZeroMemory(
    __in void* pv,
    __in size_t cb
    );
//
// Free pbData in RTCPAL_DATA_BLOB whic is allocated by CryptProtectData
// and/or CrypteUnprotectData
//
void
RTCAPI
RtcPalCryptFree(
    __inout struct RTCPAL_DATA_BLOB * Blob
    );

typedef enum _RtcPalCryptoFIPSPolicy {
    RTCPAL_FIPS_POLICY_REQUIRED,
    RTCPAL_FIPS_POLICY_NOT_REQUIRED,
    RTCPAL_FIPS_POLICY_UNKNOWN
} RtcPalCryptoFIPSPolicy;

RtcPalCryptoFIPSPolicy
RTCAPI
RtcPalGetFipsComplianceMode(
    void
    );

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_CRYPT_H__
