/*++

Module Name:

    rtcpal_data_structure.h

Abstract:

    Basic data structures

Author:

Revision History:

--*/

#ifndef __RTCPAL_DATA_STRUCTURE_H__
#define __RTCPAL_DATA_STRUCTURE_H__

#ifdef __cplusplus
extern "C" {
#endif

// RTCPAL_TASK_QUEUE
// Compared to LIST_ENTRY type of queue,
// 1. Task queue atomic enqueue and dequeue.
// 2. The PAL implementation on Windows x86/x64 platform use IOCP.
//    Compared to alternative, IOCP-based implementation may be more
//    expensive in terms of CPU cycles, but it is more predictable.
//    Underneath, IOCP uses a NTOS kernel spinlock to protect the queue.
//    Kernel lock will raise IRQL to DPC which prevent the thread
//    from being preempted (thus avoid priority-inversion issue).

typedef struct RTCPAL_TASK_QUEUE * PRTCPAL_TASK_QUEUE;

PRTCPAL_TASK_QUEUE
RtcPalTaskQueueCreate(
    void
    );

void
RtcPalTaskQueueDestroy(
    __in PRTCPAL_TASK_QUEUE Queue
    );

RTCPAL_BOOL
RtcPalTaskQueueEnqueue(
    __in PRTCPAL_TASK_QUEUE Queue,
    __in uintptr_t TaskId,
    __in uintptr_t TaskParam
    );

typedef struct RTCPAL_TASK_QUEUE_ITEM {
    uintptr_t TaskId;
    uintptr_t TaskParam;

#if defined(_WIN32_WINNT)
    // Make sure layout of RTCPAL_TASK_QUEUE_ITEM is compatible with the layout of WINNT native OVERLAPPED_ENTRY
    uintptr_t dummy1;
    DWORD dummy2;
#endif
} RTCPAL_TASK_QUEUE_ITEM, * PRTCPAL_TASK_QUEUE_ITEM;


RTCPAL_BOOL
RtcPalTaskQueueDequeue(
    __in PRTCPAL_TASK_QUEUE Queue,
    __out_ecount_part(uCount, *uNumEntriesRemoved) PRTCPAL_TASK_QUEUE_ITEM pTaskEntries,
    __in  uint32_t uCount,
    __out uint32_t * uNumEntriesRemoved,
    __in  DWORD dwMilliseconds
    );

#ifdef __cplusplus
};
#endif

#endif // __RTCPAL_DATA_STRUCTURE_H__

