#ifndef _ENCODER_INTERFACE_H_
#define _ENCODER_INTERFACE_H_

#include "rtcpal_encoder_interface.h"
#include "rtcpal_decoder_Interface.h"

typedef enum _RtcPalDeviceVendor_e
{
    RtcPalDeviceVendor_Unknown,
    RtcPalDeviceVendor_NVIDIA,
    RtcPalDeviceVendor_QCOM,
    RtcPalDeviceVendor_TI,
    RtcPalDeviceVendor_AMD,
    RtcPalDeviceVendor_INTEL
} RtcPalDeviceVendor_e;

typedef enum _RtcPalSliceControlMode_e
{
    RtcPalSliceControlMode_None = -1,
    RtcPalSliceControlMode_Macroblocks,
    RtcPalSliceControlMode_Bits,
    RtcPalSliceControlMode_MacroblockRows
} RtcPalSliceControlMode_e;

typedef enum _RtcPalSliceControlModePreference_e
{
    RtcPalSliceControlModePreference_MacroblockRows,    // Least favored
    RtcPalSliceControlModePreference_Macroblocks,
    RtcPalSliceControlModePreference_Bytes
} RtcPalSliceControlModePreference_e;

struct RtcPalConstantQpParameters {    
    uint32_t MaxFrameBytes; // Maximum allowed frame bytes. 0 means no limit.
    uint32_t Qp;              // 0 means no change
};

struct RtcPalConstantBitRateParameters {
    uint32_t PeakBitRate;
    uint32_t AverageBitRate;
    uint32_t LeakyBucketSize;
    uint32_t PeakLeakyBucketSize;
    uint32_t IQp;
};

#define MAX_TEMPORAL_LAYER  3
typedef struct _RtcPalVideoStreamConfiguration {
    uint32_t EncodedWidth;
    uint32_t EncodedHeight;    
    uint32_t PriorityId; // H.264 only
    MLE_Profile Profile; // H.264 only - only two valid values (MLE_Profile_ConstrainedBaseline, MLE_Profile_UCConstrainedHigh)
    uint32_t FrameInterval; // minimum time interval between frames in 100ns.
    uint32_t NumOfTemporalEnhancementLayers; // number of enhancement layers (for VC-1, this indicates B frame).
    RtcPalConstantBitRateParameters rgRate[MAX_TEMPORAL_LAYER]; // rate for temporal layer, total number of valid layer is indicated by NumOfTemporalEnhancementLayers 
                                                                // (caveat - this is true for h.264, but perhaps not VC-1)
} RtcPalVideoStreamConfiguration;

struct IRtcPalVideoEncoderInterface
{
    virtual HRESULT DestroyEncoder () = 0;
     
    virtual HRESULT EncodeFrame (
        __in    IRtcPalVideoFrameWrapper* pFrame
        ) = 0;

    virtual HRESULT SetStreamConfiguration(__in const RtcPalVideoStreamConfiguration *pConfig) = 0;

    virtual HRESULT SetConstantQpParameters (
        __in uint32_t LayerId,
        __in_ecount(1) const RtcPalConstantQpParameters * pQpParams
        ) = 0;

    // Function Name: SetIDR
    // Function Description: This function is used to insert IDR dynamically
    // Return : HRESULT 
    // MLEInterface: Called from MLESetParameter after passing it to a queue for immediate return
    virtual HRESULT SetIDR () = 0;

    virtual HRESULT QuerySupportedInputFormats(uint32_t* pFormats, uint32_t* pulCount) = 0;
    virtual HRESULT SetInputVideoFormat(const RtcPalVideoFormat_t* pfmt) = 0;
    virtual HRESULT ReleaseOutputBuffer(
        __in RtcPalVideoEncodedUnit_t * pEncUnit
        ) = 0;

    virtual HRESULT GetHeader(__inout RtcPalVideoEncodedUnit_t * pEncUnit, __in uint32_t uiMaxHeaderLen) = 0;
};

struct IRtcPalVideoEncoderCallback
{
    virtual void Invoke(
        __in IRtcPalVideoEncoderInterface * pSender,
        __in RtcPalVideoEvent_e eEventType
        ) = 0;

    virtual HRESULT DeliverEncodedFrame(
        __in IRtcPalVideoEncoderInterface * pSender,
        __in RtcPalVideoEncodedUnit_t * pEncUnit
        ) = 0;
};

#endif
