/*++

Abstract:

    GNUC specific definitions

Author:

Revision History:

--*/
#ifndef __RTCPAL_GCC_H__
#define __RTCPAL_GCC_H__

// In order to fix undefined INTPTR_MAX macro under Android
#if defined(ANDROID) || defined(__linux)
#define __STDC_LIMIT_MACROS
#endif

#include <stddef.h>     // for offsetof
#include <stdint.h>     // for int32_t ...

/* don't use stdcall in gcc.
   In some case, it causes hard to debug stack corruption where both
   caller and callee unwind the stack.

#if __i386
    #undef RTCPAL_CDECL
    #define RTCPAL_CDECL            __attribute__((cdecl))

    #undef RTCPAL_STDCALL
    #define RTCPAL_STDCALL          __attribute__((stdcall))

    #undef RTCPAL_THISCALL
    #define RTCPAL_THISCALL         __attribute__((thiscall))

    #undef RTCPAL_FASTCALL
    #define RTCPAL_FASTCALL         __attribute__((fastcall))
#endif // __i386
*/

#undef RTCPAL_FIELD_OFFSET
#define RTCPAL_FIELD_OFFSET(type, field)    offsetof(type,field)

#undef RTCPAL_NOTHROW
#define RTCPAL_NOTHROW                      __attribute__((nothrow))

#undef RTCPAL_SELECTANY
#define RTCPAL_SELECTANY                    __attribute__((weak))

// See http://gcc.gnu.org/wiki/Visibility for what it is about
#if __GNUC__ >= 4
    #undef RTCPAL_DLL_EXPORT
    #define RTCPAL_DLL_EXPORT  __attribute__ ((visibility("default")))

    #undef RTCPAL_DLL_IMPORT
    #define RTCPAL_DLL_IMPORT  __attribute__ ((visibility("default")))

    #undef RTCPAL_DLL_LOCAL
    #define RTCPAL_DLL_LOCAL   __attribute__ ((visibility("hidden")))
#endif

#undef RTCPAL_TYPE_ALIGNMENT
#define RTCPAL_TYPE_ALIGNMENT(t)    __alignof__(t)

#undef RTCPAL_FORCEINLINE
#define RTCPAL_FORCEINLINE  inline __attribute__((always_inline))

#undef RTCPAL_LIKELY
#define RTCPAL_LIKELY(x)    __builtin_expect((x),1)

#undef RTCPAL_UNLIKELY
#define RTCPAL_UNLIKELY(x)  __builtin_expect((x),0)

#undef RTCPAL_ALIGNED
#define RTCPAL_ALIGNED(val) __attribute__((aligned(val)))

#if __ppc64__ || __ppc__
    #undef RtcPalDebugBreak
    #define RtcPalDebugBreak() \
        do { \
            __asm__("li r0, 20\nsc\nnop\nli r0, 37\nli r4, 2\nsc\nnop\n" : : : "memory","r0","r3","r4" ); \
        } while (0)
#elif __i386__ || __x86_64__
    #undef RtcPalDebugBreak
    #define RtcPalDebugBreak() \
        do { \
            __asm__("int $3\n" : : ); \
        } while (0)
#elif __arm__
    #undef RtcPalDebugBreak
    #define RtcPalDebugBreak() \
        do { \
            __asm__("mov r0, #20\nmov ip, r0\nsvc 128\nmov r1, #37\nmov ip, r1\nmov r1, #2\nmov r2, #1\n svc 128\n" : : : "memory","ip","r0","r1","r2"); \
        } while (0)
#endif // which architecture

#undef RTCPAL_TRANSPARENT_UNION
#define RTCPAL_TRANSPARENT_UNION __attribute__((transparent_union))

#undef RTCPAL_UNREFERENCED_PARAMETER
#define RTCPAL_UNREFERENCED_PARAMETER(x)    (void)(x)

#if !defined(__linux)
#ifndef __cplusplus
#undef RTCPAL_C_ASSERT
#define RTCPAL_C_ASSERT(e) extern int RTCPAL_C_ASSERT_dummy[(e)?1:-1]; _Pragma( "unused(RTCPAL_C_ASSERT_dummy)" )
#endif
#endif //__linux

// RTCPAL_ARCH_xxx are RTCPAL macros for CPU architectures
// _M_xxxx are MSC compiler's macros. Support for them is best-effort and will
// be removed when we run into conflicts.
#if __x86_64__ || __amd64__
    #define RTCPAL_ARCH_X64     100
    #define _M_X64      RTCPAL_ARCH_X64
    #define _M_AMD64    RTCPAL_ARCH_X64
#elif __i386__
    // Always assumes Pentium Pro and beyond
    #define RTCPAL_ARCH_X86     600

    // Don't define _M_IX86 until audio/video dsp is cleaned up
    //      Many codecs assume that they can use MSC inline assembler when _M_IX86 is defined.
    // #define _M_IX86 RTCPAL_ARCH_X86
#elif __arm__
    // 6 is the same value used by WINCE MSC ARM compiler.
    #define RTCPAL_ARCH_ARM     6
    #define _M_ARM  RTCPAL_ARCH_ARM
#else
    #error "Unknown CPU architecture"
#endif

#define _stdcall
#define _fastcall

#if __i386__ || __x86_64__
    // MMWORD was added to MASM in Visual C++ 2005.
    // MMWORD is intended to represent the same type as __m64.
    #define mmword	__m64
    #define MMWORD	__m64

    // XMMWORD was added to MASM in Visual C++ 2005.
    // XMMWORD is intended to represent the same type as __m128.
    #define xmmword	__m128
    #define XMMWORD	__m128
#endif

#endif // __RTCPAL_GCC_H__

