#ifndef __LIST_ENTRY_H__
#define __LIST_ENTRY_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef InitializeListHead

//
//  VOID
//  InitializeListHead(
//      PLIST_ENTRY ListHead
//      );
//

#define InitializeListHead(ListHead) (\
    (ListHead)->Flink = (ListHead)->Blink = (ListHead))

#ifdef __cplusplus
}
#endif


// use inline function instead of macro to reduce extra parentheses
// issues and errors.
static
RTCPAL_FORCEINLINE BOOLEAN
IsListEmpty(
    PLIST_ENTRY ListHead
)
{
    return ((ListHead)->Flink == (ListHead));
}

#ifdef __cplusplus
extern "C" {
#endif

//
//  PLIST_ENTRY
//  RemoveHeadList(
//      PLIST_ENTRY ListHead
//      );
//

#define RemoveHeadList(ListHead) \
    (ListHead)->Flink;  \
    {RemoveEntryList((ListHead)->Flink)}

//
//  PLIST_ENTRY
//  RemoveTailList(
//      PLIST_ENTRY ListHead
//      );
//

#define RemoveTailList(ListHead) \
    (ListHead)->Blink;\
        {RemoveEntryList((ListHead)->Blink)}

//
//  VOID
//  RemoveEntryList(
//      PLIST_ENTRY Entry
//      );
//

#define RemoveEntryList(Entry) {\
    PLIST_ENTRY _EX_Blink;\
    PLIST_ENTRY _EX_Flink;\
    _EX_Flink = (Entry)->Flink;\
    _EX_Blink = (Entry)->Blink;\
    _EX_Blink->Flink = _EX_Flink;\
    _EX_Flink->Blink = _EX_Blink;\
    }

//
//  VOID
//  InsertTailList(
//      PLIST_ENTRY ListHead,
//      PLIST_ENTRY Entry
//      );
//

#define InsertTailList(ListHead,Entry) {\
    PLIST_ENTRY _EX_Blink;\
    PLIST_ENTRY _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Blink = _EX_ListHead->Blink;\
    (Entry)->Flink = _EX_ListHead;\
    (Entry)->Blink = _EX_Blink;\
    _EX_Blink->Flink = (Entry);\
    _EX_ListHead->Blink = (Entry);\
    }

//
//  VOID
//  InsertHeadList(
//      PLIST_ENTRY ListHead,
//      PLIST_ENTRY Entry
//      );
//

#define InsertHeadList(ListHead,Entry) {\
    PLIST_ENTRY _EX_Flink;\
    PLIST_ENTRY _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Flink = _EX_ListHead->Flink;\
    (Entry)->Flink = _EX_Flink;\
    (Entry)->Blink = _EX_ListHead;\
    _EX_Flink->Blink = (Entry);\
    _EX_ListHead->Flink = (Entry);\
    }

#define MoveList(NewHead, OldHead)  { \
    InsertHeadList(OldHead, NewHead); \
    RemoveEntryList(OldHead); \
    InitializeListHead(OldHead); \
    }

#endif // InitializeListHead

static
RTCPAL_FORCEINLINE
uint32_t
RtcPalGetListLength(
    const LIST_ENTRY * pHead
    )
{
    const LIST_ENTRY * pEntry = pHead->Flink;
    uint32_t count = 0;

    while(pEntry != pHead) {
        count++;
        pEntry = pEntry->Flink;
    }

    return count;
}

#ifdef __cplusplus
};
#endif

#endif // __LIST_ENTRY_H__

