
#ifndef __RTCPAL_NETWORK_H__
#define __RTCPAL_NETWORK_H__

#ifdef __cplusplus
extern "C" {
#endif

uint32_t
RTCAPI
RtcPalNetStartup(
    VOID
    );

void
RTCAPI
RtcPalNetCleanup(
    VOID
    );


typedef struct RtcPalIOCP RtcPalIOCP;
typedef struct RtcPalSocket RtcPalSocket;

typedef struct RtcPalOverlapped {
    uintptr_t PalReserved[ASYNC_STATUS_PAL_RESERVED];
    uint32_t Status;
    union {
        uint32_t BytesTransferred;
        uint32_t NumberOfTraceRtRecords;
        RtcPalSocket * AcceptSocket;
    };
} RtcPalOverlapped;

typedef struct RtcPalOverlappedEntry {
    uintptr_t CompletionKey;
    RtcPalOverlapped * Overlapped;
    uintptr_t Internal;
    uint32_t NumberOfBytesTransferred;
} RtcPalOverlappedEntry;

typedef struct RtcPalNetBuf {
    uint32_t len;
    __field_bcount(len) CHAR * buf;
} RtcPalNetBuf;

#ifndef SOCKET_ERROR
#define SOCKET_ERROR    (-1)
#endif

enum RtcPalSocketType {
    RtcPalSocket_Udp = 0,
    RtcPalSocket_Tcp,
    RtcPalSocket_TcpListen,

    // A special socket for trace route
    RtcPalSocket_TraceRt
};

extern
struct RtcPalSocket *
RTCAPI
RtcPalCreateSocket(
    int af,
    enum RtcPalSocketType type
    );

extern
int
RTCAPI
RtcPalBind(
    __in struct RtcPalSocket * s,
    __in_bcount(namelen) const struct sockaddr * name,
    __in int namelen
    );

extern
int
RTCAPI
RtcPalGetSockName(
    __in struct RtcPalSocket * s,
    __out_bcount_part(*namelen,*namelen) struct sockaddr FAR * name,
    __inout int FAR * namelen
    );

extern
int
RTCAPI
RtcPalGetPeerName(
    __in struct RtcPalSocket * s,
    __out_bcount_part(*namelen,*namelen) struct sockaddr FAR * name,
    __inout int FAR * namelen
    );

extern
int
RTCAPI
RtcPalConnect(
    __in struct RtcPalSocket * s,
    __in_bcount(namelen) const struct sockaddr * name,
    __in int namelen,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
RtcPalConnectToHostname(
    __in struct RtcPalSocket * s,
    __in __nullterminated const WCHAR * hostname,
    __in USHORT port,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
HRESULT
RTCAPI
RtcPalIsAutomaticProxyTraversalSupported(
    __out RTCPAL_BOOL * pfSupported
    );

extern
int
RTCAPI
RtcPalListen(
    __in struct RtcPalSocket * s,
    __in int backlog
    );

extern
struct RtcPalSocket *
RTCAPI
RtcPalAccept(
    __in struct RtcPalSocket * s,
    __out_bcount_opt(*addrlen) struct sockaddr * addr,
    __inout_opt int * addrlen,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
void
RTCAPI
RtcPalCloseSocket(
    struct RtcPalSocket * s,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
rtcpal_send(
    __in struct RtcPalSocket * s,
    __in_bcount(len) const char FAR * buf,
    __in int len
    );

extern
int
RTCAPI
rtcpal_sendto(
    __in struct RtcPalSocket * s,
    __in_bcount(len) const char FAR * buf,
    __in int len,
    __in_bcount(tolen) const struct sockaddr FAR * to,
    __in int tolen
    );

#define RTCPAL_MAX_VECTOR_BUFFER    8
extern
int
RTCAPI
RtcPalSend(
    __in struct RtcPalSocket * s,
    __in_ecount(BufferCount) const RtcPalNetBuf * Buffers,
    __in_range(1,RTCPAL_MAX_VECTOR_BUFFER) uint32_t BufferCount,
    __out_opt uint32_t * NumberOfBytesSent,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
RtcPalSendTo(
    __in struct RtcPalSocket * s,
    __in_ecount(BufferCount) const RtcPalNetBuf * Buffers,
    __in_range(1,RTCPAL_MAX_VECTOR_BUFFER) uint32_t BufferCount,
    __out_opt uint32_t * NumberOfBytesSent,
    __in_bcount_opt(Tolen) const struct sockaddr FAR * To,
    __in int Tolen,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
rtcpal_recv(
    __in struct RtcPalSocket * s,
    __out_bcount_part(len, return) __out_data_source(NETWORK) char FAR * buf,
    __in int len
    );

extern
int
RTCAPI
rtcpal_recvfrom(
    __in struct RtcPalSocket * s,
    __out_bcount_part(len, return) __out_data_source(NETWORK) char FAR * buf,
    __in int len,
    __out_bcount_part_opt(*fromlen, *fromlen) struct sockaddr FAR * from,
    __inout_opt int FAR * fromlen
    );

extern
int
RTCAPI
RtcPalRecv(
    __in struct RtcPalSocket * s,
    __out_bcount_part(len, *NumberOfBytesRecvd) __out_data_source(NETWORK) char FAR * buf,
    __in int len,
    __out_opt uint32_t * NumberOfBytesRecvd,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
RtcPalRecvFrom(
    __in struct RtcPalSocket * s,
    __out_bcount_part(len, *NumberOfBytesRecvd) __out_data_source(NETWORK) char FAR * buf,
    __in int len,
    __out_opt uint32_t * NumberOfBytesRecvd,
    __out_bcount_part_opt(*Fromlen,*Fromlen) struct sockaddr FAR * From,
    __inout_opt LPINT Fromlen,
    __inout_opt RtcPalOverlapped * Overlapped
    );

struct RtcPalTraceRtEntry {
    // Roundtrip latency in milliseconds
    // If this value is INFINITE, the corresponding hop
    // hasn't replied.
    uint32_t Latency;

    // Address of the replying hop
    // This field is valid only when Latency is not INFINITE
    struct sockaddr_in6 Address;
};

// This API is supported only with RtcPalSocket_TraceRt type
extern
int
RTCAPI
RtcPalTraceRt(
    __in struct RtcPalSocket * s,
    __in uint32_t NumberOfRecords,
    __out_ecount(NumberOfRecords) struct RtcPalTraceRtEntry * pRecords,
    __in_bcount(Tolen) const struct sockaddr FAR * To,
    __in int Tolen,
    __inout_opt RtcPalOverlapped * Overlapped
    );

extern
int
RTCAPI
RtcPalSocketSetSendBufSize(
    __in struct RtcPalSocket * s,
    uint32_t sndbuf
    );

extern
int
RTCAPI
RtcPalSocketSetRecvBufSize(
    __in struct RtcPalSocket * s,
    uint32_t rcvbuf
    );

extern
int
RTCAPI
RtcPalSocketSetLinger(
    __in struct RtcPalSocket * s,
    uint16_t Linger
    );

extern
int
RTCAPI
RtcPalSocketSetNonBlockingMode(
    __in struct RtcPalSocket * s,
    RTCPAL_BOOL Enabled
    );

extern
int
RTCAPI
RtcPalSocketSetBroadcast(
    __in struct RtcPalSocket * s,
    RTCPAL_BOOL Enabled
    );

extern
int
RTCAPI
RtcPalSocketSetTcpNoDelay(
    __in struct RtcPalSocket * s,
    RTCPAL_BOOL Enabled
    );

extern
int
RTCAPI
RtcPalSocketSetQoSDSCP(
    __in struct RtcPalSocket * s,
    uint8_t DSCP
    );

// Enable low latency mode on socket (for WINRT only)
// Low Latency mode, once enabled, can't be disabled!
extern
int
RTCAPI
RtcPalSocketEnableLowLatency(
    __in struct RtcPalSocket * s
    );

extern
RtcPalIOCP *
RTCAPI
RtcPalCreateSocketIOCP(
    __in uint32_t NumberOfConcurrentThreads
    );

extern
void
RTCAPI
RtcPalCloseSocketIOCP(
    __in struct RtcPalIOCP * IOCP
    );

extern
RTCPAL_BOOL
RTCAPI
RtcPalRegisterSocket(
    struct RtcPalIOCP * IOCP,
    struct RtcPalSocket * sd,
    uintptr_t CompletionKey
    );

extern
RTCPAL_BOOL
RTCAPI
RtcPalPostSocketIOCPQueuedCompletionStatus(
    __in struct RtcPalIOCP * IOCP,
    __in uint32_t NumberOfBytesTransferred,
    __in uintptr_t dwCompletionKey,
    __in_opt RtcPalOverlapped * Overlapped
    );

extern
RTCPAL_BOOL
RTCAPI
RtcPalGetSocketIOCPQueuedCompletionStatus(
    __in struct RtcPalIOCP * IOCP,
    __out_ecount_part(uCount, *uNumEntriesRemoved) RtcPalOverlappedEntry * CompletionPortEntries,
    __in uint32_t uCount,
    __out uint32_t * uNumEntriesRemoved
    );

uint32_t
RTCAPI
RtcPalNetAddressToStringA(
    __in_bcount(namelen) const struct sockaddr * name,
    __in int namelen,
    __out_ecount_part(*AddressStringLength,*AddressStringLength) CHAR * AddressString,
    __inout uint32_t * AddressStringLength
    );

uint32_t
RTCAPI
RtcPalNetAddressToStringW(
    __in_bcount(namelen) const struct sockaddr * name,
    __in int namelen,
    __out_ecount_part(*AddressStringLength,*AddressStringLength) WCHAR * AddressString,
    __inout uint32_t * AddressStringLength
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPv4AddressA(
    __in __nullterminated const char * string,
    __out struct sockaddr_in * addr
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPv4AddressW(
    __in __nullterminated const WCHAR * string,
    __out struct sockaddr_in * addr
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPv6AddressA(
    __in __nullterminated const char * string,
    __out struct sockaddr_in6 * addr
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPv6AddressW(
    __in __nullterminated const WCHAR * string,
    __out struct sockaddr_in6 * addr
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPAddressA(
    __in __nullterminated const char * string,
    __out_bcount(addrlen) struct sockaddr * addr,
    __in size_t addrlen
    );

RTCPAL_ERROR
RTCAPI
RtcPalNetStringToIPAddressW(
    __in __nullterminated const WCHAR * string,
    __out_bcount(addrlen) struct sockaddr * addr,
    __in size_t addrlen
    );

#define RTCPAL_UNKNOWN_PREFIX_LENGTH    0xffu
struct RtcPalUnicastIPAddress;
struct RtcPalUnicastIPAddress {
    struct RtcPalUnicastIPAddress * Next;
    uint32_t AddressFamily;
    union {
        struct in_addr v4;
        struct in6_addr v6;
    } IPAddress;

    uint8_t PrefixLength;
};

// Here are a few commonly used IfType. For complete assignment,
// see http://www.iana.org/assignments/ianaiftype-mib
enum RtcPalIfType {
    RtcPalIfTypeOther = 1,
    RtcPalIfTypeEthernetCSMACD = 6,         // Ethernet
    RtcPalIfTypePPP = 23,                   // VPN
    RtcPalIfTypeSoftwareLoopback = 24,      // Loopback Pseudo-Interface (127.0.0.1 and ::1 etc)
    RtcPalIfTypeIEEE80211 = 71,             // WiFi
    RtcPalIfTypeTunnel = 131,               // Tunnel type encapsulation interface
    RtcPalIfTypeIEEE80216_WMAN = 237,       // WiMax
    RtcPalIfTypeWWANPP = 243,               // GSM WWan
    RtcPalIfTypeWWANPP2 = 244               // CDMA WWan
};

enum RtcPalTunnelType {
    RtcPalTunnelTypeNone = 0,
    RtcPalTunnelTypeOther = 1,
    RtcPalTunnelTypeDirect = 2,
    RtcPalTunnelType6to4 = 11,
    RtcPalTunnelTypeISATAP = 13,
    RtcPalTunnelTypeTeredo = 14
};

enum RtcPalCategory {
    RtcPalCategoryUnknown = 0,
    RtcPalCategoryPublic,
    RtcPalCategoryPrivate,
    RtcPalCategoryDomainAuthenticated
};

#define RTCPAL_PHYSICAL_ADDRESS_MAX_LENGTH  16
#define RTCPAL_UNKNOWN_LINK_SPEED           0ll
#define RTCPAL_NETWORK_NAME_MAX_LENGTH      16

typedef GUID RTCPAL_NETWORK_ID;
struct RtcPalIPAdapterAddresses;
struct RtcPalIPAdapterAddresses {
    struct RtcPalIPAdapterAddresses * Next;
    uint32_t IfIndex;
    enum RtcPalIfType IfType;
    const char * IfName;
    const WCHAR * FriendlyName;
    uint32_t PhysicalAddressLength;
    uint8_t PhysicalAddress[RTCPAL_PHYSICAL_ADDRESS_MAX_LENGTH];
    uint64_t TransmitLinkSpeed;
    uint64_t ReceiveLinkSpeed;
    enum RtcPalTunnelType TunnelType;

    RTCPAL_NETWORK_ID NetworkId;
    WCHAR NetworkName[RTCPAL_NETWORK_NAME_MAX_LENGTH];
    enum RtcPalCategory NetworkCategory;

    // for WiFi adapter only
    uint8_t BSSID[6];

    struct RtcPalUnicastIPAddress * FirstUnicastAddress;
};

uint32_t
RTCAPI
RtcPalGetIfAddrs(
    __in uint32_t AddressFamily,
    __out struct RtcPalIPAdapterAddresses ** iaa
    );

void
RTCAPI
RtcPalFreeIfAddrs(
    __in struct RtcPalIPAdapterAddresses * iaa
    );

int
RTCAPI
RtcPalGetHostCName(
    __out_ecount(namelen) char * name,
    __in int namelen
    );

#define RtcPalGetAddrInfo   getaddrinfo
#define RtcPalFreeAddrInfo  freeaddrinfo
#define rtcpal_getaddrinfo  getaddrinfo
#define rtcpal_freeaddrinfo freeaddrinfo

#if !defined(_WIN32_WINNT)
    typedef struct addrinfoW {
        int	ai_flags;	/* AI_PASSIVE, AI_CANONNAME, AI_NUMERICHOST */
        int	ai_family;	/* PF_xxx */
        int	ai_socktype;	/* SOCK_xxx */
        int	ai_protocol;	/* 0 or IPPROTO_xxx for IPv4 and IPv6 */
        socklen_t ai_addrlen;	/* length of ai_addr */
        WCHAR	*ai_canonname;	/* canonical name for hostname */
        struct	sockaddr *ai_addr;	/* binary address */
        struct	addrinfoW *ai_next;	/* next structure in linked list */
    } ADDRINFOW, * PADDRINFOW;

    int RtcPalGetAddrInfoW(const WCHAR *, const WCHAR *,
			    const struct addrinfoW *,
			    struct addrinfoW **);
    void RtcPalFreeAddrInfoW(struct addrinfoW *);
    #define GetAddrInfoW RtcPalGetAddrInfoW
    #define FreeAddrInfoW RtcPalFreeAddrInfoW
#endif

uint32_t
RTCAPI
RtcPalGetBestSourceAddress(
    __in_ecount(1) const struct sockaddr * PeerAddr,
    __out_bcount_part(*BestSourceLen,*BestSourceLen) struct sockaddr * BestSource,
    __inout int * BestSourceLen
    );

static
RTCPAL_FORCEINLINE
BOOLEAN
RtcPalIN6AddrEqual(const struct in6_addr *x, const struct in6_addr *y)
{
    return memcmp(x, y, 16) == 0;
}

static
RTCPAL_FORCEINLINE
BOOLEAN
RtcPalIN6AddrIsEqual(const struct sockaddr_in6 *a, const struct sockaddr_in6 *b)
{
    RTCPAL_ASSERT(a->sin6_family == AF_INET6);
    return (BOOLEAN)(a->sin6_scope_id == b->sin6_scope_id &&
                     RtcPalIN6AddrEqual(&a->sin6_addr, &b->sin6_addr));
}

static
RTCPAL_FORCEINLINE
BOOLEAN
RtcPalIN4AddrEqual(const struct in_addr *a, const struct in_addr *b)
{
    return (BOOLEAN)(a->s_addr == b->s_addr);
}

static
RTCPAL_FORCEINLINE
BOOLEAN
RtcPalIN4AddrIsEqual(const struct sockaddr_in *a, const struct sockaddr_in *b)
{
    RTCPAL_ASSERT(a->sin_family == AF_INET);
    return (BOOLEAN)(RtcPalIN4AddrEqual(&a->sin_addr, &b->sin_addr));
}

static
RTCPAL_FORCEINLINE
BOOLEAN
RtcPalInetAddrIsEqual(const struct sockaddr * a, const struct sockaddr *b)
{
    if (a->sa_family == AF_INET6) {
        return (BOOLEAN)
            (b->sa_family == AF_INET6 &&
             RtcPalIN6AddrIsEqual((const struct sockaddr_in6*)a, (const struct sockaddr_in6*)b));
    } else {
	    RTCPAL_ASSERT(a->sa_family == AF_INET);
        return (BOOLEAN)
            (b->sa_family == AF_INET &&
             RtcPalIN4AddrIsEqual((const struct sockaddr_in*)a, (const struct sockaddr_in*)b));
    }
}

#if RTCPAL_HAVE_WLANAPI
RTCPAL_BOOL
RtcPalWlanEnterMediaStreamingMode(
    __in __nullterminated const char * szIfName,
    __out_ecount(1) HANDLE * phStreamingModeSession
    );

void
RtcPalWlanLeaveMediaStreamingMode(
    __in_ecount(1) HANDLE hStreamingModeSession
    );
#endif // RTCPAL_HAVE_WLANAPI

#ifdef __cplusplus
};
#endif

// The following functions are C++ only as they require C++ overloading
#ifdef __cplusplus

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsAnyIPv4Address(
    const struct sockaddr_in * addr
    )
{
    return (addr->sin_family == AF_INET) && (addr->sin_addr.s_addr == htonl(INADDR_ANY));
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsAnyIPv6Address(
    const struct sockaddr_in6 * addr
    )
{
    return (addr->sin6_family == AF_INET6) && 
         (addr->sin6_addr.s6_words[0] == 0) &&
         (addr->sin6_addr.s6_words[1] == 0) &&
         (addr->sin6_addr.s6_words[2] == 0) &&
         (addr->sin6_addr.s6_words[3] == 0) &&
         (addr->sin6_addr.s6_words[4] == 0) &&
         (addr->sin6_addr.s6_words[5] == 0) &&
         (addr->sin6_addr.s6_words[6] == 0) &&
         (addr->sin6_addr.s6_words[7] == 0);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsAnyIPAddress(
    const struct sockaddr * addr
    )
{
    if (addr->sa_family == AF_INET) {
        return RtcPalIsAnyIPv4Address((const struct sockaddr_in *)addr);
    } else if (addr->sa_family == AF_INET6) {
        return RtcPalIsAnyIPv6Address((const struct sockaddr_in6 *)addr);
    } else {
        return FALSE;
    }
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLoopbackIPv4Address(
    const struct in_addr addr
    )
{
    return (addr.s_addr == htonl(INADDR_LOOPBACK));
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLoopbackIPv4Address(
    const struct sockaddr_in * addr
    )
{
    return (addr->sin_family == AF_INET) && RtcPalIsLoopbackIPv4Address(addr->sin_addr);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLoopbackIPv6Address(
    const struct in6_addr * addr
    )
{
    return (addr->s6_words[0] == 0) &&
         (addr->s6_words[1] == 0) &&
         (addr->s6_words[2] == 0) &&
         (addr->s6_words[3] == 0) &&
         (addr->s6_words[4] == 0) &&
         (addr->s6_words[5] == 0) &&
         (addr->s6_words[6] == 0) &&
         (addr->s6_bytes[14] == 0) &&
         (addr->s6_bytes[15] == 1);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLoopbackIPv6Address(
    const struct sockaddr_in6 * addr
    )
{
    return (addr->sin6_family == AF_INET6) && RtcPalIsLoopbackIPv6Address(&addr->sin6_addr);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsIPLoopbackAddress(
    const struct sockaddr * addr
    )
{
    if (addr->sa_family == AF_INET) {
        return RtcPalIsLoopbackIPv4Address((const struct sockaddr_in *)addr);
    } else if (addr->sa_family == AF_INET6) {
        return RtcPalIsLoopbackIPv6Address((const struct sockaddr_in6 *)addr);
    } else {
        return FALSE;
    }
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsBroadcastIPv4Address(
    const struct in_addr addr
    )
{
    return (addr.s_addr == htonl(INADDR_BROADCAST));
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsMulticastIPv4Address(
    const struct in_addr addr
    )
{
    const uint8_t * ipaddr = (const uint8_t *)&addr;
    return ((ipaddr[0] & 0xf0) == 0xe0);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsMulticastIPv6Address(
    const struct in6_addr * addr
    )
{
    return (addr->s6_bytes[0] == 0xff);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLinkLocalIPv4Address(
    const struct in_addr addr
    )
{
    const uint8_t * ipaddr = (const uint8_t *)&addr;
    return (ipaddr[0] == 169) && (ipaddr[1] == 254);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLinkLocalIPv4Address(
    const struct sockaddr_in * addr
    )
{
    return (addr->sin_family == AF_INET) && RtcPalIsLinkLocalIPv4Address(addr->sin_addr);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLinkLocalIPv6Address(
    const struct in6_addr * addr
    )
{
    return (addr->s6_bytes[0] == 0xfe) && (addr->s6_bytes[1] == 0x80);
}


RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsLinkLocalIPv6Address(
    const struct sockaddr_in6 * addr
    )
{
    return (addr->sin6_family == AF_INET6) && RtcPalIsLinkLocalIPv6Address(&addr->sin6_addr);
}

RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsIPLinkLocalAddress(
    const struct sockaddr * addr
    )
{
    if (addr->sa_family == AF_INET) {
        return RtcPalIsLinkLocalIPv4Address((const struct sockaddr_in *)addr);
    } else if (addr->sa_family == AF_INET6) {
        return RtcPalIsLinkLocalIPv6Address((const struct sockaddr_in6 *)addr);
    } else {
        return FALSE;
    }
}

uint16_t
FORCEINLINE
RTCAPI
RtcPalGetPort(
    __in_ecount(1) const struct sockaddr * addr
    )
{
    if (addr->sa_family == AF_INET) {
        return ntohs(((struct sockaddr_in *)addr)->sin_port);
    } else if (addr->sa_family == AF_INET6) {
        return ntohs(((struct sockaddr_in6 *)addr)->sin6_port);
    } else {
        RTCPAL_ASSERT(0);
        return 0;
    }
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv4AnyAddress(
    __in_ecount(1) struct sockaddr_in * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin_len = 0;
#endif
    addr->sin_family = AF_INET;
    addr->sin_addr.s_addr = htonl(INADDR_ANY);
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv6AnyAddress(
    __in_ecount(1) struct sockaddr_in6 * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin6_len = 0;
#endif

    addr->sin6_family = AF_INET6;
    memset(addr->sin6_addr.s6_words, 0, 16);
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv4LoopbackAddress(
    __out_ecount(1) struct sockaddr_in * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin_len = 0;
#endif
    addr->sin_family = AF_INET;
    addr->sin_addr.s_addr = htonl(INADDR_LOOPBACK);
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv6LoopbackAddress(
    __out_ecount(1) SOCKADDR_IN6 * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin6_len = 0;
#endif
    addr->sin6_family = AF_INET6;
    memset(addr->sin6_addr.s6_words, 0, 16);
    addr->sin6_addr.s6_bytes[15] = 1;
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv4BroadcastAddress(
    __out_ecount(1) struct sockaddr_in * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin_len = 0;
#endif
    addr->sin_family = AF_INET;
    addr->sin_addr.s_addr = htonl(INADDR_BROADCAST);
}

void
FORCEINLINE
RTCAPI
RtcPalSetIPv6LocalAllNodes(
    __out_ecount(1) struct sockaddr_in6 * addr
    )
{
#ifdef _UNIX_MACOSX
    addr->sin6_len = 0;
#endif
    addr->sin6_family = AF_INET6;
    memset(addr->sin6_addr.s6_words, 0, 16);

    // ff02::1      all nodes on the local network segment
    addr->sin6_addr.s6_bytes[0] = 0xff;
    addr->sin6_addr.s6_bytes[1] = 0x02;
    addr->sin6_addr.s6_bytes[15] = 0x01;
}


RTCPAL_BOOL
FORCEINLINE
RTCAPI
RtcPalIsIPv6ISATAP(
    const struct in6_addr * addr
    )
{
#if defined(RTCPAL_LITTLE_ENDIAN)
    if (((addr->s6_words[4] & 0xfffd) != 0) || (addr->s6_words[5] != 0xfe5e)) {
        return FALSE;
    }
#elif defined(RTCPAL_BIG_ENDIAN)
    if (((addr->s6_words[4] & 0xfdff) != 0) || (addr->s6_words[5] != 0x5efe)) {
        return FALSE;
    }
#else
    #error "unsupported endian"
#endif

    return TRUE;
}

#endif  // __cplusplus

#endif // __RTCPAL_NETWORK_H__

