// To reduce build time, we don't run configure perl script at build time
// for detecting build environment. Unfortunately, this also means that
// we can't rely on rtcpal_autoconfig.h to tell us the target platform.
//
// Platform macro will have to be set in the build configuration. For
// MacOSX/iOS, it is part of xcconfig file.
#ifndef _RTCPAL_PLATFORM_SPECIFIC_H_
#define _RTCPAL_PLATFORM_SPECIFIC_H_

#if defined(WINPHONE)
    #include <windows/rtcpal_private.h>
    #include <products/rtcpal_winphone_config.h>

    // On WINPHONE builds we need this remap, since Core System / Mobile Core doesn't have WaitForMultipleObjects
    #define WaitForMultipleObjects(a,b,c,d) WaitForMultipleObjectsEx((a),(b),(c),(d),FALSE)

#elif defined(_WIN32_WINNT)
    #include <windows/rtcpal_private.h>

    #include <products/rtcpal_winnt_pc_config.h>

#elif defined(_WIN32_WCE)
    #include <windows/rtcpal_private_ce.h>
    #include <products/rtcpal_wince_config.h>
#elif defined(_UNIX_MACOSX)
    #include <unix/rtcpal_macosx_autoconfig.h>
    #include <unix/rtcpal_private.h>
    #include <products/rtcpal_darwin_mac_config.h>
#elif defined(_UNIX_IPHONE) || defined(_UNIX_IPAD)
    #include <unix/rtcpal_ios_autoconfig.h>
    #include <unix/rtcpal_private.h>

    #if defined(_UNIX_IPHONE)
        #include <products/rtcpal_darwin_ios_config.h>
    #else
        #include <products/rtcpal_darwin_ipad_config.h>
    #endif

    #include <unix/rtcpal_private_ios.h>
#elif defined(ANDROID)
    #include <unix/rtcpal_android_autoconfig.h>
    #include <products/rtcpal_linux_android_config.h>
    #include <unix/rtcpal_private.h>
    #include <unix/rtcpal_private_android.h>
#elif defined(__linux)
    #include <unix/rtcpal_linux_autoconfig.h>
    #include <products/rtcpal_linux_android_config.h>
    #include <unix/rtcpal_private.h>
    #include <unix/rtcpal_private_android.h>
#else
    // Unknown platform still?
    #error "Unknown Platform"
#endif

#endif // _RTCPAL_PLATFORM_SPECIFIC_H_