/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    mmvr_android.h

Abstract:

    Android-based MMVR interface

Specification:

    http://office/15/specs/Specs/Lync Media/Media PAL for Android/Lync Media Dev Plan - Video Pipeline on Android.docx

--*/
#ifndef __MMVR_ANDROID_H__
#define __MMVR_ANDROID_H__

#pragma once

#include <jni.h>

#ifdef __cplusplus
extern "C" {
#endif

struct MMVRAndroidRenderer;
typedef struct MMVRAndroidRenderer* HMMVRANDROIDRENDERER;

typedef enum tagMMVRAndroidEvent_e
{
    EMMVRAndroid_None,
    EMMVRAndroid_ImageSizeChanged,    // Render Image size changed
    EMMVRAndroid_SmartCropInfoChanged,// Smart crop info changed
    EMMVRAndroid_NotifyUIRender,      // Notify UI do render
}MMVRAndroidEvent_e;

typedef enum tagMMVRAutoFitMode_e
{
    EMMVRAutoFit_Stretch = 0,   // video frame will be stetched to client area
    EMMVRAutoFit_Crop,          // left and right will be cropped
    EMMVRAutoFit_LetterBoxing,  // left and right will be padding with black backgroud
    EMMVRAutoFit_SmartCrop,     // Crop video to known region of interest, preserving original aspect ratio (letterboxing)
}MMVRAutoFitMode_e;

/*
typedef struct MMVRAutoFitDataTag
{
      DWORD  x;     // scale point’s x position
      DWORD  y;     // scale point’s y position
      float fScale; // scale factors
} MMVRAutoFitData;
*/

typedef union MMVRAndroidEventDataTag
{
    struct
    {
      	DWORD dwNewWidth;
      	DWORD dwNewHeight;
    } ImageInfo;
    struct
    {
        DWORD dwLeft;
        DWORD dwTop;
        DWORD dwRight;
        DWORD dwBottom;
        BOOL  bValid;
    } SmartCropInfo;
	
}MMVRAndroidEventData;

typedef
HRESULT
(*PMMVRANDROIDRENDERERCALLBACK)(
    __in   HMMVRANDROIDRENDERER    hVideoRenderer,
    __in   MMVRAndroidEvent_e      eEventType,
    __in   MMVRAndroidEventData*   eventData,
    __in   void* pvContext
    );

// ----------------------------------------------------------------------------
// MMVRVideoCreateAndroidRenderer
//
// Create and retrieve a handle to an Android Renderer
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoCreateAndroidRenderer (
    __in    PMMVRANDROIDRENDERERCALLBACK,   // not used by now
    __in    void* pvCallbackContext,
    __out   HMMVRANDROIDRENDERER * phVideoRenderer
    );

// ----------------------------------------------------------------------------
// MMVRVideoGetAndroidRendererContext
//
// Retrieve the render context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoGetAndroidRendererContext(
    __in    HMMVRANDROIDRENDERER hMMVRAndroid,
    __out   const void** ppVoidContext
    );

HRESULT
MMVRVideoDestroyAndroidRenderer(
    __in    HMMVRANDROIDRENDERER phVideoRenderer
    );

HRESULT
MMVRVideoSetUICallback(
    __in HMMVRANDROIDRENDERER hMMVRAndroid,
    __in JNIEnv* env,
    __in jobject uiObj,
    __in PMMVRANDROIDRENDERERCALLBACK pCallback);

HRESULT
MMVRVideoAndroidRendererDoRender(
    __in    HMMVRANDROIDRENDERER phVideoRenderer
    );

HRESULT
MMVRVideoAndroidSetAutoFitMode(
    __in    HMMVRANDROIDRENDERER phVideoRenderer,
    __in    MMVRAutoFitMode_e eAutoFitMode
    );

HRESULT
MMVRVideoAndroidSurfaceChanged(
    __in    HMMVRANDROIDRENDERER phVideoRenderer,
    __in    DWORD   dwWidth,
    __in    DWORD   dwHeight
    );

#ifdef __cplusplus
}; // extern "C"
#endif
#endif // __MMVR_ANDROID_H__

