//
// UTF16 WCHAR related declarations.
//
// This file is used for compilers which doesn't have 2-byte WCHAR CRT, but can be
// configured to treat wchar_t as 16-bit unsigned integer.
//
// You should use MSC version of rtcpal_wc16 for compiler that supports 2-byte WCHAR CRT.
//

#ifndef __RTCPAL_WC16_H__
#define __RTCPAL_WC16_H__

#include <wchar.h>
#include <wctype.h>

#ifdef __cplusplus
    #include <unix/rtcpal_enter_stl_fix.h>
    #include <string>
    #include <unix/rtcpal_leave_stl_fix.h>
#endif

//
// poison every wchar_t related CRT functions.
// Note: GCC poison them only in C mode. It doesn't do that when in C++ mode
//

#ifdef _ENABLE_POISON_
#ifdef __GNUC__
    #pragma GCC poison fgetws fputwc fputws fwprintf fwscanf mbsnrtowcs mbsrtowcs putwc putwchar swprintf swscanf vfwprintf vfwscanf vswscanf vwprintf vwscanf wcscat wcschr wcscmp wcscoll wcscpy wcscspn wcsftime wcsftime wcslcat wcslcpy wcslen wcsncat wcsncmp wcsncpy wcsnrtombs wcspbrk wcsrchr wcsrtombs wcsspn wcsstr wcstod wcstof wcstok wcstol wcstold wcstoll wcstoul wcstoull wcswidth wcsxfrm wcwidth wmemchr wmemcmp wmemcpy wmemmove wmemset wscanf
#else
    #error "Untested compiler"
#endif
#endif // _ENABLE_POISON_

#ifdef __cplusplus
extern "C" {
#endif

extern
int
rtcpal_wopen_s(int * pfh, const WCHAR * filename, int oflag, int pmode);

extern
int
rtcpal_wcreat(const WCHAR * filename, int pmode);

extern
FILE *
rtcpal_wfopen(const wchar_t * filename, const wchar_t * mode);

extern
int
rtcpal_wfopen_s(FILE ** pFile, const wchar_t * filename, const wchar_t * mode);

#if defined(__linux)
#define rtcpal_wprintf wprintf
#else
extern
int
rtcpal_wprintf(const wchar_t *, ...);
#endif

extern
int
rtcpal_wmemcmp(const wchar_t * , const wchar_t * , size_t);

extern
wchar_t *
rtcpal_wmemchr(const wchar_t *, wchar_t , size_t);

extern
wchar_t *
rtcpal_wmemmove(wchar_t *, const wchar_t *, size_t);

extern
wchar_t *
rtcpal_wmemcpy(wchar_t *, const wchar_t *, size_t);

extern
wchar_t *
rtcpal_wmemset(wchar_t *, wchar_t, size_t);

extern
size_t
rtcpal_wcslen(const wchar_t * str);

extern
wchar_t *
rtcpal_wcspbrk(const wchar_t * str, const wchar_t * strCharSet);

extern
int
rtcpal_wcscmp(const wchar_t * str1, const wchar_t * str2);

extern
int
rtcpal_wcsicmp(const wchar_t * str1, const wchar_t * str2);

extern
int
rtcpal_wcsnicmp(const wchar_t * str1, const wchar_t * str2, size_t n);

extern
wchar_t *
rtcpal_wcscpy(wchar_t * dst, const wchar_t * src);

extern
size_t
rtcpal_wcsnlen(const wchar_t * str, size_t n);

extern
int
rtcpal_wcsncmp(const wchar_t * str1, const wchar_t * str2, size_t n);

extern
wchar_t *
rtcpal_wcsncpy(  wchar_t * dst, const wchar_t * src, size_t n );

extern
wchar_t *
rtcpal_wcschr(const wchar_t * str, const wchar_t in_chr);

extern
wchar_t *
rtcpal_wcsstr(const wchar_t * str, const wchar_t * pat);

extern
size_t
rtcpal_wcsspn(const wchar_t * str, const wchar_t * set);

extern
size_t
rtcpal_wcscspn(const wchar_t * str, const wchar_t * set);

extern
wchar_t
rtcpal_btowc(int);

extern
wchar_t *
rtcpal_wcslwr(wchar_t *str); 

extern
wchar_t *
rtcpal_wcsrev(wchar_t * str);

extern
size_t
rtcpal_wcstombs(char * s, const wchar_t * pwcs, size_t n);

extern
size_t
rtcpal_mbstowcs(wchar_t * pwcs, const char* s, size_t n);

extern
wchar_t *
rtcpal_wcsncat(wchar_t * dst, const wchar_t * src, size_t n);

extern
wchar_t *
rtcpal_wcsrchr(const wchar_t * str, wchar_t in_chr);

extern
unsigned long
rtcpal_wcstoul(const wchar_t *, wchar_t **, int);

extern
long
rtcpal_wcstol(const wchar_t * , wchar_t **, int);

extern
int64_t
rtcpal_wcstoll(const wchar_t *nptr, wchar_t **endptr, int ibase);

extern
uint64_t
rtcpal_wcstoull(const wchar_t *nptr, wchar_t **endptr, int ibase);

extern
wchar_t *
rtcpal_fgetws(wchar_t * ws, int n, FILE * stream);

extern
int
rtcpal_swscanf(const wchar_t * ws, const wchar_t * format, ...);

int
rtcpal_mbstowcs_s(
    __out_opt size_t * _PtNumOfCharConverted,
    __out_ecount_part_opt(_SizeInWords, *_PtNumOfCharConverted) wchar_t * _DstBuf,
    __in size_t _SizeInWords,
    __in_ecount_z(_MaxCount) const char * _SrcBuf,
    __in size_t _MaxCount
    );

double __cdecl rtcpal_wtof(
    const wchar_t* pSrc
    );

int __cdecl rtcpal_wtoi(
    const wchar_t* pSrc
    );

#undef _wtof
#define _wtof rtcpal_wtof

#undef _wtoi
#define _wtoi rtcpal_wtoi

#if defined(ANDROID) || defined(__linux)
#undef mbstowcs
#define mbstowcs rtcpal_mbstowcs

#undef wcstombs
#define wcstombs rtcpal_wcstombs
#endif

#undef mbstowcs_s
#define mbstowcs_s  rtcpal_mbstowcs_s

#undef _wcsnicmp
#define _wcsnicmp rtcpal_wcsnicmp

#undef _wcsicmp
#define _wcsicmp rtcpal_wcsicmp

#define fscanf_s fscanf
#define fprintf_s fprintf

#define rtcpal_towlower towlower
#define rtcpal_towupper towupper
#define rtcpal_iswalnum iswalnum
#define rtcpal_iswalpha iswalpha
#define rtcpal_iswascii iswascii
#define rtcpal_iswblank iswblank
#define rtcpal_iswcntrl iswcntrl
#define rtcpal_iswdigit iswdigit
#define rtcpal_iswgraph iswgraph
#define rtcpal_iswhexnumber iswhexnumber
#define rtcpal_iswideogram iswideogram
#define rtcpal_iswlower iswlower
#define rtcpal_iswnumber iswnumber
#define rtcpal_iswphonogram iswphonogram
#define rtcpal_iswprint iswprint
#define rtcpal_iswpunct iswpunct
#define rtcpal_iswrune  iswrune
#define rtcpal_iswspace iswspace
#define rtcpal_iswgraph iswgraph
#define rtcpal_iswhexnumber iswhexnumber
#define rtcpal_iswideogram iswideogram
#define rtcpal_iswlower iswlower
#define rtcpal_iswnumber iswnumber
#define rtcpal_iswphonogram iswphonogram
#define rtcpal_iswprint iswprint
#define rtcpal_iswpunct iswpunct
#define rtcpal_iswrune iswrune
#define rtcpal_iswspace iswspace
#define rtcpal_iswspecial iswspecial
#define rtcpal_iswupper iswupper
#define rtcpal_iswxdigit iswxdigit

#ifdef __cplusplus
};
#endif

#ifdef __cplusplus
    // Unfortunately, GCC (sometimes?) links to compiled versions of some low-level wchar_t standard library code,
    // which is of course compiled with sizeof(wchar_t) == 4. This doesn't happen in ship, presumably because
    // the compiler is inlining more aggressively, but we also want to minimize differences between debug/ship.
    // Hackaround is to provide specializations of the known problem routines and hope that GCC prefers them
    // to the version in the 4-byte-wchar compiled stdlib binary.
    // NOTE: These cannot be inline, or the linker will ignore them.
    namespace std {
        template<const wchar_t*> static size_t   distance( const wchar_t* first, const wchar_t* last ) { return last - first; }
        template<wchar_t*> static void     advance( wchar_t*& i, size_t n ) { i += n; }
    }

    namespace wc16 {
        struct wchar16_traits {
           typedef wchar_t      char_type;
           typedef wint_t       int_type;
           typedef std::streampos    pos_type;
           typedef std::streamoff    off_type;
		   typedef mbstate_t    state_type;

           static void
           assign(char_type& c1, const char_type& c2)
           { c1 = c2; }

           static bool
           eq(const char_type& c1, const char_type& c2)
           { return c1 == c2; }

           static bool
           lt(const char_type& c1, const char_type& c2)
           { return c1 < c2; }

           static int
           compare(const char_type* s1, const char_type* s2, size_t n)
           { return rtcpal_wmemcmp(s1, s2, n); }

           static size_t
           length(const char_type* s)
           { return rtcpal_wcslen(s); }

           static const char_type*
           find(const char_type* s, size_t n, const char_type& a)
           { return rtcpal_wmemchr(s, a, n); }

           static char_type*
           move(char_type* s1, const char_type* s2, size_t n)
           { return rtcpal_wmemmove(s1, s2, n); }

           static char_type*
           copy(char_type* s1, const char_type* s2, size_t n)
           { return rtcpal_wmemcpy(s1, s2, n); }

           static char_type*
           assign(char_type* s, size_t n, char_type a)
           { return rtcpal_wmemset(s, a, n); }

           static char_type
           to_char_type(const int_type& c) { return static_cast<char_type>(c); }

           static int_type
           to_int_type(const char_type& c) { return static_cast<int_type>(c); }

           static bool
           eq_int_type(const int_type& c1, const int_type& c2)
           { return c1 == c2; }

           static int_type
           eof() { return static_cast<int_type>(WEOF); }

           static int_type
           not_eof(const int_type& c)
           { return eq_int_type(c, eof()) ? 0 : c; }
        };

        typedef std::basic_string< wchar_t, wchar16_traits > wstring;
        typedef std::basic_ostringstream< wchar_t, wchar16_traits > wostringstream;
        typedef std::basic_ofstream< wchar_t, wchar16_traits > wofstream;
    };    // namespace wc16

    typedef wc16::wstring rtcpal_wstring;
    typedef wc16::wostringstream rtcpal_wostringstream;

#endif    // __cplusplus

#endif // __RTCPAL_WC16_H__

