#pragma once

#if !defined(_WIN32)

#ifndef _RTCPAL_CONCURRENT_QUEUE_H_
#define _RTCPAL_CONCURRENT_QUEUE_H_

#include <queue>
#include <mutex>

namespace Concurrency
{

template <typename T>
class concurrent_queue
{
public:

    T& front()
    {
        std::unique_lock<std::mutex> mlock(_mutex);
        return _queue.front();
    } 

    const T& front() const
    {
        std::unique_lock<std::mutex> mlock(_mutex);
        return _queue.front();
    }

    size_t unsafe_size() const
    {
        return _queue.size();
    }

    bool empty() const
    {
        std::unique_lock<std::mutex> mlock(const_cast<concurrent_queue*>(this)->_mutex);
        return _queue.empty();
    }

    bool try_pop(T& item)
    {
        bool isDequeued = false;
        std::unique_lock<std::mutex> mlock(_mutex);
        if(!_queue.empty())
        {
            item = _queue.front();
            _queue.pop();
            isDequeued = true;
        }
        return isDequeued;
    }

    void push(const T& item)
    {
        std::unique_lock<std::mutex> mlock(_mutex);
        _queue.push(item);
    }

    void push(T&& item)
    {
        std::unique_lock<std::mutex> mlock(_mutex);
        _queue.push(std::move(item));
    }
private:
    std::queue<T> _queue;
    std::mutex _mutex;


};

}

#endif // _RTCPAL_CONCURRENT_QUEUE_H_
#endif // !_WIN32
