// MSC specific CRT functions

#ifndef __RTCPAL_MSC_CRT__
#define __RTCPAL_MSC_CRT__

// Don't define __forceinline to RTCPAL_FORCEINLINE, which does the real
// "forced" inline.
//
// GCC optimizer aggressively throw away "forced" inline functions if
// they are not referenced by any other functions at compile time. MSC
// seems handle __forceinline differently. It will still attempt to compile
// it as if there is no "__forceinline" keyword ==> linker will be happy.
//
// Because of the above behavior differences, it can cause "undefined symbol"
// error at link time in GCC for "CWMVideoObjectEncoder::MVCostProgressive2"
// in videodsp. Note: that particular VIDEODSP function is implemented in
// C++ files which isn't included by any other files.
//
// #define __forceinline RTCPAL_FORCEINLINE

#ifdef __cplusplus
    #define __forceinline inline
#else
    // avoid duplicate symbol error at link time
    #define __forceinline static inline
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define _declspec(whatever)

#define __int64 long long
#define __int32 int
// int16_t adn int8_t are already defined in C++11
//typedef int16_t __int16;
//typedef int8_t __int8;

// Windows structural exception
#define __try   try
#define __except(...)    catch(...)

#define DECLSPEC_NOINLINE
#define _cdecl
#define __cdecl
#define __fastcall

int _stricmp(const char * a, const char * b);

#define _strcmpi _stricmp

#ifdef __max
#error "name conflict: __max"
#endif

#ifdef __min
#error "name conflict: __min"
#endif

#define __max(a,b)  (((a) > (b)) ? (a) : (b))
#define __min(a,b)  (((a) < (b)) ? (a) : (b))

#ifndef _countof
#define _countof(_Array) (sizeof(_Array) / sizeof(_Array[0]))
#endif

#define _MAX_PATH          260
#define _MAX_FNAME  256 /* max. length of file name component */

void * _aligned_malloc( size_t size, size_t alignment);
void _aligned_free(void * p);

__checkReturn int
_strnicmp(__in_z const char * _Str1, __in_z const char * _Str2, __in size_t _MaxCount);

typedef int32_t __time32_t;
#define _UI32_MAX     UINT32_MAX

#define _malloca(x)     alloca(x)
#define _freea(x)

#ifndef _I64_MAX
#define _I64_MAX    INT64_MAX
#endif

#ifndef _I64_MIN
#define _I64_MIN    INT64_MIN
#endif

__checkReturn int fopen_s(__deref_out_opt FILE ** _File, __in_z const char * _Filename, __in_z const char * _Mode);
__checkReturn int getenv_s(__out size_t * _ReturnSize, __out_ecount_z(_DstSize) char * _DstBuf, __in size_t _DstSize, __in_z const char * _VarName);

int vsprintf_s(__out_ecount_z(_Size) char * _DstBuf, __in size_t _Size, __in_z __format_string const char * _Format, va_list _ArgList);
__checkReturn int _wtoi(__in_z const wchar_t *_Str);
int vswprintf_s(__out_ecount_z(_SizeInWords) wchar_t * _Dst, __in size_t _SizeInWords, __in_z __format_string const wchar_t * _Format, va_list _ArgList);

__checkReturn int _wcsicmp(__in_z const wchar_t * _Str1, __in_z const wchar_t * _Str2);
__checkReturn int _wcsnicmp(__in_z const wchar_t * _Str1, __in_z const wchar_t * _Str2, __in size_t _MaxCount);

__checkReturn FILE * _wfopen(__in_z const wchar_t * _Filename, __in_z const wchar_t * _Mode);
__checkReturn int _wfopen_s(__deref_out_opt FILE ** _File, __in_z const wchar_t * _Filename, __in_z const wchar_t * _Mode);

int _wcslwr_s(__inout_ecount_z(_SizeInWords) wchar_t * _Str, __in size_t _SizeInWords);
int _wcsupr_s(__inout_ecount_z(_Size) wchar_t * _Str, __in size_t _Size);
int wcstombs_s(__out_opt size_t * _PtNumOfCharConverted, __out_bcount_part_opt(_DstSizeInBytes, *_PtNumOfCharConverted) char * _Dst, __in size_t _DstSizeInBytes, __in_z const wchar_t * _Src, __in size_t _MaxCountInBytes);

int _strlwr_s(__inout_ecount_z(_Size) char * _Str, __in size_t _Size);
int _strupr_s(__inout_ecount_z(_Size) char * _Str, __in size_t _Size);

int _mbslwr_s(__inout_bcount_z_opt(_SizeInBytes) unsigned char *_Str, __in size_t _SizeInBytes);
int _mbsupr_s(__inout_bcount_z(_SizeInBytes) unsigned char *_Str, __in size_t _SizeInBytes);

intptr_t _get_heap_handle(void);

static
int64_t RTCPAL_FORCEINLINE _abs64(int64_t v) {
    return (v < 0)?(-v): v;
}

#define _alloca alloca
#define _resetstkoflw()

#define _read read
#define _lseeki64 lseek

#ifndef _controlfp_s
#define _controlfp_s(whatever1,whatever2,whatever3)
#endif // _controlfp_s

#ifdef __cplusplus
}
#endif

#endif // __RTCPAL_MSC_CRT__


