/***
*	mbusafecrt.h - public declarations for SafeCRT lib
*
*       Copyright (c) Microsoft Corporation. All rights reserved.
*
*	Purpose:
*       This file contains the public declarations SafeCRT
*       functions ported to MacOS. These are the safe versions of
*		functions standard functions banned by SWI
****/

/* shields! */

#ifndef MBUSAFECRT_H
#define MBUSAFECRT_H

#ifdef __cplusplus
#include <cwchar>
#include <cstdarg>
#else
#include <wchar.h>
#include <stdarg.h>
#endif // __cplusplus

/* MacOS does not define a specifc type for errnos, but SafeCRT does
   The following is taken from crtdefs.h and needs to be guarded with #ifndef
   because it conflicts when you include headers from winapis
*/ 
#ifndef _ERRCODE_DEFINED
#define _ERRCODE_DEFINED
typedef int errcode;
typedef int errno_t;
#endif

#ifndef __CRTDECL
#define __CRTDECL	__cdecl
#endif

#ifndef __cdecl
#define __cdecl	
#endif

/* _TRUNCATE */
#define _TRUNCATE ((size_t)-1)

/* errno value that specific to SafeCRT */
#define STRUNCATE       80

/*
 * Sizes for buffers used by the _makepath() and _splitpath() functions.
 * note that the sizes include space for 0-terminator
 */
#define MBUSAFECRT_MAX_PATH   260 /* max. length of full pathname */
#define MBUSAFECRT_MAX_DRIVE  3   /* max. length of drive component */
#define MBUSAFECRT_MAX_DIR    256 /* max. length of path component */
#define MBUSAFECRT_MAX_FNAME  256 /* max. length of file name component */
#define MBUSAFECRT_MAX_EXT    256 /* max. length of extension component */

/* helper macros for cpp overloads */
#if defined (__cplusplus)

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_0(_ReturnType, _FuncName, _DstType, _Dst) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_Size]) \
    { \
        return _FuncName(_Dst, _Size); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_Size], _TType1 _TArg1) \
    { \
        return _FuncName(_Dst, _Size, _TArg1); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_Size], _TType1 _TArg1, _TType2 _TArg2) \
    { \
        return _FuncName(_Dst, _Size, _TArg1, _TArg2); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_3(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_Size], _TType1 _TArg1, _TType2 _TArg2, _TType3 _TArg3) \
    { \
        return _FuncName(_Dst, _Size, _TArg1, _TArg2, _TArg3); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_4(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3, _TType4, _TArg4) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_Size], _TType1 _TArg1, _TType2 _TArg2, _TType3 _TArg3, _TType4 _TArg4) \
    { \
        return _FuncName(_Dst, _Size, _TArg1, _TArg2, _TArg3, _TArg4); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_HType1 _HArg1, _DstType (&_Dst)[_Size], _TType1 _TArg1) \
    { \
        return _FuncName(_HArg1, _Dst, _Size, _TArg1); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_2(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_HType1 _HArg1, _DstType (&_Dst)[_Size], _TType1 _TArg1, _TType2 _TArg2) \
    { \
        return _FuncName(_HArg1, _Dst, _Size, _TArg1, _TArg2); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_3(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_HType1 _HArg1, _DstType (&_Dst)[_Size], _TType1 _TArg1, _TType2 _TArg2, _TType3 _TArg3) \
    { \
        return _FuncName(_HArg1, _Dst, _Size, _TArg1, _TArg2, _TArg3); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_2_0(_ReturnType, _FuncName, _HType1, _HArg1, _HType2, _HArg2, _DstType, _Dst) \
    extern "C++" \
    { \
    template <size_t _Size> \
    inline \
    _ReturnType __CRTDECL _FuncName(_HType1 _HArg1, _HType2 _HArg2, _DstType (&_Dst)[_Size]) \
    { \
        return _FuncName(_HArg1, _HArg2, _Dst, _Size); \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1_ARGLIST(_ReturnType, _FuncName, _VFuncName, _DstType, _Dst, _TType1, _TArg1) \
    extern "C++" \
    { \
    template <size_t _DstSize> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_DstSize], _TType1 _TArg1, ...) \
    { \
        va_list _ArgList; \
        va_start(_ArgList, _TArg1); \
        _ReturnType retVal = _VFuncName(_Dst, _DstSize, _TArg1, _ArgList); \
		va_end(_ArgList); \
		return retVal; \
    } \
    }

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2_ARGLIST(_ReturnType, _FuncName, _VFuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2) \
    extern "C++" \
    { \
    template <size_t _DstSize> \
    inline \
    _ReturnType __CRTDECL _FuncName(_DstType (&_Dst)[_DstSize], _TType1 _TArg1, _TType2 _TArg2, ...) \
    { \
        va_list _ArgList; \
        va_start(_ArgList, _TArg2); \
        _ReturnType retVal = _VFuncName(_Dst, _DstSize, _TArg1, _TArg2, _ArgList); \
		va_end(_ArgList); \
		return retVal; \
    } \
    }

#else  /* defined (__cplusplus) && _MBUSAFECRT_SECURE_CPP_OVERLOAD_SECURE_NAMES */

#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_0(_ReturnType, _FuncName, _DstType, _Dst)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_3(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_4(_ReturnType, _FuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3, _TType4, _TArg4)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_2(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_3(_ReturnType, _FuncName, _HType1, _HArg1, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2, _TType3, _TArg3)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_2_0(_ReturnType, _FuncName, _HType1, _HArg1, _HType2, _HArg2, _DstType, _Dst)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1_ARGLIST(_ReturnType, _FuncName, _VFuncName, _DstType, _Dst, _TType1, _TArg1)
#define __MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2_ARGLIST(_ReturnType, _FuncName, _VFuncName, _DstType, _Dst, _TType1, _TArg1, _TType2, _TArg2)

#endif  /* defined (__cplusplus) && _MBUSAFECRT_SECURE_CPP_OVERLOAD_SECURE_NAMES */

#ifdef __cplusplus
	extern "C" {
#endif

extern errno_t memcpy_s(void * _Dst, size_t _DstSize, const void * _Src, size_t _MaxCount);
extern errno_t memmove_s(void * _Dst, size_t _DstSize, const void * _Src, size_t _MaxCount);

static inline errno_t wmemcpy_s(wchar_t *_S1, size_t _N1, const wchar_t *_S2, size_t _N2)
{
	return memcpy_s(_S1, _N1*sizeof(wchar_t), _S2, _N2*sizeof(wchar_t));
}

static inline errno_t wmemmove_s(wchar_t *_S1, size_t _N1, const wchar_t *_S2, size_t _N2)
{
	return memmove_s(_S1, _N1*sizeof(wchar_t), _S2, _N2*sizeof(wchar_t));
}

//------------------------------------------------------------------------------
//	memset_s
//
//	The memset_s() function writes n bytes of value c (converted to an unsigned char) to the ptr buffer.
//	Unlikely memset, this should never get optimized out by compiler.
//	It is equivalent to SecureZeroMemory Windows API.
//------------------------------------------------------------------------------
static inline __attribute__((always_inline)) void * memset_s(void *ptr, int c, size_t n)
{
	volatile unsigned char * vptr = (volatile unsigned char *) ptr;
	while (n) {
		*vptr = (unsigned char) c;
		vptr++;
		n--;
	}
	return ptr;
}

//------------------------------------------------------------------------------
//
// ALL safecrt routines return null terminated buffers!
//
// All numberOfElements values are in number of characters of appropriate type, NOT number of bytes (see 
// http://msdn.microsoft.com/en-us/library/d45bbxx4(VS.80).aspx for example).
//------------------------------------------------------------------------------

typedef void ( *tSafeCRT_AssertFuncPtr )( const char* inExpression, const char* inComment, const char* inFile, const unsigned long inLineNum );
void MBUSafeCRTSetAssertFunc( tSafeCRT_AssertFuncPtr inAssertFuncPtr );

extern errno_t strcat_s( char* ioDest, size_t numberOfElements, const char* inSrc );
extern errno_t wcscat_s( wchar_t* ioDest, size_t numberOfElements, const wchar_t* inSrc );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(errno_t, strcat_s, char, _Dest,  const char *, _Source)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(errno_t, wcscat_s, wchar_t, _Dest, const wchar_t *, _Source)

extern errno_t strncat_s( char* ioDest, size_t numberOfElements, const char* inSrc, size_t inCount );
extern errno_t wcsncat_s( wchar_t* ioDest, size_t numberOfElements, const wchar_t* inSrc, size_t inCount );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(errno_t, strncat_s, char, _Dest, const char *, _Source, size_t, _Count)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(errno_t, wcsncat_s, wchar_t, _Dest, const wchar_t *, _Source, size_t, _Count)

extern errno_t strcpy_s( char* outDest, size_t numberOfElements, const char* inSrc );
extern errno_t wcscpy_s( wchar_t* outDest, size_t numberOfElements, const wchar_t* inSrc );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(errno_t, strcpy_s, char, _Dest, const char *, _Source)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1(errno_t, wcscpy_s, wchar_t, _Dest, const wchar_t *, _Source)

extern errno_t strncpy_s( char* outDest, size_t numberOfElements, const char* inSrc, size_t inCount );
extern errno_t wcsncpy_s( wchar_t* outDest, size_t numberOfElements, const wchar_t* inSrc, size_t inCount );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(errno_t, strncpy_s, char, _Dest, const char *, _Source, size_t, _Count)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2(errno_t, wcsncpy_s, wchar_t, _Dest, const wchar_t *, _Source, size_t, _Count)

extern char* strtok_s( char* inString, const char* inControl, char** ioContext );
extern wchar_t* wcstok_s( wchar_t* inString, const wchar_t* inControl, wchar_t** ioContext );

// strnlen is not required unless the source string is completely untrusted (e.g. anonymous input on a website)
#ifndef SUPPRESS_STRNLEN
	extern size_t strnlen( const char* inString, size_t inMaxSize );
#endif

extern errno_t _itoa_s( int inValue, char* outBuffer, size_t numberOfElements, int inRadix );
extern errno_t _itow_s( int inValue, wchar_t* outBuffer, size_t numberOfElements, int inRadix );
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _itoa_s, int, _Value, char, _Dest, int, _Radix)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _itow_s, int, _Value, wchar_t, _Dest, int, _Radix)

//------------------------------------------------------------------------------
//	_ltoa_s
//
// Parameters:
// value		- Number to be converted.
// str			- Buffer for the resulting string.
// sizeOfstr	- Size of the str in bytes for _ltoa_s or words for _ltow_s.
// radix		- Base of value.
//
// Return Value:
//		Zero if the function was successful or an error code.
//------------------------------------------------------------------------------

extern errno_t _ltoa_s( long inValue, char* outBuffer, size_t numberOfElements, int inRadix );
extern errno_t _ltow_s( long inValue, wchar_t* outBuffer, size_t numberOfElements, int inRadix );
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _ltoa_s, long, _Value, char, _Dest, int, _Radix)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _ltow_s, long, _Value, wchar_t, _Dest, int, _Radix)

extern errno_t _ultoa_s( unsigned long inValue, char* outBuffer, size_t numberOfElements, int inRadix );
extern errno_t _ultow_s( unsigned long inValue, wchar_t* outBuffer, size_t numberOfElements, int inRadix );
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _ultoa_s, unsigned long, _Value, char, _Dest, int, _Radix)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_1_1(errno_t, _ultow_s, unsigned long, _Value, wchar_t, _Dest, int, _Radix)

extern errno_t _i64toa_s( long long inValue, char* outBuffer, size_t numberOfElements, int inRadix );
extern errno_t _i64tow_s( long long inValue, wchar_t* outBuffer, size_t numberOfElements, int inRadix );

extern errno_t _ui64toa_s( unsigned long long inValue, char* outBuffer, size_t numberOfElements, int inRadix );
extern errno_t _ui64tow_s( unsigned long long inValue, wchar_t* outBuffer, size_t numberOfElements, int inRadix );

extern errno_t _makepath_s( char* outDest, size_t numberOfElements, const char* inDrive, const char* inDirectory, const char* inFilename, const char* inExtension );
extern errno_t _wmakepath_s( wchar_t* outDest, size_t numberOfElements, const wchar_t* inDrive, const wchar_t* inDirectory, const wchar_t* inFilename, const wchar_t* inExtension );

extern errno_t _splitpath_s( const char* inPath, char* outDrive, size_t inDriveSize, char* outDirectory, size_t inDirectorySize, char* outFilename, size_t inFilenameSize, char* outExtension, size_t inExtensionSize );
extern errno_t _wsplitpath_s( const wchar_t* inPath, wchar_t* outDrive, size_t inDriveSize, wchar_t* outDirectory, size_t inDirectorySize, wchar_t* outFilename, size_t inFilenameSize, wchar_t* outExtension, size_t inExtensionSize );

//------------------------------------------------------------------------------
//	If the buffer is too small for the text being printed then the buffer is set 
//	 to an empty string and -1 is returned .
//------------------------------------------------------------------------------
extern int sprintf_s( char *string, size_t sizeInBytes, const char *format, ... );
extern int swprintf_s( wchar_t *string, size_t sizeInWords, const wchar_t *format, ... );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1_ARGLIST(int, sprintf_s, _vsprintf_s,  char, _Dest, const char *, _Format)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_1_ARGLIST(int, swprintf_s, _vswprintf_s, wchar_t, _Dest, const wchar_t *, _Format)

//------------------------------------------------------------------------------
//	_snprintf_s - stores count or fewer characters in string and appends a terminating null.
//	
// If the entire string (with terminating null) fits in buffer, then _snprintf_s 
//	returns the number of characters written (not including the terminating null); 
//	otherwise, _snprintf_s returns -1 to indicate that truncation occurred.
//
// If the storage required to store the data and a terminating null exceeds sizeOfBuffer, 
//	string is cleared to NULL errno is set to ERANGE, and returns -1
// If string or format is a NULL pointer, or if count is less than or equal to zero, 
//	errno is set to EINVAL and returns -1. 
//------------------------------------------------------------------------------
extern int _snprintf_s( char *string, size_t sizeInBytes, size_t count, const char *format, ... );
extern int _snwprintf_s( wchar_t *string, size_t sizeInWords, size_t count, const wchar_t *format, ... );

__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2_ARGLIST(int, _snprintf_s, _vsnprintf_s,  char, _Dest, size_t, _Size, const char *,_Format)
__MBU_DEFINE_CPP_OVERLOAD_SECURE_FUNC_0_2_ARGLIST(int, _snwprintf_s, _vsnwprintf_s, wchar_t, _Dest, size_t, _Count, const wchar_t *, _Format)

//------------------------------------------------------------------------------
// WARNING! Thses functions do not behave like the non '_s' versions in all conditions
//
// Unlike standard vsprintf, vsnprintf, vswprintf, vsnwprintf which will return
//	the number of characters (not including the \0) that would have been written 
//	if the buffer was too small, these routines negative values if the buffer can't 
//	contain the requested operation
//
// Returns the number of characters written, not including the terminating null, 
//	or a negative value if an output error occurs.
// If the return value is negative, errno gives a better description of condition
//	errno		meaning
//	EINVAL		- buffer is NULL
//	EINVAL		- format is NULL
//	EINVAL		- count <= 0
//	ERANGE (and buffer set to an empty string)	- sizeOfBuffer too small (and count != _TRUNCATE)
//------------------------------------------------------------------------------
extern int _vsprintf_s( char* string, size_t sizeInBytes, const char* format, va_list arglist );
extern int _vsnprintf_s( char* string, size_t sizeInBytes, size_t count, const char* format, va_list arglist );

extern int _vswprintf_s( wchar_t* string, size_t sizeInWords, const wchar_t* format, va_list arglist );
extern int _vsnwprintf_s( wchar_t* string, size_t sizeInWords, size_t count, const wchar_t* format, va_list arglist );

extern int _vscprintf (const char *format, va_list ap);
extern int _vscwprintf (const wchar_t *format, va_list ap);

#define _vsprintf_p vsnprintf
#define _vswprintf_p vswprintf
//------------------------------------------------------------------------------
// WARNING! Thses functions do not behave like the non '_s' versions in all conditions
//
// The sscanf_s function reads data from buffer into the location given by each argument. 
// The arguments after the format string specify pointers to variables with a 
//	type that corresponds to a type specifier in format. Unlike the less secure 
//	version sscanf_s, a buffer size parameter sizeOfBuffer is required when using
//	the type field characters c, C, s, S and [. This parameter must be supplied
//	as an additional parameter after each buffer which requires it
//------------------------------------------------------------------------------
extern int sscanf_s( const char *string, const char *format, ... );
extern int swscanf_s( const wchar_t *string, const wchar_t *format, ... );

extern int _snscanf_s( const char *string, size_t count, const char *format, ... );
extern int _snwscanf_s( const wchar_t *string, size_t count, const wchar_t *format, ... );

//------------------------------------------------------------------------------
// MacBU Specific pascal string routines
//------------------------------------------------------------------------------
void CopyPascalStringToC_s (const unsigned char* src /*aka ConstStr255Param*/, char* dst, size_t dstSizeInBytes);
void CopyCStringToPascal_s (const char* src, unsigned char dst[256] /*aka Str255*/, size_t maxDstSizeInBytes);

//------------------------------------------------------------------------------
//	CopyPascalString_s (previously PLstrcpy_s)
//
//    pDst -	the destination pascal string
//    dstSizeInBytes -	size of the pDst pascal string (including leading size byte)
//    pSrc -	the source pascal string
//
//	Return:
//    This function returns pDst.
//------------------------------------------------------------------------------
unsigned char* CopyPascalString_s (unsigned char* pDst/*StringPtr*/, size_t dstSizeInBytes, const unsigned char* pSrc /*ConstStr255Param*/);

//------------------------------------------------------------------------------
//	AppendPascalString_s (previously PLstrcat_s)
//
//    pDst -	the destination pascal string
//    dstSizeInBytes -	size of the pDst pascal string (including leading size byte)
//    pSrc -	the source pascal string
//
//	Return:
//    This function returns pDst.
//------------------------------------------------------------------------------
unsigned char* AppendPascalString_s (unsigned char* pDst /*StringPtr*/, size_t dstSizeInBytes, const unsigned char* pSrc /*ConstStr255Param*/);

//------------------------------------------------------------------------------
// DEPRICATED	this is only provided to support legacy code, do not add to new code
//	use _ltoa_s
//------------------------------------------------------------------------------
void NumToString_s(long theNum, unsigned char theString[256] /*aka Str255*/, unsigned int stringSize); 

#ifdef __cplusplus
	}
#endif

#endif	/* MBUSAFECRT_H */
