#ifndef __mediaobj_h__
#define __mediaobj_h__

#include <ddraw.h>
#include <propidl.h>
#include <strmif.h>

EXTERN_C const IID IID_IMediaBuffer;
class IMediaBuffer : public IUnknown
{
public:
    virtual HRESULT STDMETHODCALLTYPE SetLength( 
        DWORD cbLength) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetMaxLength( 
        /* [out] */ __RPC__out DWORD *pcbMaxLength) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetBufferAndLength( 
        /* [out] */ __RPC__deref_out_opt BYTE **ppBuffer,
        /* [out] */ __RPC__out DWORD *pcbLength) = 0;
    
};

typedef AM_MEDIA_TYPE DMO_MEDIA_TYPE;
typedef struct _DMO_OUTPUT_DATA_BUFFER
    {
    IMediaBuffer *pBuffer;
    DWORD dwStatus;
    REFERENCE_TIME rtTimestamp;
    REFERENCE_TIME rtTimelength;
    } 	DMO_OUTPUT_DATA_BUFFER;

enum _DMO_INPUT_DATA_BUFFER_FLAGS
    {	DMO_INPUT_DATA_BUFFERF_SYNCPOINT	= 0x1,
	DMO_INPUT_DATA_BUFFERF_TIME	= 0x2,
	DMO_INPUT_DATA_BUFFERF_TIMELENGTH	= 0x4
    } ;

enum _DMO_OUTPUT_DATA_BUFFER_FLAGS
    {	DMO_OUTPUT_DATA_BUFFERF_SYNCPOINT	= 0x1,
	DMO_OUTPUT_DATA_BUFFERF_TIME	= 0x2,
	DMO_OUTPUT_DATA_BUFFERF_TIMELENGTH	= 0x4,
	DMO_OUTPUT_DATA_BUFFERF_INCOMPLETE	= 0x1000000
    } ;

enum _DMO_INPUT_STATUS_FLAGS
    {	DMO_INPUT_STATUSF_ACCEPT_DATA	= 0x1
    } ;

enum _DMO_INPUT_STREAM_INFO_FLAGS
    {	DMO_INPUT_STREAMF_WHOLE_SAMPLES	= 0x1,
	DMO_INPUT_STREAMF_SINGLE_SAMPLE_PER_BUFFER	= 0x2,
	DMO_INPUT_STREAMF_FIXED_SAMPLE_SIZE	= 0x4,
	DMO_INPUT_STREAMF_HOLDS_BUFFERS	= 0x8
    } ;

enum _DMO_OUTPUT_STREAM_INFO_FLAGS
    {	DMO_OUTPUT_STREAMF_WHOLE_SAMPLES	= 0x1,
	DMO_OUTPUT_STREAMF_SINGLE_SAMPLE_PER_BUFFER	= 0x2,
	DMO_OUTPUT_STREAMF_FIXED_SAMPLE_SIZE	= 0x4,
	DMO_OUTPUT_STREAMF_DISCARDABLE	= 0x8,
	DMO_OUTPUT_STREAMF_OPTIONAL	= 0x10
    } ;

enum _DMO_SET_TYPE_FLAGS
    {	DMO_SET_TYPEF_TEST_ONLY	= 0x1,
	DMO_SET_TYPEF_CLEAR	= 0x2
    } ;

enum _DMO_PROCESS_OUTPUT_FLAGS
    {	DMO_PROCESS_OUTPUT_DISCARD_WHEN_NO_BUFFER	= 0x1
    } ;



EXTERN_C const IID IID_IMediaObject;
class IMediaObject : public IUnknown
{
public:
    virtual HRESULT STDMETHODCALLTYPE GetStreamCount( 
        /* [out] */ __RPC__out DWORD *pcInputStreams,
        /* [out] */ __RPC__out DWORD *pcOutputStreams) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputStreamInfo( 
        DWORD dwInputStreamIndex,
        /* [out] */ __RPC__out DWORD *pdwFlags) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetOutputStreamInfo( 
        DWORD dwOutputStreamIndex,
        /* [out] */ __RPC__out DWORD *pdwFlags) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputType( 
        DWORD dwInputStreamIndex,
        DWORD dwTypeIndex,
        /* [out] */ __RPC__out DMO_MEDIA_TYPE *pmt) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetOutputType( 
        DWORD dwOutputStreamIndex,
        DWORD dwTypeIndex,
        /* [out] */ __RPC__out DMO_MEDIA_TYPE *pmt) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE SetInputType( 
        DWORD dwInputStreamIndex,
        /* [in] */ __RPC__in const DMO_MEDIA_TYPE *pmt,
        DWORD dwFlags) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE SetOutputType( 
        DWORD dwOutputStreamIndex,
        /* [in] */ __RPC__in const DMO_MEDIA_TYPE *pmt,
        DWORD dwFlags) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputCurrentType( 
        DWORD dwInputStreamIndex,
        /* [out] */ __RPC__out DMO_MEDIA_TYPE *pmt) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetOutputCurrentType( 
        DWORD dwOutputStreamIndex,
        /* [out] */ __RPC__out DMO_MEDIA_TYPE *pmt) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputSizeInfo( 
        DWORD dwInputStreamIndex,
        /* [out] */ __RPC__out DWORD *pcbSize,
        /* [out] */ __RPC__out DWORD *pcbMaxLookahead,
        /* [out] */ __RPC__out DWORD *pcbAlignment) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetOutputSizeInfo( 
        DWORD dwOutputStreamIndex,
        /* [out] */ __RPC__out DWORD *pcbSize,
        /* [out] */ __RPC__out DWORD *pcbAlignment) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputMaxLatency( 
        DWORD dwInputStreamIndex,
        /* [out] */ __RPC__out REFERENCE_TIME *prtMaxLatency) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE SetInputMaxLatency( 
        DWORD dwInputStreamIndex,
        REFERENCE_TIME rtMaxLatency) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE Flush( void) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE Discontinuity( 
        DWORD dwInputStreamIndex) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE AllocateStreamingResources( void) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE FreeStreamingResources( void) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetInputStatus( 
        DWORD dwInputStreamIndex,
        /* [out] */ __RPC__out DWORD *dwFlags) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE ProcessInput( 
        DWORD dwInputStreamIndex,
        __RPC__in_opt IMediaBuffer *pBuffer,
        DWORD dwFlags,
        REFERENCE_TIME rtTimestamp,
        REFERENCE_TIME rtTimelength) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE ProcessOutput( 
        DWORD dwFlags,
        DWORD cOutputBufferCount,
        /* [size_is][out][in] */ __RPC__inout_ecount_full(cOutputBufferCount) DMO_OUTPUT_DATA_BUFFER *pOutputBuffers,
        /* [out] */ __RPC__out DWORD *pdwStatus) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE Lock( 
        LONG bLock) = 0;

};

enum _DMO_VIDEO_OUTPUT_STREAM_FLAGS
    {	DMO_VOSF_NEEDS_PREVIOUS_SAMPLE	= 0x1
    } ;

EXTERN_C const IID IID_IDMOVideoOutputOptimizations;

class IDMOVideoOutputOptimizations : public IUnknown
{
public:
    virtual HRESULT STDMETHODCALLTYPE QueryOperationModePreferences( 
        ULONG ulOutputStreamIndex,
        __RPC__in DWORD *pdwRequestedCapabilities) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE SetOperationMode( 
        ULONG ulOutputStreamIndex,
        DWORD dwEnabledFeatures) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetCurrentOperationMode( 
        ULONG ulOutputStreamIndex,
        __RPC__in DWORD *pdwEnabledFeatures) = 0;
    
    virtual HRESULT STDMETHODCALLTYPE GetCurrentSampleRequirements( 
        ULONG ulOutputStreamIndex,
        __RPC__in DWORD *pdwRequestedFeatures) = 0;
    
};

#endif // __mediaobj_h__
