/************************************************************************
*                                                                       *
*   winbase.h -- This module defines the 32-Bit Windows Base APIs       *
*                                                                       *
*   Copyright (c) Microsoft Corp. All rights reserved.                  *
*                                                                       *
************************************************************************/
#ifndef _WINBASE_
#define _WINBASE_

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _SECURITY_ATTRIBUTES {
    RTCPAL_BOOL bInheritHandle;
    int nLength;
    void * lpSecurityDescriptor;
} SECURITY_ATTRIBUTES, *PSECURITY_ATTRIBUTES, *LPSECURITY_ATTRIBUTES;

#define INVALID_HANDLE_VALUE ((HANDLE)(LONG_PTR)-1)
#define INVALID_FILE_SIZE ((DWORD)0xFFFFFFFF)
#define INVALID_SET_FILE_POINTER ((DWORD)-1)
#define INVALID_FILE_ATTRIBUTES ((DWORD)-1)

#define FILE_BEGIN           0
#define FILE_CURRENT         1
#define FILE_END             2

#define WAIT_FAILED ((DWORD)0xFFFFFFFF)
#define WAIT_OBJECT_0       ((STATUS_WAIT_0 ) + 0 )

#define WAIT_ABANDONED         ((STATUS_ABANDONED_WAIT_0 ) + 0 )
#define WAIT_ABANDONED_0       ((STATUS_ABANDONED_WAIT_0 ) + 0 )

#define WAIT_IO_COMPLETION                  STATUS_USER_APC
#define STILL_ACTIVE                        STATUS_PENDING

int
WINAPI
lstrlenA(
    __in LPCSTR lpString
    );

int
WINAPI
lstrlenW(
    __in LPCWSTR lpString
    );
#ifdef UNICODE
#define lstrlen  lstrlenW
#else
#define lstrlen  lstrlenA
#endif // !UNICODE

int
WINAPI
lstrcmpA(
    __in LPCSTR lpString1,
    __in LPCSTR lpString2
    );

int
WINAPI
lstrcmpW(
    __in LPCWSTR lpString1,
    __in LPCWSTR lpString2
    );

int
WINAPI
lstrcmpiA(
    __in LPCSTR lpString1,
    __in LPCSTR lpString2
    );

int
WINAPI
lstrcmpiW(
    __in LPCWSTR lpString1,
    __in LPCWSTR lpString2
    );
#ifdef UNICODE
#define lstrcmpi lstrcmpiW
#else
#define lstrcmpi lstrcmpiA
#endif // !UNICODE

#define NORMAL_PRIORITY_CLASS             0x00000020
#define IDLE_PRIORITY_CLASS               0x00000040
#define HIGH_PRIORITY_CLASS               0x00000080
#define REALTIME_PRIORITY_CLASS           0x00000100
#define BELOW_NORMAL_PRIORITY_CLASS       0x00004000
#define ABOVE_NORMAL_PRIORITY_CLASS       0x00008000

#define HEAP_NO_SERIALIZE               0x00000001      // winnt
#define HEAP_GENERATE_EXCEPTIONS        0x00000004      // winnt
#define HEAP_ZERO_MEMORY                0x00000008      // winnt

typedef struct _OVERLAPPED {
    uintptr_t Internal;
    uintptr_t InternalHigh;
    union {
        struct {
            uint32_t Offset;
            uint32_t OffsetHigh;
        };
        PVOID Pointer;
    };

    HANDLE  hEvent;
} OVERLAPPED, *LPOVERLAPPED;

typedef struct _OVERLAPPED_ENTRY {
    ULONG_PTR lpCompletionKey;
    LPOVERLAPPED lpOverlapped;
    ULONG_PTR Internal;
    DWORD dwNumberOfBytesTransferred;
} OVERLAPPED_ENTRY, *LPOVERLAPPED_ENTRY;

RTCPAL_BOOL
WINAPI
GetExitCodeThread(
    __in  HANDLE hThread,
    __out LPDWORD lpExitCode
    );

DWORD
WINAPI
SuspendThread(
    __in HANDLE hThread
    );

#define SecureZeroMemory RtcPalSecureZeroMemory

#define LMEM_ZEROINIT       0x0040

#ifdef MoveMemory
#error "MoveMemory: conflict"
#endif
#define MoveMemory memmove

#ifdef CopyMemory
#error "CopyMemory: conflict"
#endif
#define CopyMemory memcpy

RTCPAL_BOOL
WINAPI
QueryPerformanceCounter(
    __out LARGE_INTEGER *lpPerformanceCount
    );

RTCPAL_BOOL
WINAPI
QueryPerformanceFrequency(
    __out LARGE_INTEGER *lpFrequency
    );

__checkReturn
DWORD
WINAPI
GetLastError(
    VOID
    );

VOID
WINAPI
SetLastError(
    __in DWORD dwErrCode
    );

#define TLS_OUT_OF_INDEXES ((DWORD)0xFFFFFFFF)

DWORD
WINAPI
TlsAlloc(
    VOID
    );

LPVOID
WINAPI
TlsGetValue(
    __in DWORD dwTlsIndex
    );

RTCPAL_BOOL
WINAPI
TlsSetValue(
    __in     DWORD dwTlsIndex,
    __in_opt LPVOID lpTlsValue
    );

RTCPAL_BOOL
WINAPI
TlsFree(
    __in DWORD dwTlsIndex
    );

#define CREATE_NEW          1
#define CREATE_ALWAYS       2
#define OPEN_EXISTING       3
#define OPEN_ALWAYS         4
#define TRUNCATE_EXISTING   5

#define OPEN_MODE_MASK      0xf
#define OPEN_NON_BLOCK      0x10

RTCPAL_BOOL
WINAPI
SetPriorityClass(
    __in HANDLE hProcess,
    __in DWORD dwPriorityClass
    );

DWORD
WINAPI
GetPriorityClass(
    __in HANDLE hProcess
    );

#define IsBadReadPtr(lp,ucb)    (NULL == (lp))
#define IsBadWritePtr(lp,ucb)   (NULL == (lp))

#define SEM_FAILCRITICALERRORS      0x0001
#define SEM_NOGPFAULTERRORBOX       0x0002
#define SEM_NOALIGNMENTFAULTEXCEPT  0x0004
#define SEM_NOOPENFILEERRORBOX      0x8000
UINT
WINAPI
SetErrorMode(
    __in UINT uMode
    );

VOID
WINAPI
FatalAppExitA(
    __in UINT uAction,
    __in LPCSTR lpMessageText
    );
VOID
WINAPI
FatalAppExitW(
    __in UINT uAction,
    __in LPCWSTR lpMessageText
    );
#ifdef UNICODE
#define FatalAppExit  FatalAppExitW
#else
#define FatalAppExit  FatalAppExitA
#endif // !UNICODE

__bcount_opt(dwSize)
LPVOID
WINAPI
VirtualAlloc(
    __in_opt LPVOID lpAddress,
    __in     SIZE_T dwSize,
    __in     DWORD flAllocationType,
    __in     DWORD flProtect
    );

RTCPAL_BOOL
WINAPI
VirtualFree(
    __in LPVOID lpAddress,
    __in SIZE_T dwSize,
    __in DWORD dwFreeType
    );

RTCPAL_BOOL
WINAPI
CloseHandle(
    __in HANDLE hObject
    );

RTCPAL_BOOL
WINAPI
DuplicateHandle(
    __in        HANDLE hSourceProcessHandle,
    __in        HANDLE hSourceHandle,
    __in        HANDLE hTargetProcessHandle,
    __deref_out LPHANDLE lpTargetHandle,
    __in        DWORD dwDesiredAccess,
    __in        RTCPAL_BOOL bInheritHandle,
    __in        DWORD dwOptions
    );

int
WINAPI
MulDiv(
    __in int nNumber,
    __in int nNumerator,
    __in int nDenominator
    );

typedef struct _SYSTEM_INFO {
    union {
        DWORD dwOemId;          // Obsolete field...do not use
        struct {
            WORD wProcessorArchitecture;
            WORD wReserved;
        };
    };
    DWORD dwPageSize;
    LPVOID lpMinimumApplicationAddress;
    LPVOID lpMaximumApplicationAddress;
    DWORD_PTR dwActiveProcessorMask;
    DWORD dwNumberOfProcessors;
    DWORD dwProcessorType;
    DWORD dwAllocationGranularity;
    WORD wProcessorLevel;
    WORD wProcessorRevision;
} SYSTEM_INFO, *LPSYSTEM_INFO;

VOID
WINAPI
GetSystemInfo(
    __out LPSYSTEM_INFO lpSystemInfo
    );

#define FILE_MAP_COPY       SECTION_QUERY
#define FILE_MAP_WRITE      SECTION_MAP_WRITE
#define FILE_MAP_READ       SECTION_MAP_READ
#define FILE_MAP_ALL_ACCESS SECTION_ALL_ACCESS

__out_opt
HANDLE
WINAPI
CreateFileMappingA(
    __in     HANDLE hFile,
    __in_opt LPSECURITY_ATTRIBUTES lpFileMappingAttributes,
    __in     DWORD flProtect,
    __in     DWORD dwMaximumSizeHigh,
    __in     DWORD dwMaximumSizeLow,
    __in_opt LPCSTR lpName
    );
__out_opt
HANDLE
WINAPI
CreateFileMappingW(
    __in     HANDLE hFile,
    __in_opt LPSECURITY_ATTRIBUTES lpFileMappingAttributes,
    __in     DWORD flProtect,
    __in     DWORD dwMaximumSizeHigh,
    __in     DWORD dwMaximumSizeLow,
    __in_opt LPCWSTR lpName
    );
#ifdef UNICODE
#define CreateFileMapping  CreateFileMappingW
#else
#define CreateFileMapping  CreateFileMappingA
#endif // !UNICODE

__out
HANDLE
WINAPI
OpenFileMappingA(
    __in DWORD dwDesiredAccess,
    __in RTCPAL_BOOL bInheritHandle,
    __in LPCSTR lpName
    );
__out
HANDLE
WINAPI
OpenFileMappingW(
    __in DWORD dwDesiredAccess,
    __in RTCPAL_BOOL bInheritHandle,
    __in LPCWSTR lpName
    );
#ifdef UNICODE
#define OpenFileMapping  OpenFileMappingW
#else
#define OpenFileMapping  OpenFileMappingA
#endif // !UNICODE

__out_opt __out_data_source(FILE)
LPVOID
WINAPI
MapViewOfFile(
    __in HANDLE hFileMappingObject,
    __in DWORD dwDesiredAccess,
    __in DWORD dwFileOffsetHigh,
    __in DWORD dwFileOffsetLow,
    __in SIZE_T dwNumberOfBytesToMap
    );

RTCPAL_BOOL
WINAPI
UnmapViewOfFile(
    __in LPCVOID lpBaseAddress
    );

UINT
WINAPI
GetProfileIntA(
    __in LPCSTR lpAppName,
    __in LPCSTR lpKeyName,
    __in INT nDefault
    );
UINT
WINAPI
GetProfileIntW(
    __in LPCWSTR lpAppName,
    __in LPCWSTR lpKeyName,
    __in INT nDefault
    );
#ifdef UNICODE
#define GetProfileInt  GetProfileIntW
#else
#define GetProfileInt  GetProfileIntA
#endif // !UNICODE

#define STD_INPUT_HANDLE    ((DWORD)-10)
#define STD_OUTPUT_HANDLE   ((DWORD)-11)
#define STD_ERROR_HANDLE    ((DWORD)-12)

HANDLE
WINAPI
GetStdHandle(
    __in DWORD nStdHandle
    );

RTCPAL_BOOL
WINAPI
AllocConsole( VOID );

RTCPAL_BOOL
WINAPI
SetConsoleTitleA(
    __in LPCSTR lpConsoleTitle
    );
RTCPAL_BOOL
WINAPI
SetConsoleTitleW(
    __in LPCWSTR lpConsoleTitle
    );
#ifdef UNICODE
#define SetConsoleTitle  SetConsoleTitleW
#else
#define SetConsoleTitle  SetConsoleTitleA
#endif // !UNICODE

#if defined(RTCPAL_LITTLE_ENDIAN)
    typedef struct _FILETIME {
        uint32_t dwLowDateTime;
        uint32_t dwHighDateTime;
    } FILETIME, *PFILETIME, *LPFILETIME;
#elif defined(RTCPAL_BIG_ENDIAN)
    typedef struct _FILETIME {
        uint32_t dwHighDateTime;
        uint32_t dwLowDateTime;
    } FILETIME, *PFILETIME, *LPFILETIME;
#else
    #error "Unknown endian"
#endif

typedef struct _SYSTEMTIME {
    uint16_t wYear;
    uint16_t wMonth;
    uint16_t wDayOfWeek;
    uint16_t wDay;
    uint16_t wHour;
    uint16_t wMinute;
    uint16_t wSecond;
    uint16_t wMilliseconds;
} SYSTEMTIME, *PSYSTEMTIME, *LPSYSTEMTIME;

EXTERN_C
LPFILETIME
WINAPI
GetSystemTimeAsFileTime(
    __out LPFILETIME lpFileTime
    );

RTCPAL_BOOL
WINAPI
SystemTimeToFileTime(
    __in  CONST SYSTEMTIME *lpSystemTime,
    __out LPFILETIME lpFileTime
    );

RTCPAL_BOOL
WINAPI
FileTimeToSystemTime(
    __in  CONST FILETIME *lpFileTime,
    __out LPSYSTEMTIME lpSystemTime
    );


RTCPAL_BOOL
WINAPI
DeviceIoControl(
    __in        HANDLE hDevice,
    __in        DWORD dwIoControlCode,
    __in_bcount_opt(nInBufferSize) LPVOID lpInBuffer,
    __in        DWORD nInBufferSize,
    __out_bcount_part_opt(nOutBufferSize, *lpBytesReturned) LPVOID lpOutBuffer,
    __in        DWORD nOutBufferSize,
    __out_opt   LPDWORD lpBytesReturned,
    __inout_opt LPOVERLAPPED lpOverlapped
    );

typedef struct _MEMORYSTATUSEX {
    DWORD dwLength;
    DWORD dwMemoryLoad;
    uint64_t ullTotalPhys;
    uint64_t ullAvailPhys;
    uint64_t ullTotalPageFile;
    uint64_t ullAvailPageFile;
    uint64_t ullTotalVirtual;
    uint64_t ullAvailVirtual;
    uint64_t ullAvailExtendedVirtual;
} MEMORYSTATUSEX, *LPMEMORYSTATUSEX;

RTCPAL_BOOL
WINAPI
SetEnvironmentVariableA(
    __in     LPCSTR lpName,
    __in_opt LPCSTR lpValue
    );
RTCPAL_BOOL
WINAPI
SetEnvironmentVariableW(
    __in     LPCWSTR lpName,
    __in_opt LPCWSTR lpValue
    );
#ifdef UNICODE
#define SetEnvironmentVariable  SetEnvironmentVariableW
#else
#define SetEnvironmentVariable  SetEnvironmentVariableA
#endif // !UNICODE

#define CREATE_SUSPENDED                  0x00000004

DWORD
WINAPI
FormatMessageA(
    __in     DWORD dwFlags,
    __in_opt LPCVOID lpSource,
    __in     DWORD dwMessageId,
    __in     DWORD dwLanguageId,
    __out    LPSTR lpBuffer,
    __in     DWORD nSize,
    __in_opt va_list *Arguments
    );
DWORD
WINAPI
FormatMessageW(
    __in     DWORD dwFlags,
    __in_opt LPCVOID lpSource,
    __in     DWORD dwMessageId,
    __in     DWORD dwLanguageId,
    __out    LPWSTR lpBuffer,
    __in     DWORD nSize,
    __in_opt va_list *Arguments
    );
#ifdef UNICODE
#define FormatMessage  FormatMessageW
#else
#define FormatMessage  FormatMessageA
#endif // !UNICODE

#define FORMAT_MESSAGE_ALLOCATE_BUFFER 0x00000100
#define FORMAT_MESSAGE_IGNORE_INSERTS  0x00000200
#define FORMAT_MESSAGE_FROM_SYSTEM     0x00001000

DWORD
WINAPI
GetModuleBaseNameA(
    __in HANDLE hProcess,
    __in_opt HMODULE hModule,
    __out_ecount(nSize) LPSTR lpBaseName,
    __in DWORD nSize
    );
DWORD
WINAPI
GetModuleBaseNameW(
    __in HANDLE hProcess,
    __in_opt HMODULE hModule,
    __out_ecount(nSize) LPWSTR lpBaseName,
    __in DWORD nSize
    );
#ifdef UNICODE
#define GetModuleBaseName  GetModuleBaseNameW
#else
#define GetModuleBaseName  GetModuleBaseNameA
#endif // !UNICODE

#define EncodePointer(p)	((PVOID)(p))
#define DecodePointer(p)	(p)

RTCPAL_BOOL
RTCAPI
RtcPalCreateDirectoryA(
    __in     LPCSTR lpPathName,
    __in LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );
#undef CreateDirectoryA
#define CreateDirectoryA RtcPalCreateDirectoryA
EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalCreateDirectoryW(
    __in     LPCWSTR lpPathName,
    __in LPSECURITY_ATTRIBUTES lpSecurityAttributes
    );
#undef CreateDirectoryW
#define CreateDirectoryW RtcPalCreateDirectoryW
#undef CreateDirectory
#define CreateDirectory RtcPalCreateDirectory
#ifdef UNICODE
#define RtcPalCreateDirectory  RtcPalCreateDirectoryW
#else
#define RtcPalCreateDirectory  RtcPalCreateDirectoryA
#endif // !UNICODE

EXTERN_C
WINAPI
DWORD
GetCurrentDirectoryA(
    DWORD nBufferLength,
    LPSTR lpBuffer
    );

EXTERN_C
WINAPI
DWORD
GetCurrentDirectoryW(
    DWORD nBufferLength,
    LPWSTR lpBuffer
    );

#ifdef UNICODE
#define GetCurrentDirectory GetCurrentDirectoryW
#else
#define GetCurrentDirectory GetCurrentDirectoryA
#endif // !UNICODE

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalDeleteFileA(
    __in LPCSTR lpFileName
    );
EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalDeleteFileW(
    __in LPCWSTR lpFileName
    );
#undef DeleteFileA
#define DeleteFileA RtcPalDeleteFileA
#undef DeleteFileW
#define DeleteFileW RtcPalDeleteFileW
#undef DeleteFile
#define DeleteFile RtcPalDeleteFile
#ifdef UNICODE
#define RtcPalDeleteFile  RtcPalDeleteFileW
#else
#define RtcPalDeleteFile  RtcPalDeleteFileA
#endif // !UNICODE

EXTERN_C
__out
HANDLE
RTCAPI
RtcPalCreateFileA(
    __in     LPCSTR lpFileName,
    __in     DWORD dwDesiredAccess,
    __in     DWORD dwShareMode,
    __in_opt LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __in     DWORD dwCreationDisposition,
    __in     DWORD dwFlagsAndAttributes,
    __in_opt HANDLE hTemplateFile
    );
EXTERN_C
__out
HANDLE
RTCAPI
RtcPalCreateFileW(
    __in     LPCWSTR lpFileName,
    __in     DWORD dwDesiredAccess,
    __in     DWORD dwShareMode,
    __in_opt LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __in     DWORD dwCreationDisposition,
    __in     DWORD dwFlagsAndAttributes,
    __in_opt HANDLE hTemplateFile
    );
#ifdef UNICODE
#define RtcPalCreateFile  RtcPalCreateFileW
#else
#define RtcPalCreateFile  RtcPalCreateFileA
#endif // !UNICODE

#undef CreateFileA
#define CreateFileA RtcPalCreateFileA
#undef CreateFileW
#define CreateFileW RtcPalCreateFileW
#undef CreateFile
#define CreateFile RtcPalCreateFile

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalMoveFileA(
    __in LPCSTR lpExistingFileName,
    __in LPCSTR lpNewFileName
    );
EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalMoveFileW(
    __in LPCWSTR lpExistingFileName,
    __in LPCWSTR lpNewFileName
    );
#ifdef UNICODE
#define RtcPalMoveFile  RtcPalMoveFileW
#else
#define RtcPalMoveFile  RtcPalMoveFileA
#endif // !UNICODE
#undef MoveFileA
#define MoveFileA RtcPalMoveFileA
#undef MoveFileW
#define MoveFileW RtcPalMoveFileW
#undef MoveFile
#define MoveFile RtcPalMoveFile

DWORD
RTCAPI
RtcPalGetTempPathA(
    __in DWORD nBufferLength,
    __out_ecount_part_opt(nBufferLength, return + 1) LPSTR lpBuffer
    );
#undef GetTempPathA
#define GetTempPathA RtcPalGetTempPathA
DWORD
RTCAPI
RtcPalGetTempPathW(
    __in DWORD nBufferLength,
    __out_ecount_part_opt(nBufferLength, return + 1) LPWSTR lpBuffer
    );
#ifdef UNICODE
#define RtcPalGetTempPath  RtcPalGetTempPathW
#else
#define RtcPalGetTempPath  RtcPalGetTempPathA
#endif // !UNICODE
#undef GetTempPathA
#define GetTempPathA RtcPalGetTempPathA
#undef GetTempPathW
#define GetTempPathW RtcPalGetTempPathW
#undef GetTempPath
#define GetTempPath RtcPalGetTempPath

RTCPAL_BOOL
RTCAPI
RtcPalFlushFileBuffers(
    __in    HANDLE hFile
    );
#undef FlushFileBuffers
#define FlushFileBuffers RtcPalFlushFileBuffers

int
RtcPalGetFD(
    __in HANDLE hFile
    );

RTCPAL_BOOL
RTCAPI
RtcPalReadFile(
    __in        HANDLE hFile,
    __out_bcount_part_opt(nNumberOfBytesToRead, *lpNumberOfBytesRead) LPVOID lpBuffer,
    __in        DWORD nNumberOfBytesToRead,
    __out_opt   LPDWORD lpNumberOfBytesRead,
    __inout_opt LPOVERLAPPED lpOverlapped
    );
#undef ReadFile
#define ReadFile RtcPalReadFile


RTCPAL_BOOL
RTCAPI
RtcPalWriteFile(
    __in        HANDLE hFile,
    __in_bcount_opt(nNumberOfBytesToWrite) LPCVOID lpBuffer,
    __in        DWORD nNumberOfBytesToWrite,
    __out_opt   LPDWORD lpNumberOfBytesWritten,
    __inout_opt LPOVERLAPPED lpOverlapped
    );
#undef WriteFile
#define WriteFile RtcPalWriteFile

DWORD
RTCAPI
RtcPalSetFilePointer(
    __in        HANDLE hFile,
    __in        LONG lDistanceToMove,
    __inout_opt PLONG lpDistanceToMoveHigh,
    __in        DWORD dwMoveMethod
    );
#undef SetFilePointer
#define SetFilePointer RtcPalSetFilePointer


RTCPAL_BOOL
RtcPalSetFilePointerEx(
    HANDLE hFile,
    off_t lDistanceToMove,
    off_t * lpNewPos,
    DWORD MoveMethod
    );
#undef SetFilePointerEx
#define SetFilePointerEx RtcPalSetFilePointerEx

RTCPAL_BOOL
RTCAPI
RtcPalGetFileSizeEx(
    __in  HANDLE hFile,
    __out PLARGE_INTEGER lpFileSize
    );
#undef GetFileSizeEx
#define GetFileSizeEx RtcPalGetFileSizeEx

#define FILE_FLAG_DELETE_ON_CLOSE       0x04000000
#define FILE_FLAG_OVERLAPPED            0x40000000
#define FILE_ATTRIBUTE_READONLY         0x00000001

EXTERN_C
__out_opt
HMODULE
RTCAPI
RtcPalLoadLibraryA(
    __in LPCSTR lpLibFileName
    );
EXTERN_C
__out_opt
HMODULE
RTCAPI
RtcPalLoadLibraryW(
    __in LPCWSTR lpLibFileName
    );
#ifdef UNICODE
#define RtcPalLoadLibrary  RtcPalLoadLibraryW
#else
#define RtcPalLoadLibrary  RtcPalLoadLibraryA
#endif // !UNICODE
#undef LoadLibrary
#define LoadLibrary     RtcPalLoadLibrary
#define LoadLibraryW    RtcPalLoadLibraryW
#define LoadLibraryA    RtcPalLoadLibraryA


EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalFreeLibrary(
    __in HMODULE hLibModule
    );
#undef FreeLibrary
#define FreeLibrary RtcPalFreeLibrary

EXTERN_C
FARPROC
RTCAPI
RtcPalGetProcAddressA(
    __in HMODULE hModule,
    __in LPCSTR lpProcName
    );
#undef GetProcAddress
#define GetProcAddress RtcPalGetProcAddressA

EXTERN_C
VOID
RTCAPI
RtcPalGetSystemTime(
    __out LPSYSTEMTIME lpSystemTime
    );
#undef GetSystemTime
#define GetSystemTime RtcPalGetSystemTime

EXTERN_C
VOID
RTCAPI
RtcPalGetLocalTime(
    __out LPSYSTEMTIME lpSystemTime
    );
#undef GetLocalTime
#define GetLocalTime RtcPalGetLocalTime

EXTERN_C
DWORD
RTCAPI
RtcPalGetTickCount(
    VOID
    );
#undef GetTickCount
#define GetTickCount RtcPalGetTickCount

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalFileTimeToSystemTime(
    __in  CONST FILETIME *lpFileTime,
    __out LPSYSTEMTIME lpSystemTime
    );
#undef FileTimeToSystemTime
#define FileTimeToSystemTime RtcPalFileTimeToSystemTime

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalSystemTimeToFileTime(
    __in  CONST SYSTEMTIME *lpSystemTime,
    __out LPFILETIME lpFileTime
    );
#undef SystemTimeToFileTime
#define SystemTimeToFileTime RtcPalSystemTimeToFileTime

EXTERN_C
void
RTCAPI
RtcPalGetSystemTimeAsFileTime(
    LPFILETIME lpFileTime
    );
#undef GetSystemTimeAsFileTime
#define GetSystemTimeAsFileTime RtcPalGetSystemTimeAsFileTime


EXTERN_C
DWORD
RTCAPI
RtcPalGetCurrentProcessId(
    VOID
    );

EXTERN_C
DWORD
RTCAPI
RtcPalGetCurrentThreadId(
    VOID
    );

EXTERN_C
DWORD
RTCAPI
GetCurrentThreadId(
    VOID
    );

EXTERN_C
DWORD
RTCAPI
RtcPalTlsAlloc(
    VOID
    );
#undef TlsAlloc
#define TlsAlloc    RtcPalTlsAlloc

EXTERN_C
void*
RTCAPI
RtcPalTlsGetValue(
    __in DWORD dwTlsIndex
    );
#undef TlsGetValue
#define TlsGetValue RtcPalTlsGetValue

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalTlsSetValue(
    __in     DWORD dwTlsIndex,
    __in_opt LPVOID lpTlsValue
    );
#undef TlsSetValue
#define TlsSetValue RtcPalTlsSetValue

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalTlsFree(
    __in DWORD dwTlsIndex
    );
#undef TlsFree
#define TlsFree     RtcPalTlsFree

EXTERN_C
DWORD
RTCAPI
RtcPalGetModuleFileNameA(
    __in_opt HMODULE hModule,
    __out_ecount_part(nSize, return + 1) LPSTR lpFilename,
    __in DWORD nSize
    );
EXTERN_C
DWORD
RTCAPI
RtcPalGetModuleFileNameW(
    __in_opt HMODULE hModule,
    __out_ecount_part(nSize, return + 1) LPWSTR lpFilename,
    __in DWORD nSize
    );
#ifdef UNICODE
#define RtcPalGetModuleFileName  RtcPalGetModuleFileNameW
#else
#define RtcPalGetModuleFileName  RtcPalGetModuleFileNameA
#endif // !UNICODE

EXTERN_C
__out_opt
HMODULE
RTCAPI
RtcPalGetModuleHandleA(
    __in_opt LPCSTR lpModuleName
    );

EXTERN_C
__out_opt
HMODULE
RTCAPI
RtcPalGetModuleHandleW(
    __in_opt LPCWSTR lpModuleName
    );
#ifdef UNICODE
#define RtcPalGetModuleHandle  RtcPalGetModuleHandleW
#else
#define RtcPalGetModuleHandle  RtcPalGetModuleHandleA
#endif // !UNICODE

__out_bcount_opt( "(uFlags&LMEM_FIXED) ? uBytes : 0" )
HLOCAL
RTCAPI
RtcPalLocalAlloc(
    __in UINT uFlags,
    __in SIZE_T uBytes
    );
#undef LocalAlloc
#define LocalAlloc RtcPalLocalAlloc

HLOCAL
RTCAPI
RtcPalLocalFree(
    __deref HLOCAL hMem
    );
#undef LocalFree
#define LocalFree RtcPalLocalFree

EXTERN_C
void*
RTCAPI
RtcPalHeapAlloc(
    HANDLE hHeap,
    uint32_t dwFlags,
    size_t dwBytes
    );
#define HeapAlloc RtcPalHeapAlloc

EXTERN_C
int32_t
RTCAPI
RtcPalHeapFree(
    HANDLE hHeap,
    uint32_t dwFlags,
    void* lpMem
    );
#define HeapFree RtcPalHeapFree

EXTERN_C
HANDLE
RTCAPI
RtcPalHeapCreate(
    __in uint32_t flOptions,
    __in size_t dwInitialSize,
    __in size_t dwMaximumSize
    );
#define HeapCreate RtcPalHeapCreate

EXTERN_C
int32_t
RTCAPI
RtcPalHeapDestroy(
    __in HANDLE hHeap
    );
#define HeapDestroy RtcPalHeapDestroy

EXTERN_C
SIZE_T
RTCAPI
RtcPalHeapSize(
    __in HANDLE hHeap,
    __in DWORD dwFlags,
    __in LPCVOID lpMem
    );
#define HeapSize    RtcPalHeapSize

EXTERN_C
__bcount(dwBytes)
LPVOID
RTCAPI
RtcPalHeapReAlloc(
    __inout HANDLE hHeap,
    __in    DWORD dwFlags,
    __deref LPVOID lpMem,
    __in    SIZE_T dwBytes
    );
#define HeapReAlloc     RtcPalHeapReAlloc

EXTERN_C
int32_t
RTCAPI
RtcPalGlobalMemoryStatusEx(
    __out void* lpBuffer
    );
#define GlobalMemoryStatusEx    RtcPalGlobalMemoryStatusEx

EXTERN_C
int32_t
RTCAPI
RtcPalHeapSetInformation (
    __in_opt HANDLE HeapHandle,
    __in int32_t HeapInformationClass,
    __in_bcount_opt(HeapInformationLength) PVOID HeapInformation,
    __in size_t HeapInformationLength
    );
#define HeapSetInformation  RtcPalHeapSetInformation

EXTERN_C
HANDLE
RTCAPI
RtcPalGetProcessHeap(
    void
    );
#define GetProcessHeap  RtcPalGetProcessHeap

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalGetUserNameA (
    __out_ecount_part_opt(*pcbBuffer, *pcbBuffer) LPSTR lpBuffer,
    __inout uint32_t* pcbBuffer
    );
EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalGetUserName (
    __out_ecount_part_opt(*pcbBuffer, *pcbBuffer) LPWSTR lpBuffer,
    __inout uint32_t* pcbBuffer
    );
#define GetUserName RtcPalGetUserName
#define GetUserNameW RtcPalGetUserName
#define GetUserNameA RtcPalGetUserNameA

EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalGetComputerNameA (
    __out_ecount_part_opt(*nSize, *nSize + 1) LPSTR lpBuffer,
    __inout DWORD* nSize
    );
EXTERN_C
RTCPAL_BOOL
RTCAPI
RtcPalGetComputerNameW (
    __out_ecount_part_opt(*nSize, *nSize + 1) LPWSTR lpBuffer,
    __inout DWORD* nSize
    );
#ifdef UNICODE
#define RtcPalGetComputerName  RtcPalGetComputerNameW
#else
#define RtcPalGetComputerName  RtcPalGetComputerNameA
#endif // !UNICODE
#define GetComputerName  RtcPalGetComputerName
#define GetComputerNameW RtcPalGetComputerNameW
#define GetComputerNameA RtcPalGetComputerNameA

EXTERN_C
uint32_t
RTCAPI
RtcPalExpandEnvironmentStringsA(
    __in LPCSTR lpSrc,
    __out_ecount_part_opt(nSize, return) LPSTR lpDst,
    __in uint32_t nSize
    );
EXTERN_C
uint32_t
RTCAPI
RtcPalExpandEnvironmentStringsW(
    __in LPCWSTR lpSrc,
    __out_ecount_part_opt(nSize, return) LPWSTR lpDst,
    __in uint32_t nSize
    );
#ifdef UNICODE
#define RtcPalExpandEnvironmentStrings  RtcPalExpandEnvironmentStringsW
#else
#define RtcPalExpandEnvironmentStrings  RtcPalExpandEnvironmentStringsA
#endif // !UNICODE
#define ExpandEnvironmentStrings  RtcPalExpandEnvironmentStrings
#define ExpandEnvironmentStringsW RtcPalExpandEnvironmentStringsW
#define ExpandEnvironmentStringsA RtcPalExpandEnvironmentStringsA

RTCPAL_BOOL
RTCAPI
RtcPalSetThreadPriorityBoost(
    __in HANDLE hThread,
    __in RTCPAL_BOOL bDisablePriorityBoost
    );
#define SetThreadPriorityBoost  RtcPalSetThreadPriorityBoost

RTCPAL_BOOL
RTCAPI
RtcPalGetThreadPriorityBoost(
    __in HANDLE hThread,
    __out PRTCPAL_BOOL pDisablePriorityBoost
    );
#define GetThreadPriorityBoost  RtcPalGetThreadPriorityBoost

// Critical Section
#undef CRITICAL_SECTION
#define CRITICAL_SECTION            RTCPAL_CRITICAL_SECTION
#undef PCRITICAL_SECTION
#define PCRITICAL_SECTION           PRTCPAL_CRITICAL_SECTION
#undef LPCRITICAL_SECTION
#define LPCRITICAL_SECTION          PRTCPAL_CRITICAL_SECTION

void
RTCAPI
RtcPalInitializeCriticalSection(
    __out PRTCPAL_CRITICAL_SECTION lpCriticalSection
    );
#undef InitializeCriticalSection
#define InitializeCriticalSection   RtcPalInitializeCriticalSection

void
RTCAPI
RtcPalEnterCriticalSection(
    __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
    );
#undef EnterCriticalSection
#define EnterCriticalSection        RtcPalEnterCriticalSection

void
RTCAPI
RtcPalLeaveCriticalSection(
    __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
    );
#undef LeaveCriticalSection
#define LeaveCriticalSection        RtcPalLeaveCriticalSection

void
RTCAPI
RtcPalDeleteCriticalSection(
    __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
    );
#undef DeleteCriticalSection
#define DeleteCriticalSection       RtcPalDeleteCriticalSection

RTCPAL_BOOL
RTCAPI
RtcPalInitializeCriticalSectionAndSpinCount(
    __out PRTCPAL_CRITICAL_SECTION lpCriticalSection,
    __in uint32_t dwSpinCount
    );
#undef InitializeCriticalSectionAndSpinCount
#define InitializeCriticalSectionAndSpinCount RtcPalInitializeCriticalSectionAndSpinCount

RTCPAL_BOOL
RTCAPI
RtcPalTryEnterCriticalSection(
    __inout PRTCPAL_CRITICAL_SECTION lpCriticalSection
    );
#define TryEnterCriticalSection RtcPalTryEnterCriticalSection

__out_opt
HANDLE
RTCAPI
RtcPalCreateSemaphoreA(
    __in_opt LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,
    __in     LONG lInitialCount,
    __in     LONG lMaximumCount,
    __in_opt LPCSTR lpName
    );
#undef CreateSemaphoreA
#define CreateSemaphoreA    RtcPalCreateSemaphoreA

__out_opt
HANDLE
RTCAPI
RtcPalCreateSemaphoreW(
    __in_opt LPSECURITY_ATTRIBUTES lpSemaphoreAttributes,
    __in     LONG lInitialCount,
    __in     LONG lMaximumCount,
    __in_opt LPCWSTR lpName
    );
#undef CreateSemaphoreW
#define CreateSemaphoreW    RtcPalCreateSemaphoreW

#undef CreateSemaphore
#ifdef UNICODE
#define CreateSemaphore  CreateSemaphoreW
#else
#define CreateSemaphore  CreateSemaphoreA
#endif // !UNICODE

#undef ReleaseSemaphore
#define ReleaseSemaphore RtcPalReleaseSemaphore

__out_opt
HANDLE
RTCAPI
RtcPalCreateEventA(
    __in_opt LPSECURITY_ATTRIBUTES lpEventAttributes,
    __in     RTCPAL_BOOL bManualReset,
    __in     RTCPAL_BOOL bInitialState,
    __in_opt LPCSTR lpName
    );
#undef CreateEventA
#define CreateEventA    RtcPalCreateEventA

__out_opt
HANDLE
RTCAPI
RtcPalCreateEventW(
    __in_opt LPSECURITY_ATTRIBUTES lpEventAttributes,
    __in     RTCPAL_BOOL bManualReset,
    __in     RTCPAL_BOOL bInitialState,
    __in_opt LPCWSTR lpName
    );
#undef CreateEventW
#define CreateEventW    RtcPalCreateEventW

#ifdef UNICODE
#define CreateEvent  CreateEventW
#else
#define CreateEvent  CreateEventA
#endif // !UNICODE

#undef ResetEvent
#define ResetEvent(h)   RtcPalResetEvent((RtcPalEventHandle)(HANDLE)h)
#undef SetEvent
#define SetEvent(h)     RtcPalSetEvent((RtcPalEventHandle)(HANDLE)h)

__out_opt
HANDLE
RTCAPI
RtcPalCreateMutexA(
    __in_opt LPSECURITY_ATTRIBUTES lpMutexAttributes,
    __in     RTCPAL_BOOL bInitialOwner,
    __in_opt LPCSTR lpName
    );
#undef CreateMutexA
#define CreateMutexA RtcPalCreateMutexA
__out_opt
HANDLE
RTCAPI
RtcPalCreateMutexW(
    __in_opt LPSECURITY_ATTRIBUTES lpMutexAttributes,
    __in     RTCPAL_BOOL bInitialOwner,
    __in_opt LPCWSTR lpName
    );
#undef CreateMutexW
#define CreateMutexW RtcPalCreateMutexW

#undef CreateMutex
#ifdef UNICODE
#define CreateMutex  CreateMutexW
#else
#define CreateMutex  CreateMutexA
#endif // !UNICODE

#undef ReleaseMutex
#define ReleaseMutex RtcPalReleaseMutex

DWORD
RTCAPI
RtcPalResumeThread(
    __in HANDLE hThread
    );
#undef ResumeThread
#define ResumeThread RtcPalResumeThread

#undef SetThreadPriority
#define SetThreadPriority   RtcPalSetThreadPriority
#undef GetThreadPriority
#define GetThreadPriority   RtcPalGetThreadPriority

#undef _beginthreadex

#ifndef _UNIX_IPHONE
#define _beginthreadex      RtcPalWin32CreateThread
#else
#define _beginthreadex(security, stack_size, start_address, arglist, initflag, thrdaddr) ((uintptr_t)RtcPalWin32CreateThread((security), (stack_size), (LPTHREAD_START_ROUTINE)(start_address), (arglist), (initflag), (LPDWORD)(thrdaddr)))
#endif

#undef _endthreadex
#define _endthreadex      	RtcPalExitThread

DWORD
RTCAPI
RtcPalWaitForSingleObjectEx(
    __in HANDLE hHandle,
    __in DWORD dwMilliseconds,
    __in RTCPAL_BOOL bAlertable
    );
#undef WaitForSingleObjectEx
#define WaitForSingleObjectEx   RtcPalWaitForSingleObjectEx

DWORD
RTCAPI
RtcPalWin32WaitForMultipleObjects(
    __in DWORD nCount,
    __in_ecount(nCount) CONST HANDLE *lpHandles,
    __in RTCPAL_BOOL bWaitAll,
    __in DWORD dwMilliseconds
    );
#undef WaitForMultipleObjects
#define WaitForMultipleObjects  RtcPalWin32WaitForMultipleObjects

#undef WaitForMultipleObjectsEx
#define WaitForMultipleObjectsEx(a,b,c,d,e) WaitForMultipleObjects((a),(b),(c),(d))

DWORD
RTCAPI
RtcPalSignalObjectAndWait(
    __in HANDLE hObjectToSignal,
    __in HANDLE hObjectToWaitOn,
    __in DWORD dwMilliseconds,
    __in RTCPAL_BOOL bAlertable
    );
#undef SignalObjectAndWait
#define SignalObjectAndWait     RtcPalSignalObjectAndWait

#undef Sleep
#define Sleep RtcPalSleep


VOID
RTCAPI
RtcPalOutputDebugStringA(
    __in_opt LPCSTR lpOutputString
    );
#undef OutputDebugStringA
#define OutputDebugStringA RtcPalOutputDebugStringA
VOID
RTCAPI
RtcPalOutputDebugStringW(
    __in_opt LPCWSTR lpOutputString
    );
#undef OutputDebugStringW
#define OutputDebugStringW RtcPalOutputDebugStringW
#ifdef UNICODE
#define OutputDebugString  OutputDebugStringW
#else
#define OutputDebugString  OutputDebugStringA
#endif // !UNICODE

RTCPAL_BOOL
RTCAPI
RtcPalCloseHandle(
    __in HANDLE hObject
    );

#undef CloseHandle
#define CloseHandle RtcPalCloseHandle

#undef DebugBreak
#define DebugBreak RtcPalDebugBreak

#undef __debugbreak
#define __debugbreak RtcPalDebugBreak

RTCPAL_BOOL
RTCAPI
RtcPalGetProcessAffinityMask(
    __in  HANDLE hProcess,
    __out PDWORD_PTR lpProcessAffinityMask,
    __out PDWORD_PTR lpSystemAffinityMask
    );
#undef GetProcessAffinityMask
#define GetProcessAffinityMask RtcPalGetProcessAffinityMask

RTCPAL_BOOL
RTCAPI
RtcPalSetProcessAffinityMask(
    __in HANDLE hProcess,
    __in DWORD_PTR dwProcessAffinityMask
    );
#undef SetProcessAffinityMask
#define SetProcessAffinityMask RtcPalSetProcessAffinityMask

uintptr_t
RTCAPI
RtcPalSetThreadAffinityMask(
    __in HANDLE hThread,
    __in uintptr_t dwAffinityMask
    );
#undef SetThreadAffinityMask
#define SetThreadAffinityMask RtcPalSetThreadAffinityMask

/*
These are MSC CRT functions. Unfortunately, MACBU implement them in wlmkernel.
#undef _aligned_malloc
#define _aligned_malloc rtcpal_aligned_malloc

#undef _aligned_free
#define _aligned_free rtcpal_aligned_free
*/

#undef AfxGetCurrentThread
#define AfxGetCurrentThread RtcPalGetCurrentThread
#undef GetCurrentThread
#define GetCurrentThread    RtcPalGetCurrentThread

//#undef GetCurrentThreadId
//#define GetCurrentThreadId  RtcPalGetCurrentThreadId

__out
HANDLE
RTCAPI
RtcPalGetCurrentProcess(
    VOID
    );
#undef GetCurrentProcess
#define GetCurrentProcess       RtcPalGetCurrentProcess

#undef GetCurrentProcessId
#define GetCurrentProcessId RtcPalGetCurrentProcessId

#undef GetModuleFileNameA
#define GetModuleFileNameA RtcPalGetModuleFileNameA
#undef GetModuleFileNameW
#define GetModuleFileNameW RtcPalGetModuleFileNameW
#ifdef UNICODE
#define GetModuleFileName  GetModuleFileNameW
#else
#define GetModuleFileName  GetModuleFileNameA
#endif // !UNICODE

#undef GetModuleHandleA
#define GetModuleHandleA RtcPalGetModuleHandleA
#undef GetModuleHandleW
#define GetModuleHandleW RtcPalGetModuleHandleW
#ifdef UNICODE
#define GetModuleHandle  GetModuleHandleW
#else
#define GetModuleHandle  GetModuleHandleA
#endif // !UNICODE

#ifdef ANDROID
// Under android there is no buildin API of IsDebuggerPresent
inline BOOL WINAPI IsDebuggerPresent(void) { return FALSE; }
inline void __noop(void) {}
#endif

// Functions for SLIST
void WINAPI InitializeSListHead(
  PSLIST_HEADER ListHead
);

PSLIST_ENTRY WINAPI InterlockedPopEntrySList(
  PSLIST_HEADER ListHead
);

PSLIST_ENTRY WINAPI InterlockedPushEntrySList(
  PSLIST_HEADER ListHead,
  PSLIST_ENTRY ListEntry
);

#ifdef __cplusplus
};
#endif
#endif // _WINBASE_

