
// WINDOWS declarations exposed from RTCPAL

#pragma once

#include "rtcpal.h"

#ifdef __cplusplus
extern "C" {
#endif

#define FAR
#define far
#define NEAR
#define CONST               const

typedef intptr_t (RTCAPI * FARPROC)();

#ifndef __in
    #define __in
    #define __in_opt
    #define __out
    #define __out_opt
    #define __inout
    #define __in_ecount(x)
    #define __in_bcount(x)
    #define __out_ecount(x)
    #define __out_ecount_part(size,len)
    #define __out_ecount_full(x)
    #define __out_bcount(x)
    #define __out_bcount_full(x)
    #define __deref_out
    #define __deref_out_ecount(x)
    #define __deref_out_bcount(x)
#endif

#ifndef __in_range
    #define __in_range(a,b)
    #define __field_range(x,y)
    #define __drv_interlocked
#endif

#ifndef C_ASSERT
#define C_ASSERT    RTCPAL_C_ASSERT
#endif

#ifndef FIELD_OFFSET
#define FIELD_OFFSET    RTCPAL_FIELD_OFFSET
#endif

#ifndef UNREFERENCED_PARAMETER
#define UNREFERENCED_PARAMETER RTCPAL_UNREFERENCED_PARAMETER
#endif

#ifndef ROUND_UP_COUNT
#define ROUND_UP_COUNT      RTCPAL_ROUND_UP_COUNT
#endif

#ifndef ROUND_UP_POINTER
#define ROUND_UP_POINTER    RTCPAL_ROUND_UP_POINTER
#endif

#ifndef FORCEINLINE
#define FORCEINLINE     RTCPAL_FORCEINLINE
#endif

#define DUMMYSTRUCTNAME

//
// All WINDOWS primitive data types are derived from POSIX data types
// Before media stack is converted to POSIX compliant, we have to keep these WINDOWS junks.
//
typedef uint8_t BYTE;
typedef uint8_t byte;
typedef uint8_t UCHAR;
typedef UCHAR * PUCHAR;
typedef char * PSZ;
typedef BYTE * PBYTE;
typedef BYTE * LPBYTE;
typedef BYTE BOOLEAN;
typedef BYTE * PBOOLEAN;
typedef uint8_t UINT8;
typedef uint8_t * PUINT8;
typedef int8_t INT8;
typedef int8_t * PINT8;

#ifndef ANDROID
typedef uint16_t u_short;
#endif
#ifndef __linux
typedef uint16_t u_short;
#endif
typedef uint16_t WORD;
typedef WORD * PWORD;
typedef WORD * LPWORD;
typedef uint16_t USHORT;
typedef USHORT * PUSHORT;
typedef uint16_t UINT16;
typedef UINT16 * PUINT16;

typedef int16_t SHORT;
typedef SHORT * PSHORT;
typedef int16_t INT16;
typedef INT16 * PINT16;

typedef uint32_t DWORD32;
typedef DWORD32 * PDWORD32;
typedef DWORD32 * LPDWORD32;
typedef uint32_t ULONG32;
typedef ULONG32 * PULONG32;

typedef int32_t LONG32, * PLONG32;

// To be compatible with MACBU shim
#if 1
// To make Linux ETL binary compatitle with windows tools
typedef uint32_t DWORD;
typedef uint32_t ULONG;
typedef int32_t LONG;
#else
typedef unsigned long DWORD;
typedef unsigned long ULONG;
typedef long LONG;
#endif


typedef DWORD * PDWORD;
typedef DWORD * LPDWORD;
typedef ULONG * PULONG;
typedef LONG * LPLONG;
typedef LONG * PLONG;

typedef uint32_t UINT32;
typedef UINT32 * PUINT32;
typedef UINT32 * LPUINT32;

typedef LONG HRESULT;
#define _HRESULT_DEFINED
typedef int32_t INT32;
typedef INT32 * PINT32;
typedef INT32 * LPINT32;

typedef int64_t LONGLONG;
typedef LONGLONG * PLONGLONG;
typedef LONGLONG INT64;
typedef int64_t * PINT64;
typedef LONGLONG LONG64;
typedef LONG64 * PLONG64;

typedef uint64_t ULONG64;
typedef uint64_t * PULONG64;
typedef uint64_t ULONGLONG;
typedef ULONGLONG * PULONGLONG;
typedef ULONGLONG DWORD64;
typedef DWORD64 * PDWORD64;
typedef DWORD64 * LPDWORD64;
typedef ULONGLONG UINT64;
typedef UINT64 * PUINT64;
typedef UINT64 * LPUINT64;

#ifndef QWORD_DEFINED
#define QWORD_DEFINED
typedef ULONGLONG QWORD;
#endif

typedef size_t SIZE_T;

typedef union _LARGE_INTEGER {
    struct {
        uint32_t LowPart;
        int32_t HighPart;
    } DUMMYSTRUCTNAME;
    struct {
        uint32_t LowPart;
        int32_t HighPart;
    } u;
    LONGLONG QuadPart;
} LARGE_INTEGER;

typedef LARGE_INTEGER *PLARGE_INTEGER;

typedef union _ULARGE_INTEGER {
    struct {
        uint32_t LowPart;
        uint32_t HighPart;
    } DUMMYSTRUCTNAME;
    struct {
        uint32_t LowPart;
        uint32_t HighPart;
    } u;
    ULONGLONG QuadPart;
} ULARGE_INTEGER;

typedef int64_t REFERENCE_TIME;

typedef ULARGE_INTEGER *PULARGE_INTEGER;

// typedef int RTCPAL_BOOL;
typedef RTCPAL_BOOL * PRTCPAL_BOOL;
typedef RTCPAL_BOOL * LPRTCPAL_BOOL;

#ifndef __OBJC__
    typedef RTCPAL_BOOL BOOL;
    typedef RTCPAL_BOOL * PBOOL;
    typedef RTCPAL_BOOL * LPBOOL;
#endif

typedef int INT;
typedef int * PINT;
typedef int * LPINT;
typedef unsigned int UINT;
typedef unsigned int * PUINT;

typedef intptr_t INT_PTR;
typedef uintptr_t UINT_PTR;

typedef intptr_t LONG_PTR;
typedef uintptr_t ULONG_PTR;
typedef uintptr_t * PULONG_PTR;
typedef ULONG_PTR DWORD_PTR;
typedef ULONG_PTR * PDWORD_PTR;

typedef float FLOAT;
typedef FLOAT * PFLOAT;
typedef double DOUBLE;

#ifndef VOID
    // Use #define instead of typedef due to different behaviors in MSFT C and GCC.
    //
    // void keyword has dual role.
    //
    // 1) Used as a data type, such as "void *"
    // 2) Used for indicating a function doesn't have any parameter in C (not C++).
    //    int f(void) - a function taking no parameter and returning an integer
    //    int f() - a function taking any number of parameters and returning an integer.
    //              Note: C++ interpret this differently.
    //
    // MSFT C and GCC compiler handles the 2nd role differently. GCC expects seeing
    // keyword "void" for indicating a function which doesn't take any parameter.
    // It will raise an error and stop when it sees a typedef version.
    // GCC will accept
    //
    //      int f(VOID *);
    //      int f(void *);
    //      int f(void);
    //
    // It will reject
    //      int f(VOID);
    //
    #define VOID void
#endif

typedef void * PVOID;
typedef void * LPVOID;
typedef CONST void * LPCVOID;

typedef char CHAR;
typedef CHAR * LPSTR;
typedef const CHAR * LPCSTR, * PCSTR;
typedef CHAR * PCHAR, * PSTR;

#if !RTCPAL_USHORT_WCHAR
typedef wchar_t WCHAR;
#else
typedef unsigned short WCHAR;
#endif

typedef WCHAR * LPWSTR, * PWCHAR, * PWSTR;
typedef const WCHAR * LPCWSTR;
typedef const WCHAR * PCWSTR;

//
// To make binary compatability, GCC shouldn't use -fshort-wchar.
//

#if (WCHAR_MAX > 0xFFFFu)
C_ASSERT(sizeof(WCHAR) == 4);
#else
C_ASSERT(sizeof(WCHAR) == 2);
#if !RTCPAL_USHORT_WCHAR
C_ASSERT(sizeof(L'\0') == 2);
#endif
#endif // WCHAR_MAX

#define MAKEWORD(a, b)      ((WORD)(((BYTE)(((DWORD_PTR)(a)) & 0xff)) | ((WORD)((BYTE)(((DWORD_PTR)(b)) & 0xff))) << 8))
#define MAKELONG(a, b)      ((LONG)(((WORD)(((DWORD_PTR)(a)) & 0xffff)) | ((DWORD)((WORD)(((DWORD_PTR)(b)) & 0xffff))) << 16))
#define LOWORD(l)           ((WORD)(((DWORD_PTR)(l)) & 0xffff))
#define HIWORD(l)           ((WORD)((((DWORD_PTR)(l)) >> 16) & 0xffff))
#define LOBYTE(w)           ((BYTE)(((DWORD_PTR)(w)) & 0xff))
#define HIBYTE(w)           ((BYTE)((((DWORD_PTR)(w)) >> 8) & 0xff))

typedef void * HANDLE;
typedef HANDLE * LPHANDLE, * PHANDLE;
#define INVALID_HANDLE_VALUE    ((HANDLE)(LONG_PTR)-1)

typedef void * HKEY;
typedef HKEY * PHKEY;

#define ZeroMemory(p,s) memset(p, 0, s)

#define TRUE    1
#define FALSE   0


//
// Calculate the size of a field in a structure of type type, without
// knowing or stating the type of the field.
//
#define RTL_FIELD_SIZE(type, field) (sizeof(((type *)0)->field))

//
// Calculate the size of a structure of type type up through and
// including a field.
//
#define RTL_SIZEOF_THROUGH_FIELD(type, field) \
    (FIELD_OFFSET(type, field) + RTL_FIELD_SIZE(type, field))

//
//  RTL_CONTAINS_FIELD usage:
//
//      if (RTL_CONTAINS_FIELD(pBlock, pBlock->cbSize, dwMumble)) { // safe to use pBlock->dwMumble
//
#define RTL_CONTAINS_FIELD(Struct, Size, Field) \
    ( (((PCHAR)(&(Struct)->Field)) + sizeof((Struct)->Field)) <= (((PCHAR)(Struct))+(Size)) )

//
// Return the number of elements in a statically sized array.
//   DWORD Buffer[100];
//   RTL_NUMBER_OF(Buffer) == 100
// This is also popularly known as: NUMBER_OF, ARRSIZE, _countof, NELEM, etc.
//
#define RTL_NUMBER_OF_V1(A) (sizeof(A)/sizeof((A)[0]))

#ifdef __cplusplus
};
#endif

#if defined(__cplusplus) && \
    !defined(MIDL_PASS) && \
    !defined(RC_INVOKED) && \
    !defined(_PREFAST_) && \
    (_MSC_FULL_VER >= 13009466) && \
    !defined(SORTPP_PASS)
//
// RtlpNumberOf is a function that takes a reference to an array of N Ts.
//
// typedef T array_of_T[N];
// typedef array_of_T &reference_to_array_of_T;
//
// RtlpNumberOf returns a pointer to an array of N chars.
// We could return a reference instead of a pointer but older compilers do not accept that.
//
// typedef char array_of_char[N];
// typedef array_of_char *pointer_to_array_of_char;
//
// sizeof(array_of_char) == N
// sizeof(*pointer_to_array_of_char) == N
//
// pointer_to_array_of_char RtlpNumberOf(reference_to_array_of_T);
//
// We never even call RtlpNumberOf, we just take the size of dereferencing its return type.
// We do not even implement RtlpNumberOf, we just decare it.
//
// Attempts to pass pointers instead of arrays to this macro result in compile time errors.
// That is the point.
//
extern "C++" // templates cannot be declared to have 'C' linkage
template <typename T, size_t N>
char (*RtlpNumberOf( UNALIGNED T (&)[N] ))[N];


#define RTL_NUMBER_OF_V2(A) (sizeof(*RtlpNumberOf(A)))

//
// This does not work with:
//
// void Foo()
// {
//    struct { int x; } y[2];
//    RTL_NUMBER_OF_V2(y); // illegal use of anonymous local type in template instantiation
// }
//
// You must instead do:
//
// struct Foo1 { int x; };
//
// void Foo()
// {
//    Foo1 y[2];
//    RTL_NUMBER_OF_V2(y); // ok
// }
//
// OR
//
// void Foo()
// {
//    struct { int x; } y[2];
//    RTL_NUMBER_OF_V1(y); // ok
// }
//
// OR
//
// void Foo()
// {
//    struct { int x; } y[2];
//    _ARRAYSIZE(y); // ok
// }
//

#else
#define RTL_NUMBER_OF_V2(A) RTL_NUMBER_OF_V1(A)
#endif

#ifdef ENABLE_RTL_NUMBER_OF_V2
#define RTL_NUMBER_OF(A) RTL_NUMBER_OF_V2(A)
#else
#define RTL_NUMBER_OF(A) RTL_NUMBER_OF_V1(A)
#endif


#ifdef __cplusplus
extern "C" {
#endif

//
// ARRAYSIZE is more readable version of RTL_NUMBER_OF_V2, and uses
// it regardless of ENABLE_RTL_NUMBER_OF_V2
//
// _ARRAYSIZE is a version useful for anonymous types
//
#define ARRAYSIZE(A)    RTL_NUMBER_OF_V2(A)
#define _ARRAYSIZE(A)   RTL_NUMBER_OF_V1(A)

//
// An expression that yields the type of a field in a struct.
//
#define RTL_FIELD_TYPE(type, field) (((type*)0)->field)

// RTL_ to avoid collisions in the global namespace.
//
// Given typedef struct _FOO { BYTE Bar[123]; } FOO;
// RTL_NUMBER_OF_FIELD(FOO, Bar) == 123
//
#define RTL_NUMBER_OF_FIELD(type, field) (RTL_NUMBER_OF(RTL_FIELD_TYPE(type, field)))

//
// eg:
// typedef struct FOO {
//   DWORD Integer;
//   PVOID Pointer;
// } FOO;
//
// RTL_PADDING_BETWEEN_FIELDS(FOO, Integer, Pointer) == 0 for Win32, 4 for Win64
//
#define RTL_PADDING_BETWEEN_FIELDS(T, F1, F2) \
    ((FIELD_OFFSET(T, F2) > FIELD_OFFSET(T, F1)) \
        ? (FIELD_OFFSET(T, F2) - FIELD_OFFSET(T, F1) - RTL_FIELD_SIZE(T, F1)) \
        : (FIELD_OFFSET(T, F1) - FIELD_OFFSET(T, F2) - RTL_FIELD_SIZE(T, F2)))

// RTL_ to avoid collisions in the global namespace.
#if defined(__cplusplus)
#define RTL_CONST_CAST(type) const_cast<type>
#else
#define RTL_CONST_CAST(type) (type)
#endif


// like sizeof
// usually this would be * CHAR_BIT, but we don't necessarily have #include <limits.h>
#define RTL_BITS_OF(sizeOfArg) (sizeof(sizeOfArg) * 8)

#define RTL_BITS_OF_FIELD(type, field) (RTL_BITS_OF(RTL_FIELD_TYPE(type, field)))

//
// Calculate the address of the base of the structure given its type, and an
// address of a field within the structure.
//
#ifndef CONTAINING_RECORD
    #define CONTAINING_RECORD(address, type, field) ((type *)( \
                                                  (PCHAR)(address) - FIELD_OFFSET(type,field)))
#endif

#define IN
#define OUT
#define INOUT
#define OPTIONAL

// #ifndef max
// #define max(a,b)            (((a) > (b)) ? (a) : (b))
// #endif

//#ifndef min
//#define min(a,b)            (((a) < (b)) ? (a) : (b))
//#endif

#define NI_MAXHOST      1025  /* Max size of a fully-qualified domain name */

typedef LONG NTSTATUS;

#ifndef NULL
    #define NULL 0
#endif

#define INFINITE            0xFFFFFFFF  // Infinite timeout

typedef struct RTCPAL_ALIGNED(16) _SLIST_ENTRY {
    struct _SLIST_ENTRY * Next;
} SLIST_ENTRY, * PSLIST_ENTRY;

#if defined(_UNIX_MACOSX)
typedef OSQueueHead SLIST_HEADER, *PSLIST_HEADER;
#else
// TODO: implement SLIST manipulation functions
typedef struct RTCPAL_ALIGNED(16) _SLIST_HEADER {
    uint32_t u32Size;
    PSLIST_ENTRY First;
} SLIST_HEADER, * PSLIST_HEADER;
#endif

struct _LIST_ENTRY {
    struct _LIST_ENTRY * Flink;
    struct _LIST_ENTRY * Blink;
};
typedef struct _LIST_ENTRY LIST_ENTRY, * PLIST_ENTRY;

// begin - required by objidl.h
typedef void * HWND;
typedef void * HGLOBAL;
typedef void * HTASK;
typedef void * HENHMETAFILE;
typedef void * HBITMAP;
// end - required by objidl.h

// begin - required by objbase.h
#define __stdcall
typedef void * PSECURITY_DESCRIPTOR, * LPSECURITY_DESCRIPTOR;
typedef void * HINSTANCE;
// end - required by objbase.h

// begin - required oleidl.h
typedef void * HMENU;
typedef void * HACCEL;
// end - required oleidl.h

// begin - required urlmon.h
#define CALLBACK
// end - required urlmon.h

// begin - required ocidl.h
typedef UINT_PTR WPARAM;
typedef LONG_PTR LPARAM;
typedef LONG_PTR LRESULT;
typedef void * HFONT;
typedef void * HRGN;
typedef void * HPALETTE;
typedef uintptr_t HANDLE_PTR;

// Currently, we need to be consistent with MACBU headers
// TODO: use gcc nothrow so that we can be consistent with the real semantics
#define DECLSPEC_NOTHROW

#ifdef UNICODE
typedef struct tagTEXTMETRICW TEXTMETRIC;
#else
typedef struct tagTEXTMETRICA TEXTMETRIC;
#endif
// end - required ocidl.h

// begin - required strif.h
typedef DWORD COLORREF;
typedef HANDLE HMONITOR;
// end - required strif.h

// begin - required ddraw.h
#define PASCAL
// end - required ddraw.h

// begin - required mmsystem.h
#define DECLARE_HANDLE(name) struct name##__; typedef struct name##__ *name
// end - required mmsystem.h

// begin - required olectl.h
typedef HINSTANCE HMODULE;
// end - required olectl.h

typedef WORD ATOM;

#define MAX_PATH          260

#ifdef RTCPAL_ARCH_X64
    #define MEMORY_ALLOCATION_ALIGNMENT 16
#else
    #define MEMORY_ALLOCATION_ALIGNMENT 8
#endif

/*
typedef struct _STRING {
    __maybevalid USHORT Length;
    __maybevalid USHORT MaximumLength;
    __field_bcount_part_opt(MaximumLength, Length) PCHAR Buffer;
} STRING;
typedef STRING *PSTRING;

typedef STRING ANSI_STRING;
typedef PSTRING PANSI_STRING;
*/

#ifdef __cplusplus
};
#endif

