#ifndef _WINREG_
#define _WINREG_

#ifdef __cplusplus
extern "C" {
#endif

// defined in winreg.h, but it might be referenced elsewhere like shlwapi.h
typedef LONG LSTATUS;
typedef ACCESS_MASK REGSAM;

//
// Registry API
//
#undef HKEY_CLASSES_ROOT
#define HKEY_CLASSES_ROOT                   (( HKEY ) (ULONG_PTR)((LONG)0x80000000) )
#define HKEY_CURRENT_USER                   (( HKEY ) (ULONG_PTR)((LONG)0x80000001) )
#define HKEY_LOCAL_MACHINE                  (( HKEY ) (ULONG_PTR)((LONG)0x80000002) )
#define HKEY_USERS                          (( HKEY ) (ULONG_PTR)((LONG)0x80000003) )
#define HKEY_PERFORMANCE_DATA               (( HKEY ) (ULONG_PTR)((LONG)0x80000004) )
#define HKEY_PERFORMANCE_TEXT               (( HKEY ) (ULONG_PTR)((LONG)0x80000050) )
#define HKEY_PERFORMANCE_NLSTEXT            (( HKEY ) (ULONG_PTR)((LONG)0x80000060) )
#define HKEY_CURRENT_CONFIG                 (( HKEY ) (ULONG_PTR)((LONG)0x80000005) )
#define HKEY_DYN_DATA                       (( HKEY ) (ULONG_PTR)((LONG)0x80000006) )
#define HKEY_CURRENT_USER_LOCAL_SETTINGS    (( HKEY ) (ULONG_PTR)((LONG)0x80000007) )

#define RTCPAL_HKEY_CLASSES_ROOT                            (( HKEY ) (ULONG_PTR)((LONG)0x80000000) )
#define RTCPAL_HKEY_CURRENT_USER                            (( HKEY ) (ULONG_PTR)((LONG)0x80000001) )
#define RTCPAL_HKEY_LOCAL_MACHINE                           (( HKEY ) (ULONG_PTR)((LONG)0x80000002) )
#define RTCPAL_HKEY_USERS                                   (( HKEY ) (ULONG_PTR)((LONG)0x80000003) )
#define RTCPAL_HKEY_PERFORMANCE_DATA                        (( HKEY ) (ULONG_PTR)((LONG)0x80000004) )
#define RTCPAL_HKEY_PERFORMANCE_TEXT                        (( HKEY ) (ULONG_PTR)((LONG)0x80000050) )
#define RTCPAL_HKEY_PERFORMANCE_NLSTEXT                     (( HKEY ) (ULONG_PTR)((LONG)0x80000060) )
#define RTCPAL_HKEY_CURRENT_CONFIG                          (( HKEY ) (ULONG_PTR)((LONG)0x80000005) )
#define RTCPAL_HKEY_DYN_DATA                                (( HKEY ) (ULONG_PTR)((LONG)0x80000006) )
#define RTCPAL_HKEY_CURRENT_USER_LOCAL_SETTINGS             (( HKEY ) (ULONG_PTR)((LONG)0x80000007) )

#define RTCPAL_HKEY_RESERVED_MASK           0xfffffff0u
#define RTCPAL_HKEY_RESERVED                0x80000000u

#define RTCPAL_KEY_READ                                     0x00000001
#define RTCPAL_KEY_WRITE                                    0x00000002

// These should map 1:1 to Win32's HKEY_*.
#define RTCPAL_REG_NONE                                     ( 0 )   // No value type
#define RTCPAL_REG_SZ                                       ( 1 )   // Unicode nul terminated string
#define RTCPAL_REG_EXPAND_SZ                                ( 2 )   // Unicode nul terminated string
                                                                    // (with environment variable references)
#define RTCPAL_REG_BINARY                                   ( 3 )   // Free form binary
#define RTCPAL_REG_DWORD                                    ( 4 )   // 32-bit number
#define RTCPAL_REG_DWORD_LITTLE_ENDIAN                      ( 4 )   // 32-bit number (same as REG_DWORD)
#define REG_DWORD_BIG_ENDIAN                                ( 5 )   // 32-bit number
#define RTCPAL_REG_LINK                                     ( 6 )   // Symbolic Link (unicode)
#define RTCPAL_REG_MULTI_SZ                                 ( 7 )   // Multiple Unicode strings
#define RTCPAL_REG_RESOURCE_LIST                            ( 8 )   // Resource list in the resource map
#define RTCPAL_REG_FULL_RESOURCE_DESCRIPTOR                 ( 9 )  // Resource list in the hardware description
#define RTCPAL_REG_RESOURCE_REQUIREMENTS_LIST               ( 10 )
#define RTCPAL_REG_QWORD                                    ( 11 )  // 64-bit number
#define RTCPAL_REG_QWORD_LITTLE_ENDIAN                      ( 11 )  // 64-bit number (same as REG_QWORD)

#define REG_NONE                    ( 0 )   // No value type
#define REG_SZ                      ( 1 )   // Unicode nul terminated string
#define REG_EXPAND_SZ               ( 2 )   // Unicode nul terminated string
#define REG_BINARY                  ( 3 )   // Free form binary
#define REG_DWORD                   ( 4 )   // 32-bit number
#define REG_QWORD                   ( 11 )  // 64-bit number
#define REG_OPTION_NON_VOLATILE     (0x00000000L)   // Key is preserved

RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyA(
    __in HKEY hKey,
    __in_opt LPCSTR lpSubKey,
    __out PHKEY phkResult
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyW(
    __in HKEY hKey,
    __in_opt LPCWSTR lpSubKey,
    __out PHKEY phkResult
    );
#ifdef UNICODE
#define RegCreateKey  RtcPalRegCreateKeyW
#else
#define RegCreateKey  RtcPalRegCreateKeyA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyExA (
    __in HKEY hKey,
    __in LPCSTR lpSubKey,
    __reserved DWORD Reserved,
    __in_opt LPSTR lpClass,
    __in DWORD dwOptions,
    __in REGSAM samDesired,
    __in_opt CONST LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __out PHKEY phkResult,
    __out_opt LPDWORD lpdwDisposition
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegCreateKeyExW (
    __in HKEY hKey,
    __in LPCWSTR lpSubKey,
    __reserved DWORD Reserved,
    __in_opt LPWSTR lpClass,
    __in DWORD dwOptions,
    __in REGSAM samDesired,
    __in_opt CONST LPSECURITY_ATTRIBUTES lpSecurityAttributes,
    __out PHKEY phkResult,
    __out_opt LPDWORD lpdwDisposition
    );
#undef RegCreateKeyExA
#define RegCreateKeyExA RtcPalRegCreateKeyExA
#undef RegCreateKeyExW
#define RegCreateKeyExW RtcPalRegCreateKeyExW
#undef RegCreateKeyEx
#ifdef UNICODE
#define RegCreateKeyEx  RtcPalRegCreateKeyExW
#else
#define RegCreateKeyEx  RtcPalRegCreateKeyExA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegDeleteKeyA (
    __in HKEY hKey,
    __in LPCSTR lpSubKey
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegDeleteKeyW (
    __in HKEY hKey,
    __in LPCWSTR lpSubKey
    );
#undef RegDeleteKey
#ifdef UNICODE
#define RegDeleteKey  RtcPalRegDeleteKeyW
#else
#define RegDeleteKey  RtcPalRegDeleteKeyA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueExA (
    __in HKEY hKey,
    __in_opt LPCSTR lpValueName,
    __reserved DWORD Reserved,
    __in DWORD dwType,
    __in_bcount_opt(cbData) CONST BYTE* lpData,
    __in DWORD cbData
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueExW (
    __in HKEY hKey,
    __in_opt LPCWSTR lpValueName,
    __reserved DWORD Reserved,
    __in DWORD dwType,
    __in_bcount_opt(cbData) CONST BYTE* lpData,
    __in DWORD cbData
    );
#undef RegSetValueExA
#define RegSetValueExA RtcPalRegSetValueExA
#undef RegSetValueExW
#define RegSetValueExW RtcPalRegSetValueExW
#undef RegSetValueEx
#ifdef UNICODE
#define RegSetValueEx  RtcPalRegSetValueExW
#else
#define RegSetValueEx  RtcPalRegSetValueExA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueA(
    __in HKEY hKey,
    __in_opt LPCSTR lpSubKey,
    __in DWORD dwType,
    __in_bcount_opt(cbData) LPCSTR lpData,
    __in DWORD cbData
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegSetValueW(
    __in HKEY hKey,
    __in_opt LPCWSTR lpSubKey,
    __in DWORD dwType,
    __in_bcount_opt(cbData) LPCWSTR lpData,
    __in DWORD cbData
    );

#ifdef UNICODE
#define RegSetValue  RtcPalRegSetValueW
#else
#define RegSetValue  RtcPalRegSetValueA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegDeleteValueA(
    __in HKEY hKey,
    __in_opt LPCSTR lpValueName
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegDeleteValueW(
    __in HKEY hKey,
    __in_opt LPCWSTR lpValueName
    );
#ifdef UNICODE
#define RegDeleteValue  RtcPalRegDeleteValueW
#else
#define RegDeleteValue  RtcPalRegDeleteValueA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegEnumKeyExA(
    __in HKEY hKey,
    __in DWORD dwIndex,
    __out_ecount_part_opt(*lpcchName, *lpcchName + 1) LPSTR lpName,
    __inout LPDWORD lpcchName,
    __reserved LPDWORD lpReserved,
    __out_ecount_part_opt(*lpcchClass,*lpcchClass + 1) LPSTR lpClass,
    __inout_opt LPDWORD lpcchClass,
    __out_opt PFILETIME lpftLastWriteTime
    );
#undef RegEnumKeyExA
#define RegEnumKeyExA   RtcPalRegEnumKeyExA
RTCPAL_ERROR
RTCAPI
RtcPalRegEnumKeyExW (
    __in HKEY hKey,
    __in DWORD dwIndex,
    __out_ecount_part_opt(*lpcchName, *lpcchName + 1) LPWSTR lpName,
    __inout LPDWORD lpcchName,
    __reserved LPDWORD lpReserved,
    __out_ecount_part_opt(*lpcchClass,*lpcchClass + 1) LPWSTR lpClass,
    __inout_opt LPDWORD lpcchClass,
    __out_opt PFILETIME lpftLastWriteTime
    );
#undef RegEnumKeyExW
#define RegEnumKeyExW   RtcPalRegEnumKeyExW
#ifdef UNICODE
#define RegEnumKeyEx  RtcPalRegEnumKeyExW
#else
#define RegEnumKeyEx  RtcPalRegEnumKeyExA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegFlushKey(
    __in HKEY hKey
    );
#undef RegFlushKey
#define RegFlushKey RtcPalRegFlushKey

RTCPAL_ERROR
RTCAPI
RtcPalRegNotifyChangeKeyValue(
    __in HKEY hKey,
    __in RTCPAL_BOOL bWatchSubtree,
    __in DWORD dwNotifyFilter,
    __in_opt HANDLE hEvent,
    __in RTCPAL_BOOL fAsynchronous
    );
#undef RegNotifyChangeKeyValue
#define RegNotifyChangeKeyValue RtcPalRegNotifyChangeKeyValue

RTCPAL_ERROR
RTCAPI
RtcPalRegGetKeySecurity (
    __in HKEY hKey,
    __in SECURITY_INFORMATION SecurityInformation,
    __out_bcount_opt(*lpcbSecurityDescriptor) PSECURITY_DESCRIPTOR pSecurityDescriptor,
    __inout LPDWORD lpcbSecurityDescriptor
    );
#undef RtcPalRegGetKeySecurity
#define RegGetKeySecurity RtcPalRegGetKeySecurity

RTCPAL_ERROR
RTCAPI
RtcPalRegSetKeySecurity (
    __in HKEY hKey,
    __in SECURITY_INFORMATION SecurityInformation,
    __in PSECURITY_DESCRIPTOR pSecurityDescriptor
    );
#undef RtcPalRegSetKeySecurity
#define RegSetKeySecurity RtcPalRegSetKeySecurity

RTCPAL_ERROR
RTCAPI
RtcPalRegOpenKeyExA (
    __in HKEY hKey,
    __in_opt LPCSTR lpSubKey,
    __in_opt DWORD ulOptions,
    __in REGSAM samDesired,
    __out PHKEY phkResult
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegOpenKeyExW (
    __in HKEY hKey,
    __in_opt LPCWSTR lpSubKey,
    __in_opt DWORD ulOptions,
    __in REGSAM samDesired,
    __out PHKEY phkResult
    );

#undef RegOpenKeyExA
#define RegOpenKeyExA RtcPalRegOpenKeyExA
#undef RegOpenKeyExW
#define RegOpenKeyExW RtcPalRegOpenKeyExW
#undef RegOpenKeyEx
#ifdef UNICODE
#define RegOpenKeyEx  RtcPalRegOpenKeyExW
#else
#define RegOpenKeyEx  RtcPalRegOpenKeyExA
#endif // !UNICODE

RTCPAL_ERROR
RTCAPI
RtcPalRegCloseKey (
    __in HKEY hKey
    );
#undef RegCloseKey
#define RegCloseKey RtcPalRegCloseKey

RTCPAL_ERROR
RTCAPI
RtcPalRegQueryValueExA(
    __in HKEY hKey,
    __in_opt LPCSTR lpValueName,
    __reserved LPDWORD lpReserved,
    __out_opt LPDWORD lpType,
    __out_bcount_part_opt(*lpcbData, *lpcbData) __out_data_source(REGISTRY) LPBYTE lpData,
    __inout_opt LPDWORD lpcbData
    );
RTCPAL_ERROR
RTCAPI
RtcPalRegQueryValueExW(
    __in HKEY hKey,
    __in_opt LPCWSTR lpValueName,
    __reserved LPDWORD lpReserved,
    __out_opt LPDWORD lpType,
    __out_bcount_part_opt(*lpcbData, *lpcbData) __out_data_source(REGISTRY) LPBYTE lpData,
    __inout_opt LPDWORD lpcbData
    );
#undef RegQueryValueExA
#define RegQueryValueExA    RtcPalRegQueryValueExA

#undef RegQueryValueExW
#define RegQueryValueExW    RtcPalRegQueryValueExW

#undef RegQueryValueEx
#ifdef UNICODE
#define RegQueryValueEx  RegQueryValueExW
#else
#define RegQueryValueEx  RegQueryValueExA
#endif // !UNICODE


#ifdef __cplusplus
};
#endif

#endif // _WINREG_

