#pragma once
#ifndef __MMVR_AIRSPACE_EXTERNAL_API__
#define __MMVR_AIRSPACE_EXTERNAL_API__








#include <mmvr_Common.h>




#ifdef __cplusplus
extern "C"  {
#endif

// ============================================================================
// MMVR Companion Supporting AirSpace rendering
// ============================================================================

// ----------------------------------------------------------------------------
// HANDLES Specific to the AirSpace API's
// ----------------------------------------------------------------------------

// forward declaration, so this file can be used without including any of our internal headers.
struct MMVRAirSpaceRenderer;

typedef LONG   MMVRAIRSPACETEXTUREHANDLE;
typedef void * HMMVRAIRSPACED3DDEVICE;

typedef struct MMVRAirSpaceRenderer*  HMMVRAIRSPACERENDERER;



// -----------------------------------------------------------------------------
// PMMVRDEVICELOSTCALLBACK function prototype
//
// Callback function to indicate that AirSpace device is lost.
// This is to be invoked by the MMVRAirSpaceInterface struct whenever
// the D3D device is lost.
// -----------------------------------------------------------------------------
typedef
HRESULT
(_stdcall *PMMVRDEVICELOSTCALLBACK)(
     __in  HMMVRAIRSPACERENDERER           hRenderer
);


typedef struct _MMVRAirSpaceInterface
{
    HRESULT (_stdcall *pfnInitialize)(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in  HMMVRAIRSPACERENDERER           hRenderer,
        __in  MMVRAIRSPACETEXTUREHANDLE       hFastUpdateTexture,
        __in  PMMVRDEVICELOSTCALLBACK         pfnDeviceLost
    );

    HRESULT (_stdcall *pfnRelease)(
        __in  struct _MMVRAirSpaceInterface  *pThis
    );

    HRESULT (_stdcall *pfnGetDimensions)(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __out_ecount(1) DWORD                *pdwWidth,
        __out_ecount(1) DWORD                *pdwHeight
    );

    BOOL (_stdcall *pfnHasTexture)(
        __in  struct _MMVRAirSpaceInterface  *pThis
    );

    BOOL (_stdcall *pfnGetCompositorAdapterLuid)(
        __out_ecount(dwLuidSize) BYTE        *pbLuid,
        __in DWORD                            dwLuidSize
    );

    BOOL (_stdcall *pfnBeginDrawD3D9)(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in HMMVRAIRSPACED3DDEVICE           hD3dDevice,
        __in DWORD                            dwMsToWait,
        __in void                           **ppTexture
    );

    void (_stdcall *pfnEndDraw)(
        __in  struct _MMVRAirSpaceInterface  *pThis
    );

    void (_stdcall *pfnProcessSceneNotifications)(
    );


    void                        *pContext;

    // add to the end to allow some level of binary compatibility between
    // old and new AirSpace MMVR DLL's
    BOOL (_stdcall *pfnBeginDrawD3D11)(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in HMMVRAIRSPACED3DDEVICE           hD3dDevice,
        __in DWORD                            dwMsToWait,
        __in void                           **ppTexture
    );

} MMVRAirSpaceInterface;





#ifdef __cplusplus
}
#endif





#ifdef __cplusplus
#ifndef MMVR_AIRSPACE_IMPLEMENTATION

// if running in a C++ compiler, we'll give a leg up to the user
// by providing a template class that has a default AirSpace interface



#include <AirSpaceHelpers.h>
#include <AirSpace_win32.h>

namespace MMVRAirSpace
{




// This helper class allows us to track DeviceLost events
class DeviceLostHandler: public AirSpace::IListener
{
public:
    DeviceLostHandler( PMMVRDEVICELOSTCALLBACK pfnCallback, HMMVRAIRSPACERENDERER hContext ): 
        m_pfnCallback(pfnCallback),
        m_hContext(hContext),
        m_iRefCount(1)
    {
    }

    AirSpace::uint __stdcall AddRef(void) throw()
    {
        return (++m_iRefCount);
    }

    AirSpace::uint __stdcall Release(void) throw()
    {
        AirSpace::uint newCount = --m_iRefCount;
        if( newCount < 1 )
        {
            delete this;
        }
        return newCount;
    }

    void OnNotification(AirSpace::Notification::Enum notification, AirSpace::INotificationContext*) override
    {
        if (notification != AirSpace::Notification::DeviceLost)
            return;

        m_pfnCallback( m_hContext );
    }


private:
    PMMVRDEVICELOSTCALLBACK  m_pfnCallback;
    HMMVRAIRSPACERENDERER    m_hContext;
    AirSpace::uint           m_iRefCount;
};



class Interface
{
public:
    Interface():
        m_bSceneInitialized(FALSE),
        m_pFUTexture(NULL),
        m_pDeviceLostHandler(NULL)
    {
        memset( &m_cInterface,  0, sizeof(m_cInterface) );
        m_cInterface.pfnInitialize                = Initialize;
        m_cInterface.pfnRelease                   = Release;
        m_cInterface.pfnGetDimensions             = GetDimensions;
        m_cInterface.pfnGetDimensions             = GetDimensions;
        m_cInterface.pfnHasTexture                = HasTexture;
        m_cInterface.pfnGetCompositorAdapterLuid  = GetCompositorAdapterLuid;
        m_cInterface.pfnBeginDrawD3D9             = BeginDrawD3D9;
        m_cInterface.pfnBeginDrawD3D11            = BeginDrawD3D11;
        m_cInterface.pfnProcessSceneNotifications = ProcessSceneNotifications;
        m_cInterface.pfnEndDraw                   = EndDraw;
        m_cInterface.pContext                     = this;
    }

    MMVRAirSpaceInterface *GetCInterface()
    {
        return &m_cInterface;
    }


    static HRESULT _stdcall Initialize(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in  HMMVRAIRSPACERENDERER           hRenderer,
        __in  MMVRAIRSPACETEXTUREHANDLE       hFastUpdateTexture,
        __in  PMMVRDEVICELOSTCALLBACK         pfnDeviceLost
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);

        AirSpace::ITexture* pTexture = NULL;
        BOOL bRet = FALSE;
        HRESULT hr = E_FAIL;


        // can't initialize again
        if( pOwner->m_bSceneInitialized )
        {
            hr = S_FALSE;
            goto Exit;
        }


        // Initialize AirSpace for this thread
        if (!AirSpace::InitializeScene())
        {
            goto Exit;
        }

        pOwner->m_bSceneInitialized = TRUE;

        if( hFastUpdateTexture != NULL )
        {
            AirSpace::GetScene()->GetSharedTextureFactory().CreateSharedTexture(hFastUpdateTexture, &pTexture);
        }

        if ((pTexture == NULL) || (pTexture->GetTextureType() != AirSpace::TextureType::Enum::SwapChain))
        {
            goto Exit;
        }

        // transfer texture ownership to pOwner
        pOwner->m_pFUTexture = (AirSpace::ISwapChainTexture *)pTexture;
        pTexture = NULL;

        // register for devicelost notifications
        pOwner->m_pDeviceLostHandler = new DeviceLostHandler(pfnDeviceLost, hRenderer);
        AirSpace::GetScene()->AddListener(*(pOwner->m_pDeviceLostHandler));
        bRet = TRUE;

    Exit:
        if( pTexture != NULL )
        {
            pTexture->Release();
        }
        return bRet;
    }

    static HRESULT _stdcall Release(
        __in  struct _MMVRAirSpaceInterface  *pThis
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);

        if( pOwner->m_pFUTexture != NULL )
        {
            pOwner->m_pFUTexture->Release();
            pOwner->m_pFUTexture = NULL;
        }
    
        if( pOwner->m_bSceneInitialized )
        {
            AirSpace::GetScene()->RemoveListener(*(pOwner->m_pDeviceLostHandler));
            AirSpace::ShutdownScene();
            pOwner->m_bSceneInitialized = FALSE;
        }

        if( pOwner->m_pDeviceLostHandler != NULL )
        {
            pOwner->m_pDeviceLostHandler->Release();
            pOwner->m_pDeviceLostHandler = NULL;
        }

        delete pOwner;
        return S_OK;
    }

    static HRESULT _stdcall GetDimensions(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __out_ecount(1) DWORD                *pdwWidth,
        __out_ecount(1) DWORD                *pdwHeight
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);

        if( pOwner->m_pFUTexture == NULL ) return E_UNEXPECTED;

        (*pdwWidth)  = pOwner->m_pFUTexture->GetWidth();
        (*pdwHeight) = pOwner->m_pFUTexture->GetHeight();
        return S_OK;
    }

    static BOOL _stdcall HasTexture(
        __in  struct _MMVRAirSpaceInterface  *pThis
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);
        return( pOwner->m_pFUTexture != NULL );
    }

    static BOOL _stdcall GetCompositorAdapterLuid(
        __out_ecount(dwLuidSize) BYTE        *pbLuid,
        __in DWORD                            dwLuidSize
    )
    {
        if( dwLuidSize != sizeof(LUID) ) return FALSE;
        AirSpace::GetScene()->GetSceneWin32().GetCompositorAdapterLuid( *((LUID *)pbLuid) );
        return TRUE;
    }

    static BOOL _stdcall BeginDrawD3D9(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in HMMVRAIRSPACED3DDEVICE           hD3dDevice,
        __in DWORD                            dwMsToWait,
        __in void                           **ppTexture
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);
        return pOwner->m_pFUTexture->BeginDrawD3D9(
                *((IDirect3DDevice9Ex*)hD3dDevice),
                dwMsToWait,
                (IDirect3DTexture9**)ppTexture
            );
    }

    static BOOL _stdcall BeginDrawD3D11(
        __in  struct _MMVRAirSpaceInterface  *pThis,
        __in HMMVRAIRSPACED3DDEVICE           hD3dDevice,
        __in DWORD                            dwMsToWait,
        __in void                           **ppTexture
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);
        return pOwner->m_pFUTexture->BeginDrawD3D11(
                *((ID3D11Device*)hD3dDevice),
                dwMsToWait,
                (ID3D11Texture2D**)ppTexture
            );
    }


    static void _stdcall EndDraw(
        __in  struct _MMVRAirSpaceInterface  *pThis
    )
    {
        Interface *pOwner = (Interface *)(pThis->pContext);
        pOwner->m_pFUTexture->EndDraw();
    }

    static void _stdcall ProcessSceneNotifications(
    )
    {
        AirSpace::GetScene()->ProcessNotifications();
    }



private:
    BOOL                         m_bSceneInitialized;
    AirSpace::ISwapChainTexture *m_pFUTexture;
    MMVRAirSpaceInterface        m_cInterface;
    DeviceLostHandler           *m_pDeviceLostHandler;

};

}
#endif
#endif






#ifdef __cplusplus
extern "C"  {
#endif


enum MMVRAirSpaceEvent_e
{
    EMMVRAirSpaceInitDone,            // InitResult accompanies this event
    EMMVRAirSpaceDeviceLost,          // no data accompanies this event
    EMMVRAirSpaceTextureSizeChanged,  // TextureInfo accompanies this event
    EMMVRRenderImageSizeChanged,      // RenderImageInfo accompanies this event
    EMMVRSmartCropInfoChanged         // SmartCrop Info accompanies this event
};

typedef union MMVRAirSpaceEventDataTag
{
    struct
    {
        HRESULT hr;
    } InitResult;
    struct
    {
        DWORD dwNewWidth;
        DWORD dwNewHeight;
        BOOL  bFirstFrame; // whether or not this is the very first frame randered
    } TextureInfo;
    struct
    {
        DWORD dwNewWidth;
        DWORD dwNewHeight;
        BOOL  bFirstFrame; // whether or not this is the very first frame randered
    } RenderImageInfo;
    struct
    {
        DWORD dwLeft;
        DWORD dwTop;
        DWORD dwRight;
        DWORD dwBottom;
        BOOL  bValid;
    } SmartCropInfo;
} MMVRAirSpaceEventData;




// -----------------------------------------------------------------------------
// PMMVRAIRSPACERENDERERCALLBACK function prototype
//
// Video Platform event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(_stdcall *PMMVRAIRSPACERENDERERCALLBACK)(
    __in            HMMVRAIRSPACERENDERER        hRenderer,
    __in            MMVRAirSpaceEvent_e          eEventType, // what happened?
    __in            const MMVRAirSpaceEventData *eventData,  // optional information about what happened
    __in            void* pvContext
);






// ----------------------------------------------------------------------------
// MMVRVideoCreateAirSpaceRenderer
//
// Create and retrieve a handle to an AirSpace Renderer
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoCreateAirSpaceRenderer(
    __in  MMVRAIRSPACETEXTUREHANDLE      hFastUpdateTexture,
    __in  HWND                           hWnd,
    __in  PMMVRAIRSPACERENDERERCALLBACK  pUICallback,
    __in void *                          pvUICallbackContext,
    __in  MMVRAirSpaceInterface         *pAirSpaceInterface,
    __out HMMVRAIRSPACERENDERER         *phVideoRenderer
);





// ----------------------------------------------------------------------------
// MMVRAirSpaceRendererNotifyUIDeviceLost
//
// U.I. Thread must call this function to notify the MMVR AirSpace renderer taht
//      it lost its AirSpace D3D Device
// ----------------------------------------------------------------------------
HRESULT
MMVRAirSpaceRendererNotifyUIDeviceLost(
    __in    HMMVRAIRSPACERENDERER phVideoRenderer
);






// ----------------------------------------------------------------------------
// MMVRVideoDestroyAirSpaceRenderer
//
// Destroy an AirSpace Renderer
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoDestroyAirSpaceRenderer(
    __in    HMMVRAIRSPACERENDERER phVideoRenderer
);




// ----------------------------------------------------------------------------
// MMVRVideoDestroyAirSpaceRendererContext
//
// Retrieve the rendering context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoGetAirSpaceRendererContext(
    __in    HMMVRAIRSPACERENDERER hVideoRenderer,
    __out   const void **ppvRenderContext
);



void
MMVRAirSpaceEnableTrace(
    __in BOOL bEnabled,
    __in const char *pszQualifier
    );






HRESULT
MMVRAirSpaceSetWindow(
    __in    HMMVRAIRSPACERENDERER hVideoRenderer,
    __in HWND hwndNewWindow
    );



HRESULT
MMVRAirSpaceSetImageProcessingOptions(
    __in HMMVRAIRSPACERENDERER hVideoRenderer,
    __in const MMVR_ImageProcessingOption * pOptions
    );



HRESULT
MMVRAirSpaceComputeClippingRectangles(
    __in  WORD usWindowWidth,
    __in  WORD usWindowHeight,
    __in  WORD usFrameWidth,
    __in  WORD usFrameHeight,
    __in  WORD usOrgFrameWidth,
    __in  WORD usOrgFrameHeight,
    __in  WORD usCropLeftOffset,
    __in  WORD usCropRightOffset,
    __in  WORD usCropTopOffset,
    __in  WORD usCropBottomOffset,
    __in  const MMVR_ImageProcessingOption* pOptions,
    __out RECT* prSrc,
    __out RECT* prDst
    );

#ifdef __cplusplus
}
#endif


#endif //__MMVR_AIRSPACE_EXTERNAL_API__

