#pragma once
#ifndef __MMVR_CUSTOM_EXTERNAL_API__
#define __MMVR_CUSTOM_EXTERNAL_API__


// TODO: move this declaration somewhere common to all MMVR's
//       for now, keep it within ifdef/endif so that multiple files can define it.
#ifndef __MMVR_FOURCC_DEFINED__
#define __MMVR_FOURCC_DEFINED__
typedef DWORD MMVRFOURCC;
#endif

#include <mmvr_Common.h>







// ============================================================================
// MMVR Companion Supporting Custom rendering (or "software buffer" rendering).
//    This is a rendereless MMVR, and serves the same function as the
//    IRtpVideoRenderTarget from Wave14 
// ============================================================================

// ----------------------------------------------------------------------------
// HANDLES Specific to the Custom API's
// ----------------------------------------------------------------------------

// forward declaration, so this file can be used without including any of our internal headers.
struct MMVRCustomRenderer;



typedef struct MMVRCustomRenderer*  HMMVRCUSTOMRENDERER;



enum MMVRCustomEvent_e
{
    EMMVRCustomFrame           // Frame accompanies this event
};



typedef struct MMVRCustomFrameTag
{
    // Frame data and format
    BYTE                   *pbVideoBits;
    ULONG                   ulDataLength;
    DWORD                   dwWidth;
    DWORD                   dwHeight;
    DWORD                   dwPitch;
    ULONG                   ulFourCC;
    ULONG                   ulBitsPerPixel;
    
    // Frame information
    ULONGLONG               ullTimeStamp;
    BOOL                    bNeedMirroring;
    BOOL                    bVerticalFlip;
    
    // Display frame dimensions, correspond to the window size received through
    // MMVRVideoCustomRendererSetPreferredFormat
    DWORD                   dwDisplayWidth;
    DWORD                   dwDisplayHeight;

    // Flag indicating if smart crop info is detected (or generated from no_info)
    BOOL                    bSmartCropInfo;
    
    // Smart Crop rectangle. This rectangle is relative to [0, 0, dwWidth, dwHeight]
    RECT                    rcSmartCropRect;
    
    // Smart Crop render operation. Two rectangles describe source and destination
    // of an operation such as StretchBlt, according to processing options. 
    //Source is relative to  [0, 0, dwWidth, dwHeight] and destination relative to 
    // [0, 0, dwDisplayWidth, dwDisplayHeight]
    RECT                    rcSmartCropSource;
    RECT                    rcSmartCropDest;

    // Original width and height, should be used to determine aspect ratio 
    // for anamorphic images
    DWORD                   dwOrgWidth;
    DWORD                   dwOrgHeight;

    // Delete function, should be called when callee is done with the object
    HRESULT      (_stdcall *pfnDelete)(
                             struct  MMVRCustomFrameTag *pFrame
                                     );

    // Internal data
    BOOL                   bOwnsVideoBits; // whether or not we should delete the video bits when being deleted
    void                   *_pPalInternalFrame; // user to treat this as opaque
    int                    iPalInternalFrameType; // user to treat this as opaque
} MMVRCustomFrame;




typedef union MMVRCustomEventDataTag
{
    MMVRCustomFrame *pFrame;
} MMVRCustomEventData;




// -----------------------------------------------------------------------------
// PMMVRCUSTOMRENDERERCALLBACK function prototype
//
// Video Platform event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(_stdcall *PMMVRCUSTOMRENDERERCALLBACK)(
    __in            HMMVRCUSTOMRENDERER        hRenderer,
    __in            MMVRCustomEvent_e          eEventType, // what happened?
    __in            const MMVRCustomEventData *eventData,  // optional information about what happened
    __in            void* pvContext
);



#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

// ----------------------------------------------------------------------------
// MMVRVideoCreateCustomRenderer
//
// Create and retrieve a handle to an Custom Renderer
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoCreateCustomRenderer(
    __in  PMMVRCUSTOMRENDERERCALLBACK        pUICallback,
    __in void *                              pvUICallbackContext,
    __in_ecount(dwNColorFormats) MMVRFOURCC *pfccColorFormatsAccepted,
    __in DWORD                               dwNColorFormats,
    __out HMMVRCUSTOMRENDERER                *phVideoRenderer
);




// ----------------------------------------------------------------------------
// MMVRVideoDestroyCustomRenderer
//
// Destroy a Custom Renderer
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoDestroyCustomRenderer(
    __in    HMMVRCUSTOMRENDERER phVideoRenderer
);




// ----------------------------------------------------------------------------
// MMVRVideoDestroyCustomRendererContext
//
// Retrieve the rendering context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoGetCustomRendererContext(
    __in    HMMVRCUSTOMRENDERER hVideoRenderer,
    __out   const void **ppvRenderContext
);



// ----------------------------------------------------------------------------
// MMVRVideoCustomRendererSetPreferredFormat
//
// Sets new preferred dimensions (the size to which the image will be resized
// upon arrival).
// The settings passed here apply to upcoming frames.
// if bForceResize is FALSE, the width and height are disregarded.
//
// NOTE: Can be called from either the callback or from outside.
// 
// ----------------------------------------------------------------------------
HRESULT
MMVRVideoCustomRendererSetPreferredFormat(
    __in    HMMVRCUSTOMRENDERER hVideoRenderer,
    __in    INT32       iDisplayWidth,
    __in    INT32       iDisplayHeight,
    __in    BOOL        bForceResize,    // if TRUE, the MMVR will resize to the preferred width/size
    __in    MMVRFOURCC  fccColorFormat   // if non-zero, MMVR will color-convert to this if possible
);



void
MMVRCustomEnableTrace(
    __in BOOL bEnabled,
    __in const char *pszQualifier
    );


HRESULT
MMVRVideoCustomRendererSetImageProcessingOptions(
    __in    HMMVRCUSTOMRENDERER hVideoRenderer,
    __in const MMVR_ImageProcessingOption * pOptions
    );

#ifdef __cplusplus
}
#endif // __cplusplus



#endif //__MMVR_CUSTOM_EXTERNAL_API__



