#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    mmvr_Splitter.h

Abstract:

    Splitter MMVR renderer outward facing API.

--*/


#ifndef __MMVR_SPLITTER__
#define __MMVR_SPLITTER__


#include <mmvr_Common.h>


#ifdef __cplusplus
extern "C" {
#endif

// =============================================================================
// Enums
// =============================================================================


// ============================================================================
// PAL Companion Supporting classic splitter mmvr
// ============================================================================

// ----------------------------------------------------------------------------
// HANDLES Specific to the splitter API's
// ----------------------------------------------------------------------------

typedef struct MMVRSplitterRenderer * HMEDIAVRSPLITTER;
typedef class SharedStateMMVR * HMEDIAVRSHARED;

// ----------------------------------------------------------------------------
// MediaVRCreateSplitterRenderer
//
// Create and retrieve a handle to an Splitter renderer
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRCreateSplitterRenderer(  
	__out HMEDIAVRSPLITTER * phVideoRenderer
);


// ----------------------------------------------------------------------------
// MediaVRDestroySplitterRenderer
//
// Destroy an Splitter renderer
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRDestroySplitterRenderer(
	__in HMEDIAVRSPLITTER hMMediaVRSplitter
);


// ----------------------------------------------------------------------------
// MediaVRGetSplitterRenderContext
//
// Retrieve the render context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRGetSplitterRenderContext(
	__in HMEDIAVRSPLITTER hMMediaVRSplitter,
    __out const void ** ppVoidContext
);


// ----------------------------------------------------------------------------
// MediaVRAddMMVR
//
// Add mmvr to splitter.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRAddMMVR(
	__in HMEDIAVRSPLITTER hMMediaVRSplitter,
	__in HMEDIAVRSHARED mmvr
);


// ----------------------------------------------------------------------------
// MediaVRRemoveMMVR
//
// Remove mmvr from splitter.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRRemoveMMVR(
	__in HMEDIAVRSPLITTER hMMediaVRSplitter,
	__in HMEDIAVRSHARED mmvr
);


#ifdef __cplusplus
}; // extern "C"
#endif

#endif //__MMVR_SPLITTER__
