#pragma once
/* --

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    MediaVRHWNDRenderer.h

Abstract:

    HWND-based MMVR interface

Specification:

    http://livecommteam/sites/main/media/W15/Media Stack W15 Plan Tracking/W15 PAL Video Extension Design Doc.docm

--*/


#ifndef __MMVR_HWND__
#define __MMVR_HWND__


#include <mmvr_Common.h>


#ifdef __cplusplus
extern "C" {
#endif

// =============================================================================
// Enums
// =============================================================================


// -----------------------------------------------------------------------------
// MediaVREvent_e enumeration
//
// Enumeration of MMVR events used in the event callback function as a parameter
// -----------------------------------------------------------------------------
typedef enum _MediaVREvent_e
{
    MediaVREvent_GenericError = -1,
    
    // Format change notification events
    MediaVREvent_VideoSizeChange,

} MediaVREvent_e;

// ============================================================================
// PAL Companion Supporting classic IVideoWindow form
// ============================================================================

// ----------------------------------------------------------------------------
// HANDLES Specific to the HWND API's
// ----------------------------------------------------------------------------

typedef struct MediaVRHWNDRenderer*  HMEDIAVRHWND;


// -----------------------------------------------------------------------------
// PMEDIAVRCALLBACK function prototype
//
// MMVR event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(__stdcall *PMEDIAVRCALLBACK)(
    __in            HMEDIAVRHWND hMMediaVRHWND,
    __in            MediaVREvent_e eEventType,
    __in            void* pvContext
);


// ----------------------------------------------------------------------------
// MediaVRCreateHWNDRenderer
//
// Create and retrieve a handle to an HWND Renderer
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRCreateHWNDRenderer(
    __in    HWND hWnd,
    __in    PMEDIAVRCALLBACK fnCallback,
    __in    void* pvCallbackContext,
    __in    const MMVR_ImageProcessingOption * pOptions,
    __in    BOOL fEnableD3D,
    __out   HMEDIAVRHWND* phMMediaVRHWND
);


// ----------------------------------------------------------------------------
// MediaVRDestroyHWNDRenderer
//
// Destroy an HWND Renderer
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRDestroyHWNDRenderer(
    __in    HMEDIAVRHWND hMMediaVRHWND
);


// ----------------------------------------------------------------------------
// MediaVRGetHWNDRenderContext
//
// Retrieve the render context to be used in creation of the sink.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRGetHWNDRenderContext(
    __in    HMEDIAVRHWND hMMediaVRHWND,
    __out   const void** ppVoidContext
);


// ----------------------------------------------------------------------------
// MediaVRGetHWNDRenderedVideoSize
//
// Retrieve the rendered video size.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRGetHWNDRenderedVideoSize(
    __in    HMEDIAVRHWND hMMediaVRHWND,
    __out   WORD* pwRenderWidth,
    __out   WORD* pwRenderHeight,
    __out   WORD* pwRenderCropWidth,
    __out   WORD* pwRenderCropHeight
);


// ----------------------------------------------------------------------------
// MediaVRSetHWNDImageProcessingOptions
//
// Set the image processing options.
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRSetHWNDImageProcessingOptions(
    __in HMEDIAVRHWND hMMediaVRHWND,
    __in const MMVR_ImageProcessingOption * pOptions
);


// ----------------------------------------------------------------------------
// MediaVRHWNDPaintToDC
//
// Paint the current frame to a device context
// ----------------------------------------------------------------------------
HRESULT __stdcall
MediaVRHWNDPaintToDC(
    __in HMEDIAVRHWND hMediaVRHWND,
    __in HDC hDC,
    __in int nWidth,
    __in int nHeight
);


#ifdef __cplusplus
}; // extern "C"
#endif

#endif //__MMVR_HWND__
