#pragma once
#ifndef __MMVR_MODERN_H__
#define __MMVR_MODERN_H__



#include <mmvr_Custom.h>

#ifdef __cplusplus
extern "C" {
#endif

// ============================================================================
// MMVR Supporting "Modern" applications via Media Engine
// ============================================================================

// =============================================================================
// Enums
// =============================================================================


// -----------------------------------------------------------------------------
// MediaVREvent_e enumeration
//
// Enumeration of MMVR events used in the event callback function as a parameter
// -----------------------------------------------------------------------------
typedef enum _MediaVRModernEvent_e
{
    MediaVRModernEvent_GenericError = -1,

    // Format change notification events
    MediaVRModernEvent_ContextReady,
    MediaVRModernEvent_CropInfoChanged,
    MediaVRModernEvent_SizeChanged

} MediaVRModernEvent_e;

//This can be expanded to include more event data if needed
typedef union MMVRModernEventDataTag
{
    struct
    {
        float fLeft;
        float fTop;
        float fRight;
        float fBottom;
        uint16_t usFrameWidth;
        uint16_t usFrameHeight;
        uint16_t usCropWidth;
        uint16_t usCropHeight;
    } SmartCropInfo;
    
    struct
    {
        uint16_t usFrameWidth;
        uint16_t usFrameHeight;
    } SizeInfo;
    
} MMVRModernEventData;


// ----------------------------------------------------------------------------
// HANDLES Specific to the Modern API's
// ----------------------------------------------------------------------------

typedef struct ModernRenderer*  HMEDIAVRMODERN;


// -----------------------------------------------------------------------------
// PMEDIAVRCALLBACK function prototype
//
// MMVR event callback
// -----------------------------------------------------------------------------
typedef
HRESULT
(WINAPI *PMEDIAVRMODERNCALLBACK)(
    __in            HMEDIAVRMODERN hVideoRenderer,
    __in            MediaVRModernEvent_e eEventType,
    __in            const MMVRModernEventData *eventData,  // optional information about what happened
    __in            void* pvContext
);


// ----------------------------------------------------------------------------
// MediaVRRegisterSchemeForModern
//
// Register a Scheme to be affiliated with real time video rendering
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRRegisterSchemeForModern(
    __in    WCHAR const * const Scheme
);


// ----------------------------------------------------------------------------
// MediaVRCreateModernRenderer
//
// Create and retrieve a handle to an Modern Renderer
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRCreateModernRenderer(
    __in    WCHAR const * const Uri,
    __in    PMEDIAVRMODERNCALLBACK,
    __in    void* pvCallbackContext,
    __out   HMEDIAVRMODERN* phMMediaVRModern
);


// ----------------------------------------------------------------------------
// MediaVRDestroyModernRenderer
//
// Destroy an Modern Renderer
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRDestroyModernRenderer(
    __in    HMEDIAVRMODERN hMMediaVRModern
);


// ----------------------------------------------------------------------------
// MediaVRGetModernRenderContext
//
// Retrieve the render context to be used in creation of the sink.
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRGetModernRenderContext(
    __in    HMEDIAVRMODERN hMMediaVRHWND,
    __out   const void** ppVoidContext
);

// ----------------------------------------------------------------------------
// MediaVRGetModernVideoSize
//
// Retrieve the size of the video being delivered to the compositor.
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRGetModernVideoSize(
    __in    HMEDIAVRMODERN hMMediaVRHWND,
    __out   uint32_t* pWidth,
    __out   uint32_t* pHeight
);

// ----------------------------------------------------------------------------
// MediaVRGetModernSmartCropInfo
//
// Retrieve the smart crop information for the mmvr instance.
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRGetModernSmartCropInfo(
    __in    HMEDIAVRMODERN hMMediaVRHWND,
    __out   float* pLeft,
    __out   float* pTop,
    __out   float* pRight,
    __out   float* pBottom
);


// ----------------------------------------------------------------------------
// MediaVRRendererCollection
//
// TODO: Move this method to a private header as it needs to only be consumed
// by the scheme handler. The scheme handler needs to retrieve the MMVR's pointer
// this is the method to obtain it.
//
// Method used by the scheme handler to retireve the global collection.
// ----------------------------------------------------------------------------

HRESULT
WINAPI
MediaVRGetModernRendererCollection(
   void ** ppCollection
);


void MediaVRModernEnableTrace(
    __in BOOL bEnabled,
    __in const char *pszQualifier
    );

HRESULT
MediaVRModernSetImageProcessingOptions(
    __in HMEDIAVRMODERN hMMediaVRModern,
    __in const MMVR_ImageProcessingOption * pOptions
    );
    
#ifdef __cplusplus
} // extern "C"
#endif

#endif //__MMVR_MODERN_H__

