#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#include <sal.h>

// All Windows platforms except XBOX360 runs on little endian machines
#define RTCPAL_LITTLE_ENDIAN

typedef struct _STRING {
  unsigned short Length;
  unsigned short MaximumLength;
  char *         Buffer;
} STRING, *PSTRING;

typedef STRING ANSI_STRING, *PANSI_STRING;

#include <winsock2.h>
#include <ws2tcpip.h>
#include <rtcpal_windows.h>
#include <wtypes.h>
#include <wincrypt.h>
// #include <emmintrin.h>
// #include <xmmintrin.h>
#define _mm_prefetch(...) 

// POSIX data types
typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned long uint32_t;
typedef unsigned __int64 uint64_t;
typedef signed char int8_t;
typedef signed short int16_t;
typedef signed long int32_t;
typedef signed __int64 int64_t;

#include <rtcpal_types.h>

typedef uint32_t RTCPAL_ERROR;

#define __drv_aliasesMem

#define NORMAL_PRIORITY_CLASS             0x00000020
#define IDLE_PRIORITY_CLASS               0x00000040
#define HIGH_PRIORITY_CLASS               0x00000080
#define REALTIME_PRIORITY_CLASS           0x00000100
#define BELOW_NORMAL_PRIORITY_CLASS       0x00004000
#define ABOVE_NORMAL_PRIORITY_CLASS       0x00008000

#ifndef ERROR_NOT_CAPABLE
    #define ERROR_NOT_CAPABLE                775L
#endif

#ifndef ERROR_IMPLEMENTATION_LIMIT
    #define ERROR_IMPLEMENTATION_LIMIT  1292L
#endif

#if DBG
    #ifndef DEBUG
        #define DEBUG
    #endif
#endif

#ifndef NDEBUG
    #ifndef DEBUG
        #define DEBUG
    #endif
#endif

typedef LONG    NTSTATUS;

#define RTCPAL_PATH_DELIMITER   '\\'

// Force 8-byte aligned
typedef struct RTCPAL_HMACSHA1KEY {
    union
    {
        HCRYPTKEY capi;
    } hmac;
} RTCPAL_HMACSHA1KEY;

typedef struct RTCPAL_HMACSHA1 {
    union {
        HCRYPTHASH    capi;
    } hmac;
} RTCPAL_HMACSHA1;

typedef struct RTCPAL_MD5_CTX {
    HCRYPTHASH hash;
} RTCPAL_MD5_CTX;

typedef struct RTCPAL_AESKEY_RSA {
    // Dummy structure needed for the function signature.  This should never be used under CE.
    uint32_t    cbNotUsed;
} RTCPAL_AESKEY_RSA;

typedef struct RTCPAL_AESKEY {
    union
    {
        RTCPAL_AESKEY_RSA       rsa;
        HCRYPTKEY               capi;
    } aes;
} RTCPAL_AESKEY;

#define RTCPAL_ASE128_KEY_LENGTH        16

#define YieldProcessor()

#define ASYNC_STATUS_PAL_RESERVED   32

typedef CRITICAL_SECTION RTCPAL_CRITICAL_SECTION;
typedef RTCPAL_CRITICAL_SECTION * PRTCPAL_CRITICAL_SECTION;

// On Windows CE, slim-lock == CRITICAL_SECTION
typedef CRITICAL_SECTION RTCPAL_SLIM_LOCK, * PRTCPAL_SLIM_LOCK;

#define RtcPalInitializeSlimLock(lock)  RtcPalInitializeCriticalSectionAndSpinCount(lock,0)
#define RtcPalDeleteSlimLock(lock)      RtcPalDeleteCriticalSection(lock)
#define RtcPalAcquireSlimLock(lock)     RtcPalEnterCriticalSection(lock)
#define RtcPalReleaseSlimLock(lock)     RtcPalLeaveCriticalSection(lock)
#define RtcPalTryAcquireSlimLock(lock)  RtcPalTryEnterCriticalSection(lock)

extern uint8_t _bittest(long* base, long index);
extern uint8_t _bittestandset(long *base, long index);
extern uint8_t _bittestandreset(long *base, long index);

#define bittestandreset _bittestandreset
#define bittestandset _bittestandset
#define bittest _bittest

#define RTCPAL_MAX_COMPUTERNAME_LENGTH  MAX_COMPUTERNAME_LENGTH

#ifdef __cplusplus
};
#endif

#pragma warning(disable:4366)

