/*++

Copyright (C) Microsoft Corporation. All rights reserved.

Module Name:

    rtcpal_wpp.h

Abstract:

    How does Media WPP fit into the overall tracing

    1) RTCPAL_WPP_INIT_TRACING replaces WPP_INIT_TRACING and RTCPAL_WPP_CLEANUP replaces WPP_CLEANUP
       RTCPAL_WPP_INIT_TRACING takes additional parameters for configuring the default tracing
       flags, used by the in-proc WPP consumer (implemented in the PAL)

Author:

Revision History:

--*/

#pragma once

#if 0
// DO NOT DELETE the following comments. They are configuration scanned by tracewpp.exe
// at build time.
//
// begin_wpp config
//
//     DEFINE_CPLX_TYPE(.*s, WPP_LOGXSTR, const xstr_t&, ItemPString,"s", _xstr_t_, 0, 2);
//     DEFINE_CPLX_TYPE(.*S, WPP_LOGXWCS, const xwcs_t&, ItemPWString,"s", _xwcs_t_, 0, 2);
//
//     WPP_FLAGS(-DLOG_ASTR(len, str)=xstr_t(str, (USHORT)len));
//     WPP_FLAGS(-DLOG_WSTR(len, wcs)=xwcs_t(wcs, (USHORT)len));
//     WPP_FLAGS(-DWPP_CHECK_FOR_NULL_STRING);
//     WPP_FLAGS(-DWPP_RTC_TRACING=1);
//     WPP_FLAGS(-lookfor:RTCPAL_WPP_INIT_TRACING);
//     WPP_FLAGS(-gen:{rtcpal_wpp.tpl}*.tmh);
//
// end_wpp

// begin_wpp config
//
//     FUNC TRACE(FLAVOR, LEVEL, GROUP, FLAGS, MSG, ...);
//     USEPREFIX (TRACE, "%!STDPREFIX![0x%p]", _IF_EXISTS_THIS_GET_TRACE_ID _IF_NOT_EXISTS_THIS_GET_0 );
//
//     FUNC TRACE_COMPID(FLAGS,MSG,...);
//     USEPREFIX (TRACE_COMPID, "%!STDPREFIX![0x%p]", _IF_EXISTS_THIS_GET_TRACE_ID _IF_NOT_EXISTS_THIS_GET_0);
//
//     FUNC CTRACE_COMPID(FLAGS,MSG,...);
//
//     FUNC TRACE_ALWAYS(MSG, ...);
//     USEPREFIX (TRACE_ALWAYS, "%!STDPREFIX![0x%p]", _IF_EXISTS_THIS_GET_TRACE_ID _IF_NOT_EXISTS_THIS_GET_0);
//
//     FUNC BAIL_ON_HR_FAILED_TRACE(BAIL_ON_HR_FAILED_TRACE_EXP,MSG,...);
//     USEPREFIX (BAIL_ON_HR_FAILED_TRACE, "%!STDPREFIX![0x%p] hr=%!hresult!:", _IF_EXISTS_THIS_GET_TRACE_ID _IF_NOT_EXISTS_THIS_GET_0, BAIL_ON_HR_FAILED_TRACE_EXP);
//
//     FUNC BAIL_ON_WIN32_ERROR_TRACE(BAIL_ON_WIN32_ERROR_TRACE_EXP,MSG,...);
//     USEPREFIX (BAIL_ON_WIN32_ERROR_TRACE, "%!STDPREFIX![0x%p] WIN32=%!winerr!:", _IF_EXISTS_THIS_GET_TRACE_ID _IF_NOT_EXISTS_THIS_GET_0, BAIL_ON_WIN32_ERROR_TRACE_EXP);
// end_wpp
#endif

#define WPP_BAIL_ON_HR_FAILED_TRACE_EXP_PRE(EXP) {if (FAILED(EXP)) {
#define WPP_BAIL_ON_HR_FAILED_TRACE_EXP_POST(EXP) ; goto Exit; } }
#define WPP_BAIL_ON_HR_FAILED_TRACE_EXP_ENABLED(EXP) WPP_ENABLED()
#define WPP_BAIL_ON_HR_FAILED_TRACE_EXP_LOGGER(EXP)  WPP_LOGGER()

#define WPP_BAIL_ON_WIN32_ERROR_TRACE_EXP_PRE(EXP) {if (NO_ERROR != (EXP)) {
#define WPP_BAIL_ON_WIN32_ERROR_TRACE_EXP_POST(EXP) ; goto Exit; } }
#define WPP_BAIL_ON_WIN32_ERROR_TRACE_EXP_ENABLED(EXP) WPP_ENABLED()
#define WPP_BAIL_ON_WIN32_ERROR_TRACE_EXP_LOGGER(EXP)  WPP_LOGGER()

// WPP logging
#include <evntrace.h>
#include <evntprov.h>
EXTERN_C
RTCPAL_ERROR
RTCAPI
RtcPalRegisterTraceGuids(
    __in WMIDPREQUEST Callback,
    __in PVOID CallbackContext,
    __in LPCGUID ProviderId,
    __out PTRACEHANDLE RegistrationHandle
    );

#if defined(WINPHONE)
//{ forward declaration
ULONG 
RtcPalEventRegister(
    _In_      LPCGUID ProviderId,
    _In_opt_  PENABLECALLBACK EnableCallback,
    _In_opt_  PVOID CallbackContext,
    _Out_     PREGHANDLE RegHandle
    );
    
ULONG 
RtcPalEventUnregister(
    _In_ REGHANDLE RegHandle
    );
    
ULONG 
RtcPalEventWrite(
    _In_     REGHANDLE RegHandle,
    _In_     PCEVENT_DESCRIPTOR EventDescriptor,
    _In_     ULONG UserDataCount,
    _In_opt_ PEVENT_DATA_DESCRIPTOR UserData
    );
    
ULONG 
RtcPalStartTrace(
    __out   PTRACEHANDLE SessionHandle,
    __in    LPCTSTR SessionName,
    __inout PEVENT_TRACE_PROPERTIES Properties
    );
    
ULONG 
RtcPalEnableTrace(
    _In_ ULONG Enable,
    _In_ ULONG EnableFlag,
    _In_ ULONG EnableLevel,
    _In_ LPCGUID ControlGuid,
    _In_ TRACEHANDLE SessionHandle
    );
    
ULONG 
RtcPalStopTrace(
    _In_  TRACEHANDLE SessionHandle,
    _In_  LPCTSTR SessionName,
    _Out_ PEVENT_TRACE_PROPERTIES Properties
    );

ULONG
RtcPalTraceMessage (
    _In_ TRACEHANDLE LoggerHandle,
    _In_ ULONG MessageFlags,
    _In_ LPCGUID MessageGuid,
    _In_ USHORT MessageNumber,
    ...
);

ULONG
RtcPalControlTraceW (
    _In_ TRACEHANDLE TraceHandle,
    _In_opt_ LPCWSTR InstanceName,
    _Inout_ PEVENT_TRACE_PROPERTIES Properties,
    _In_ ULONG ControlCode
    );

TRACEHANDLE
RtcPalGetTraceLoggerHandle (
    _In_ PVOID Buffer
    );

UCHAR
RTCAPI
RtcPalGetTraceEnableLevel (
    _In_ TRACEHANDLE TraceHandle
    );

ULONG
RtcPalGetTraceEnableFlags (
    _In_ TRACEHANDLE TraceHandle
    );

ULONG
RtcPalRegisterTraceGuidsW2 (
    _In_ WMIDPREQUEST RequestAddress,
    _In_opt_ PVOID RequestContext,
    _In_ LPCGUID ControlGuid,
    _In_ ULONG GuidCount,
    _In_reads_opt_(GuidCount) PTRACE_GUID_REGISTRATION TraceGuidReg,
    _In_opt_ LPCWSTR MofImagePath,
    _In_opt_ LPCWSTR MofResourceName,
    _Out_ PREGHANDLE RegistrationHandle
    );

ULONG
RtcPalUnregisterTraceGuids (
    _In_ REGHANDLE RegistrationHandle
    );

#define EventRegister       RtcPalEventRegister
#define EventUnregister     RtcPalEventUnregister
#define EventWrite          RtcPalEventWrite
#undef StartTrace
#define StartTrace          RtcPalStartTrace
#define StartTraceW         RtcPalStartTrace
#define EnableTrace         RtcPalEnableTrace
#undef StopTrace
#define StopTrace           RtcPalStopTrace
#define StopTraceW          RtcPalStopTrace
#define TraceMessage        RtcPalTraceMessage

#define ControlTraceW           RtcPalControlTraceW
#define GetTraceLoggerHandle    RtcPalGetTraceLoggerHandle
#define GetTraceEnableLevel     RtcPalGetTraceEnableLevel
#define GetTraceEnableFlags     RtcPalGetTraceEnableFlags
#undef RegisterTraceGuids
#define RegisterTraceGuids      RtcPalRegisterTraceGuidsW2
#define RegisterTraceGuidsW     RtcPalRegisterTraceGuidsW2
#define UnregisterTraceGuids    RtcPalUnregisterTraceGuids
//}
#endif

